@@ -34,6 +34,7 @@
 ;;    Simon Kaplan
 ;;    Alan Carroll
 ;; at University of Illinois Urbana-Champaign
+;; Revised for Epoch 4.0/Emacs 18.57 by Chris Love
 ;;------------------------------------------------------------------------
 ;;------------------------------------------------------------------------
 ;; This code that follows is a hook and is dependent on the function 
@@ -64,18 +65,27 @@
 
 (defconst HYPER-DIRECTORY (substitute-in-file-name "$HOME/hyper-man"))
 
-(defvar Info-active-attribute (reserve-attribute)
-  "Attribute for active buttons")
-(defvar Info-inactive-attribute (reserve-attribute)
-  "Attribute for italicized text")
-(defvar Info-underline-attribute (reserve-attribute)
-  "Attribute for underlined text")
+(defvar Info-active-style nil
+  "Style for active buttons"
+)
+
+(defvar Info-inactive-style nil
+  "Style for inactive buttons"
+)
+(defvar Info-underline-style nil
+  "Style for underlined text"
+)
 
 (if (> (number-of-colors) 2)
   (progn
-    (set-attribute-global Info-active-attribute "red" (background))
-    (set-attribute-global Info-inactive-attribute "turquoise" (background))
-    (set-attribute-global Info-underline-attribute "yellow" (background))))
+    (setq Info-active-style (make-style))
+    (set-style-background Info-active-style "red")
+    (setq Info-inactive-style (make-style))
+    (set-style-background Info-inactive-style "turquoise")
+    (setq Info-underline-style (make-style))
+    (set-style-underline Info-underline-style (foreground))
+))
+
 
 (defvar Default-Info-directory nil)
 (defvar In-Hyper-Man)
@@ -90,8 +100,8 @@
 ;; T O O L S
 ;; ---------
 
-(defun add-button-inclusive (start end attribute)
-  (add-button start end attribute
+(defun add-button-inclusive (start end style)
+  (add-button start end style
 	      (buffer-substring start end)))
 
 ;; ----------------------------------
@@ -154,32 +164,32 @@
                      (if (re-search-forward
                           "File:[ \t]*\\([^ ,\t\n]+\\)" nil t)
                          (add-button-inclusive (match-beginning 1)
-                                               (match-end 1) Info-inactive-attribute)
+                                               (match-end 1) Info-inactive-style)
                        nil))
                (goto-char (point-min))
                (setq Node-Button
                      (if (re-search-forward "Node:[ \t]*\\([^,\t\n]+\\)" nil t)
                          (add-button (match-beginning 1)
-                                            (match-end 1) Info-inactive-attribute nil)
+                                            (match-end 1) Info-inactive-style nil)
                        nil))
                (goto-char (point-min))
                (setq Prev-Button
                      (if (re-search-forward
                           "\\(Prev\\|Previous\\):[ \t]*\\([^,\t\n]+\\)" nil t)
                          (add-button (match-beginning 2)
-                                            (match-end 2) Info-active-attribute nil)
+                                            (match-end 2) Info-active-style nil)
                        nil))
                (goto-char (point-min))
                (setq Up-Button
                      (if (re-search-forward "Up:[ \t]*\\([^,\t\n]+\\)" nil t)
                          (add-button-inclusive (match-beginning 1)
-                                               (match-end 1) Info-active-attribute)
+                                               (match-end 1) Info-active-style)
                        nil))
                (goto-char (point-min))
                (setq Next-Button
                      (if (re-search-forward "Next:[ \t]*\\([^,\t\n]+\\)" nil t)
                          (add-button (match-beginning 1)
-                                            (match-end 1) Info-active-attribute nil)
+                                            (match-end 1) Info-active-style nil)
                        nil))
                ;; -------------------------------------------
                ;; Make buttons on menu names; store in a list:
@@ -190,7 +200,7 @@
                    (while (re-search-forward "^\\* \\([^:]*\\)" nil t)
                      (setq Menu-Buttons
                            (cons (add-button-inclusive
-                                  (match-beginning 1) (match-end 1) Info-active-attribute)
+                                  (match-beginning 1) (match-end 1) Info-active-style)
                                  Menu-Buttons))))
                ;; --------------------------------
                ;; Make buttons on cross references:
@@ -200,7 +210,8 @@
                (while (re-search-forward "\\*note[ \t\n]+\\([^:\t]*\\):" nil t)
                  (setq CrossRef-Buttons
                        (cons (add-button-inclusive
-                             (match-beginning 1) (match-end 1) Info-active-attribute)
+                             (match-beginning 1) (match-end 1)
+	      Info-active-style)
                              CrossRef-Buttons)))
                ;; -------------------------------------------------------
                ;; If in Hyper-Man, make buttons on underlined words
@@ -215,7 +226,7 @@
                             (delete-char 2)
                             (forward-char 1))
                           (add-button Underline-Begin
-                                             (point) Info-underline-attribute nil))
+                                             (point) Info-underline-style nil))
                         ;; -------------------------------------------------
                         ;; Get rid of remaining ^H chars - note: you might
                         ;; think that I could combine this with the above
@@ -248,7 +259,7 @@
                                     (progn
                                       (re-search-forward "[^)]+)")
                                       (point))
-                                    Info-active-attribute
+                                    Info-active-style
                                     (progn
                                       (setq slice
                                             (buffer-substring
