@@ -3,20 +3,19 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY.  No author or distributor
-;; accepts responsibility to anyone for the consequences of using it
-;; or for whether it serves any particular purpose or works at all,
-;; unless he says so in writing.  Refer to the GNU Emacs General Public
-;; License for full details.
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 1, or (at your option)
+;; any later version.
 
-;; Everyone is granted permission to copy, modify and redistribute
-;; GNU Emacs, but only under the conditions described in the
-;; GNU Emacs General Public License.   A copy of this license is
-;; supposed to have been given to you along with GNU Emacs so you
-;; can know your rights and responsibilities.  It should be in a
-;; file named COPYING.  Among other things, the copyright notice
-;; and this notice must be preserved on all copies.
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (provide 'info)
 (require 'scr-pool)
@@ -132,7 +131,7 @@ Marker points nowhere if file has no tag table.")
     (setq Info-buffers-list (cons buff Info-buffers-list))
   )
 )
-;;; --------------------------------------------------------------------------
+
 ;; Go to an info node specified as separate filename and nodename.
 ;; no-going-back is non-nil if recovering from an error in this function;
 ;; it says do not attempt further (recursive) error recovery.
@@ -142,7 +141,7 @@ Marker points nowhere if file has no tag table.")
   (if filename
     (let (temp)
       (setq filename (substitute-in-file-name filename))
-      (setq temp 
+      (setq temp
 	(expand-file-name filename
 	  ;; Use Info's default dir
 	  ;; unless the filename starts with `./'.
@@ -151,11 +150,17 @@ Marker points nowhere if file has no tag table.")
       )
       (if (file-exists-p temp)
 	(setq filename temp)
-	(setq temp (expand-file-name (downcase filename) Info-directory))
-	(if (file-exists-p temp)
-	  (setq filename temp)
-	  (error "Info file %s does not exist"
-	    (expand-file-name filename Info-directory)
+	(if (file-exists-p (concat temp ".info"))
+	  (setq filename (concat temp ".info"))
+	  (setq temp (expand-file-name (downcase filename) Info-directory))
+	  (if (file-exists-p temp)
+	    (setq filename temp)
+	    (if (file-exists-p (concat temp ".info"))
+	      (setq filename (concat temp ".info"))
+	      (error "Info file %s does not exist"
+		(expand-file-name filename Info-directory)
+	      )
+	    )
 	  )
 	)
       )
@@ -251,7 +256,7 @@ Marker points nowhere if file has no tag table.")
 	      ;; Is this an indirect file?
 	      (search-backward "\nTag table:\n")
 	      (setq pos (point))
-	      (if 
+	      (if
 		(save-excursion
 		  (forward-line 2)
 		  (looking-at "(Indirect)\n")
@@ -269,8 +274,9 @@ Marker points nowhere if file has no tag table.")
 		    )
 		    (Info-record-buffer (current-buffer))
 		    (erase-buffer)
+		    (setq case-fold-search t)
 		    (insert-buffer-substring buf)
-		    (setq Info-tag-table-marker mark) ;share the marker
+		    (setq Info-tag-table-marker mark)	; share the marker
 		    (set-marker Info-tag-table-marker
 		      (match-end 0)
 		    )
@@ -340,7 +346,6 @@ Marker points nowhere if file has no tag table.")
 	(Info-select-node)
       )
     )
-    
     ;; If we did not finish finding the specified node,
     ;; go back to the previous one.
     (or Info-current-node no-going-back
@@ -363,24 +368,26 @@ Marker points nowhere if file has no tag table.")
     (forward-line 2)
     (catch 'foo
       (while (not (looking-at "\^_"))
-        (let ((beg (point))
-              thisfilepos thisfilename)
-          (search-forward ": ")
-          (setq thisfilename  (buffer-substring beg (- (point) 2)))
-          (setq thisfilepos (read (current-buffer)))
-          (if (> thisfilepos nodepos)
-              (throw 'foo t))
-          (setq lastfilename thisfilename)
-          (setq lastfilepos thisfilepos))))
+	(if (not (eolp))
+	  (let ((beg (point))
+	        thisfilepos thisfilename)
+	    (search-forward ": ")
+	    (setq thisfilename  (buffer-substring beg (- (point) 2)))
+	    (setq thisfilepos (read (current-buffer)))
+	    (if (> thisfilepos nodepos)
+	      (throw 'foo t))
+	    (setq lastfilename thisfilename)
+	    (setq lastfilepos thisfilepos))
+	  (forward-line 1))))
     (set-buffer Info-current-buffer)
     (or (equal Info-current-subfile lastfilename)
-        (let ((buffer-read-only nil))
-          (setq buffer-file-name nil)
-          (widen)
-          (erase-buffer)
-          (insert-file-contents lastfilename)
-          (set-buffer-modified-p nil)
-          (setq Info-current-subfile lastfilename)))
+      (let ((buffer-read-only nil))
+	(setq buffer-file-name nil)
+	(widen)
+	(erase-buffer)
+	(insert-file-contents lastfilename)
+	(set-buffer-modified-p nil)
+	(setq Info-current-subfile lastfilename)))
     (goto-char (point-min))
     (search-forward "\n\^_")
     (+ (- nodepos lastfilepos) (point))))
@@ -388,37 +395,41 @@ Marker points nowhere if file has no tag table.")
 ;; Select the info node that point is in.
 (defun Info-select-node ()
   (save-excursion
-   ;; Find beginning of node.
-   (search-backward "\n\^_")
-   (forward-line 2)
-   ;; Get nodename spelled as it is in the node.
-   (re-search-forward "Node:[ \t]*")
-   (setq Info-current-node
-         (buffer-substring (point)
-                           (progn
-                            (skip-chars-forward "^,\t\n")
-                            (point))))
-   (Info-set-mode-line)
-   ;; Find the end of it, and narrow.
-   (beginning-of-line)
-   (let (active-expression)
-     (narrow-to-region (point)
-                       (if (re-search-forward "\n[\^_\f]" nil t)
-                           (prog1
-                            (1- (point))
-                            (if (looking-at "[\n\^_\f]*execute: ")
-                                (progn
-                                  (goto-char (match-end 0))
-                                  (setq active-expression
-                                        (read (current-buffer))))))
-                         (point-max)))
-     (if Info-enable-active-nodes (eval active-expression)))))
+    ;; Find beginning of node.
+    (search-backward "\n\^_")
+    (forward-line 2)
+    ;; Get nodename spelled as it is in the node.
+    (re-search-forward "Node:[ \t]*")
+    (setq Info-current-node
+      (buffer-substring (point)
+	(progn
+	  (skip-chars-forward "^,\t\n")
+	  (point))))
+    (Info-set-mode-line)
+    ;; Find the end of it, and narrow.
+    (beginning-of-line)
+    (let (active-expression)
+      (narrow-to-region (point)
+	(if (re-search-forward "\n[\^_\f]" nil t)
+	  (prog1
+	    (1- (point))
+	    (if (looking-at "[\n\^_\f]*execute: ")
+	      (progn
+		(goto-char (match-end 0))
+		(setq active-expression
+		  (read (current-buffer))))))
+	  (point-max)))
+      (if Info-enable-active-nodes (eval active-expression)))))
 
 (defun Info-set-mode-line ()
   (setq mode-line-buffer-identification
-        (concat
-         "Info:  " (Info-node-title)))
-)
+    (concat
+      "Info:  ("
+      (if Info-current-file
+	(file-name-nondirectory Info-current-file)
+	"")
+      ")"
+      (or Info-current-node ""))))
 ;;;
 (defun Info-node-title ()
   (concat "("
@@ -437,22 +448,19 @@ Marker points nowhere if file has no tag table.")
 (defun Info-goto-node (nodename)
   "Go to info node named NAME.  Give just NODENAME or (FILENAME)NODENAME."
   (interactive "sGoto node: ")
-  (if Info-disaster
-      t
-    (progn
-      (let (filename)
-	(string-match "\\s *\\((\\s *\\([^\t)]*\\)\\s *)\\s *\\|\\)\\(.*\\)"
-		      nodename)
-	(setq filename (if (= (match-beginning 1) (match-end 1))
-			   ""
-			 (substring nodename (match-beginning 2) (match-end 2)))
-	      nodename (substring nodename (match-beginning 3) (match-end 3)))
-	(let ((trim (string-match "\\s *\\'" filename)))
-	  (if trim (setq filename (substring filename 0 trim))))
-	(let ((trim (string-match "\\s *\\'" nodename)))
-	  (if trim (setq nodename (substring nodename 0 trim))))
-	(Info-find-node (if (equal filename "") nil filename)
-			(if (equal nodename "") "Top" nodename))))))
+  (let (filename)
+    (string-match "\\s *\\((\\s *\\([^\t)]*\\)\\s *)\\s *\\|\\)\\(.*\\)"
+      nodename)
+    (setq filename (if (= (match-beginning 1) (match-end 1))
+	""
+	(substring nodename (match-beginning 2) (match-end 2)))
+      nodename (substring nodename (match-beginning 3) (match-end 3)))
+    (let ((trim (string-match "\\s *\\'" filename)))
+      (if trim (setq filename (substring filename 0 trim))))
+    (let ((trim (string-match "\\s *\\'" nodename)))
+      (if trim (setq nodename (substring nodename 0 trim))))
+    (Info-find-node (if (equal filename "") nil filename)
+      (if (equal nodename "") "Top" nodename))))
 
 (defvar Info-last-search nil
   "Default regexp for Info S command to search for.")
@@ -461,88 +469,88 @@ Marker points nowhere if file has no tag table.")
   "Search for REGEXP, starting from point, and select node it's found in."
   (interactive "sSearch (regexp): ")
   (if (equal regexp "")
-      (setq regexp Info-last-search)
+    (setq regexp Info-last-search)
     (setq Info-last-search regexp))
   (let ((found ()) current
-        (onode Info-current-node)
-        (ofile Info-current-file)
-        (opoint (point))
-        (osubfile Info-current-subfile))
+      (onode Info-current-node)
+      (ofile Info-current-file)
+      (opoint (point))
+      (osubfile Info-current-subfile))
     (save-excursion
       (save-restriction
-        (widen)
-        (if (null Info-current-subfile)
-            (progn (re-search-forward regexp) (setq found (point)))
-          (condition-case err
-              (progn (re-search-forward regexp) (setq found (point)))
-            (search-failed nil)))))
+	(widen)
+	(if (null Info-current-subfile)
+	  (progn (re-search-forward regexp) (setq found (point)))
+	  (condition-case err
+	    (progn (re-search-forward regexp) (setq found (point)))
+	    (search-failed nil)))))
     (if (not found) ;can only happen in subfile case -- else would have erred
-        (unwind-protect
-            (let ((list ()))
-              (set-buffer (marker-buffer Info-tag-table-marker))
-              (goto-char (point-min))
-              (search-forward "\n\^_\nIndirect:")
-              (save-restriction
-                (narrow-to-region (point)
-                                  (progn (search-forward "\n\^_")
-                                         (1- (point))))
-                (goto-char (point-min))
-                (search-forward (concat "\n" osubfile ": "))
-                (beginning-of-line)
-                (while (not (eobp))
-                  (re-search-forward "\\(^.*\\): [0-9]+$")
-                  (goto-char (+ (match-end 1) 2))
-                  (setq list (cons (cons (read (current-buffer))
-                                         (buffer-substring (match-beginning 1)
-                                                           (match-end 1)))
-                                   list))
-                  (goto-char (1+ (match-end 0))))
-                (setq list (nreverse list)
-                      current (car (car list))
-                      list (cdr list)))
-              (while list
-                (message "Searching subfile %s..." (cdr (car list)))
-                (Info-read-subfile (car (car list)))
-                (setq list (cdr list))
-                (goto-char (point-min))
-                (if (re-search-forward regexp nil t)
-                    (setq found (point) list ())))
-              (if found
-                  (message "")
-                (signal 'search-failed (list regexp))))
-          (if (not found)
-              (progn (Info-read-subfile opoint)
-                     (goto-char opoint)
-                     (Info-select-node)))))
+      (unwind-protect
+	(let ((list ()))
+	  (set-buffer (marker-buffer Info-tag-table-marker))
+	  (goto-char (point-min))
+	  (search-forward "\n\^_\nIndirect:")
+	  (save-restriction
+	    (narrow-to-region (point)
+	      (progn (search-forward "\n\^_")
+		(1- (point))))
+	    (goto-char (point-min))
+	    (search-forward (concat "\n" osubfile ": "))
+	    (beginning-of-line)
+	    (while (not (eobp))
+	      (re-search-forward "\\(^.*\\): [0-9]+$")
+	      (goto-char (+ (match-end 1) 2))
+	      (setq list (cons (cons (read (current-buffer))
+		    (buffer-substring (match-beginning 1)
+		      (match-end 1)))
+		  list))
+	      (goto-char (1+ (match-end 0))))
+	    (setq list (nreverse list)
+	      current (car (car list))
+	      list (cdr list)))
+	  (while list
+	    (message "Searching subfile %s..." (cdr (car list)))
+	    (Info-read-subfile (car (car list)))
+	    (setq list (cdr list))
+	    (goto-char (point-min))
+	    (if (re-search-forward regexp nil t)
+	      (setq found (point) list ())))
+	  (if found
+	    (message "")
+	    (signal 'search-failed (list regexp))))
+	(if (not found)
+	  (progn (Info-read-subfile opoint)
+	    (goto-char opoint)
+	    (Info-select-node)))))
     (widen)
     (goto-char found)
     (Info-select-node)
     (or (and (equal onode Info-current-node)
-             (equal ofile Info-current-file))
-        (setq Info-history (cons (list ofile onode opoint)
-                                 Info-history)))))
+	(equal ofile Info-current-file))
+      (setq Info-history (cons (list ofile onode opoint)
+	  Info-history)))))
 
 (defun Info-extract-pointer (name &optional errorname)
   (save-excursion
-   (goto-char (point-min))
-   (forward-line 1)
-   (if (re-search-backward (concat name ":") nil t)
-       nil
-     (error (concat "Node has no " (capitalize (or errorname name)))))
-   (goto-char (match-end 0))
-   (Info-following-node-name)))
+    (goto-char (point-min))
+    (forward-line 1)
+    (if (re-search-backward (concat name ":") nil t)
+      nil
+      (error (concat "Node has no " (capitalize (or errorname name)))))
+    (goto-char (match-end 0))
+    (Info-following-node-name)))
 
 (defun Info-following-node-name (&optional allowedchars)
   (skip-chars-forward " \t")
   (buffer-substring
-   (point)
-   (progn
-     (while (looking-at (concat "[" (or allowedchars "^,\t\n") "]"))
-       (skip-chars-forward (concat (or allowedchars "^,\t\n") "("))
-       (if (looking-at "(")
-           (skip-chars-forward "^)")))
-     (skip-chars-backward " ")
-     (point))))
+    (point)
+    (progn
+      (while (looking-at (concat "[" (or allowedchars "^,\t\n") "]"))
+	(skip-chars-forward (concat (or allowedchars "^,\t\n") "("))
+	(if (looking-at "(")
+	  (skip-chars-forward "^)")))
+      (skip-chars-backward " ")
+      (point))))
 
 (defun Info-next ()
   "Go to the next node of this node."
@@ -563,7 +571,7 @@ Marker points nowhere if file has no tag table.")
   "Go back to the last node visited."
   (interactive)
   (or Info-history
-      (error "This is the first Info node you looked at"))
+    (error "This is the first Info node you looked at"))
   (let (filename nodename opoint)
     (setq filename (car (car Info-history)))
     (setq nodename (car (cdr (car Info-history))))
@@ -582,25 +590,25 @@ Marker points nowhere if file has no tag table.")
   "Follow cross reference named NAME to the node it refers to.
 NAME may be an abbreviation of the reference name."
   (interactive
-   (let ((completion-ignore-case t)
-         completions str i)
-     (save-excursion
-       (goto-char (point-min))
-       (while (re-search-forward "\\*note[ \n]*[^:]*:" nil t)
-         (setq str (buffer-substring
-                    (+ (match-beginning 0) 6)
-                    (1- (point))))
-         (setq i 0)
-         (while (setq i (string-match "[ \n\t]+" str i))
-           (setq str (concat (substring str 0 i) " "
-                             (substring str (match-end 0))))
-           (setq i (1+ i)))
-         (setq completions
-               (cons (cons str nil)
-                     completions))))
-     (if completions
-         (list (completing-read "Follow reference named: " completions nil t))
-       (error "No cross-references in this node"))))
+    (let ((completion-ignore-case t)
+	completions str i)
+      (save-excursion
+	(goto-char (point-min))
+	(while (re-search-forward "\\*note[ \n\t]*\\([^:]*\\):" nil t)
+	  (setq str (buffer-substring
+	      (match-beginning 1)
+	      (1- (point))))
+	  (setq i 0)
+	  (while (setq i (string-match "[ \n\t]+" str i))
+	    (setq str (concat (substring str 0 i) " "
+		(substring str (match-end 0))))
+	    (setq i (1+ i)))
+	  (setq completions
+	    (cons (cons str nil)
+	      completions))))
+      (if completions
+	(list (completing-read "Follow reference named: " completions nil t))
+	(error "No cross-references in this node"))))
   (let (target beg i (str (concat "\\*note " footnotename)))
     (while (setq i (string-match " " str i))
       (setq str (concat (substring str 0 i) "[ \t\n]+" (substring str (1+ i))))
@@ -608,25 +616,27 @@ NAME may be an abbreviation of the reference name."
     (save-excursion
       (goto-char (point-min))
       (or (re-search-forward str nil t)
-          (error "No cross-reference named %s" footnotename))
+	(error "No cross-reference named %s" footnotename))
       (goto-char (+ (match-beginning 0) 5))
-      (setq target (Info-extract-menu-node-name "Bad format cross reference")))
+      (setq target
+	(Info-extract-menu-node-name "Bad format cross reference" t)))
     (while (setq i (string-match "[ \t\n]+" target i))
       (setq target (concat (substring target 0 i) " "
-                           (substring target (match-end 0))))
+	  (substring target (match-end 0))))
       (setq i (+ i 1)))
     (Info-goto-node target)))
 
-(defun Info-extract-menu-node-name (&optional errmessage)
+(defun Info-extract-menu-node-name (&optional errmessage multi-line)
   (skip-chars-forward " \t\n")
   (let ((beg (point))
-        str i)
+      str i)
     (skip-chars-forward "^:")
     (forward-char 1)
     (setq str
-          (if (looking-at ":")
-              (buffer-substring beg (1- (point)))
-            (Info-following-node-name "^.,\t\n")))
+      (if (looking-at ":")
+	(buffer-substring beg (1- (point)))
+	(skip-chars-forward " \t\n")
+	(Info-following-node-name (if multi-line "^.,\t"  "^.,\t\n"))))
     (while (setq i (string-match "\n" str i))
       (aset str i ?\ ))
     str))
@@ -639,57 +649,58 @@ NAME may be an abbreviation of the reference name."
 (defun Info-menu (menu-item)
   "Go to node for menu item named (or abbreviated) NAME."
   (interactive
-   (let ((completions '())
-         ;; If point is within a menu item, use that item as the default
-         (default nil)
-         (p (point))
-         (last nil))
-     (save-excursion
-       (goto-char (point-min))
-       (if (not (search-forward "\n* menu:" nil t))
-           (error "No menu in this node"))
-       (while (re-search-forward
-                "\n\\* \\([^:\t\n]*\\):" nil t)
-         (if (and (null default)
-                  (prog1 (if last (< last p) nil)
-                    (setq last (match-beginning 0)))
-                  (<= p last))
-             (setq default (car (car completions))))
-         (setq completions (cons (cons (buffer-substring
-                                         (match-beginning 1)
-                                         (match-end 1))
-                                       (match-beginning 1))
-                                 completions)))
-       (if (and (null default) last
-                (< last p)
-                (<= p (progn (end-of-line) (point))))
-           (setq default (car (car completions)))))
-     (let ((item nil))
-       (while (null item)
-         (setq item (let ((completion-ignore-case t))
-                      (completing-read (if default
-                                           (format "Menu item (default %s): "
-                                                   default)
-                                           "Menu item: ")
-                                       completions nil t)))
-         ;; we rely on the bug (which RMS won't change for his own reasons)
-         ;; that ;; completing-read accepts an input of "" even when the
-         ;; require-match argument is true and "" is not a valid possibility
-         (if (string= item "")
-             (if default
-                 (setq item default)
-                 ;; ask again
-                 (setq item nil))))
-       (list item))))
+    (let ((completions '())
+	 ;; If point is within a menu item, use that item as the default
+	(default nil)
+	(p (point))
+	(last nil))
+      (save-excursion
+	(goto-char (point-min))
+	(if (not (search-forward "\n* menu:" nil t))
+	  (error "No menu in this node"))
+	(while (re-search-forward
+	    "\n\\* \\([^:\t\n]*\\):" nil t)
+	  (if (and (null default)
+	      (prog1 (if last (< last p) nil)
+		(setq last (match-beginning 0)))
+	      (<= p last))
+	    (setq default (car (car completions))))
+	  (setq completions (cons (cons (buffer-substring
+		  (match-beginning 1)
+		  (match-end 1))
+		(match-beginning 1))
+	      completions)))
+	(if (and (null default) last
+	    (< last p)
+	    (<= p (progn (end-of-line) (point))))
+	  (setq default (car (car completions)))))
+      (let ((item nil))
+	(while (null item)
+	  (setq item (let ((completion-ignore-case t))
+	      (completing-read (if default
+		  (format "Menu item (default %s): "
+		    default)
+		  "Menu item: ")
+		completions nil t)))
+	 ;; we rely on the bug (which RMS won't change for his own reasons)
+	 ;; that ;; completing-read accepts an input of "" even when the
+	 ;; require-match argument is true and "" is not a valid possibility
+	  (if (string= item "")
+	    (if default
+	      (setq item default)
+	         ;; ask again
+	      (setq item nil))))
+	(list item))))
   (Info-goto-node (Info-extract-menu-item menu-item)))
-  
+
 (defun Info-extract-menu-item (menu-item)
   (save-excursion
     (goto-char (point-min))
     (or (search-forward "\n* menu:" nil t)
-        (error "No menu in this node"))
-    (or (search-forward (concat "\n* " menu-item) nil t)
-        (error "No such item in menu"))
+      (error "No menu in this node"))
+    (or (search-forward (concat "\n* " menu-item ":") nil t)
+      (search-forward (concat "\n* " menu-item) nil t)
+      (error "No such item in menu"))
     (beginning-of-line)
     (forward-char 2)
     (Info-extract-menu-node-name)))
@@ -698,9 +709,9 @@ NAME may be an abbreviation of the reference name."
   (save-excursion
     (goto-char (point-min))
     (or (search-forward "\n* menu:" nil t)
-        (error "No menu in this node"))
+      (error "No menu in this node"))
     (or (search-forward "\n* " nil t count)
-        (error "Too few items in menu"))
+      (error "Too few items in menu"))
     (Info-extract-menu-node-name)))
 
 (defun Info-first-menu-item ()
@@ -731,7 +742,8 @@ NAME may be an abbreviation of the reference name."
 (defun Info-exit ()
   "Exit Info by selecting some other buffer."
   (interactive)
-  (Info-clean-up-for-exit))
+  (Info-clean-up-for-exit)
+)
 
 (defun Info-undefined ()
   "Make command be undefined in Info."
@@ -742,9 +754,9 @@ NAME may be an abbreviation of the reference name."
   "Enter the Info tutorial."
   (interactive)
   (Info-find-node "info"
-                  (if (< (window-height) 23)
-                      "Help-Small-Screen"
-                    "Help")))
+    (if (< (window-height) 23)
+      "Help-Small-Screen"
+      "Help")))
 
 (defun Info-summary ()
   "Display a brief summary of all Info commands."
@@ -756,17 +768,17 @@ NAME may be an abbreviation of the reference name."
     (goto-char (point-min))
     (let (ch flag)
       (while (progn (setq flag (not (pos-visible-in-window-p (point-max))))
-                    (message (if flag "Type Space to see more"
-                               "Type Space to return to Info"))
-                    (if (/= ?\  (setq ch (read-char)))
-                        (progn (setq unread-command-char ch) nil)
-                      flag))
-        (scroll-up)))))
+	  (message (if flag "Type Space to see more"
+	      "Type Space to return to Info"))
+	  (if (/= ?\  (setq ch (read-char)))
+	    (progn (setq unread-command-char ch) nil)
+	    flag))
+	(scroll-up)))))
 
 (defvar Info-mode-map nil
   "Keymap containing Info commands.")
 (if Info-mode-map
-    nil
+  nil
   (setq Info-mode-map (make-keymap))
   (suppress-keymap Info-mode-map)
   (define-key Info-mode-map "." 'beginning-of-buffer)
@@ -788,7 +800,6 @@ NAME may be an abbreviation of the reference name."
   (define-key Info-mode-map "f" 'Info-follow-reference)
   (define-key Info-mode-map "g" 'Info-goto-node)
   (define-key Info-mode-map "h" 'Info-help)
-;  (define-key Info-mode-map "l" 'Info-last)
   (define-key Info-mode-map "l" 'Pre-Info-last)
   (define-key Info-mode-map "m" 'Info-menu)
   (define-key Info-mode-map "n" 'Info-next)
@@ -796,7 +807,6 @@ NAME may be an abbreviation of the reference name."
   (define-key Info-mode-map "q" 'Info-exit)
   (define-key Info-mode-map "s" 'Info-search)
   (define-key Info-mode-map "u" 'Info-up)
-  (define-key Info-mode-map "v" 'verify-tag-markers)
   (define-key Info-mode-map "\177" 'scroll-down))
 
 (defun Info-mode ()
@@ -806,30 +816,30 @@ discusses one topic and contains references to other nodes
 which discuss related topics.  Info has commands to follow
 the references and show you other nodes.
 
-h       Invoke the Info tutorial.
+h	Invoke the Info tutorial.
 
 Selecting other nodes:
-n       Move to the \"next\" node of this node.
-p       Move to the \"previous\" node of this node.
-u       Move \"up\" from this node.
-m       Pick menu item specified by name (or abbreviation).
-        Picking a menu item causes another node to be selected.
-f       Follow a cross reference.  Reads name of reference.
-l       Move to the last node you were at.
+n	Move to the \"next\" node of this node.
+p	Move to the \"previous\" node of this node.
+u	Move \"up\" from this node.
+m	Pick menu item specified by name (or abbreviation).
+	Picking a menu item causes another node to be selected.
+f	Follow a cross reference.  Reads name of reference.
+l	Move to the last node you were at.
 
 Moving within a node:
-Space   scroll forward a page.     DEL  scroll backward.
-b       Go to beginning of node.
+Space	scroll forward a page.     DEL  scroll backward.
+b	Go to beginning of node.
 
 Advanced commands:
-q       Quit Info: reselect previously selected buffer.
-e       Edit contents of selected node.
-1       Pick first item in node's menu.
+q	Quit Info: reselect previously selected buffer.
+e	Edit contents of selected node.
+1	Pick first item in node's menu.
 2, 3, 4, 5   Pick second ... fifth item in node's menu.
-g       Move to node specified by name.
-        You may include a filename as well, as (FILENAME)NODENAME.
-s       Search through this Info file for specified regexp,
-        and select the node in which the next occurrence is found."
+g	Move to node specified by name.
+	You may include a filename as well, as (FILENAME)NODENAME.
+s	Search through this Info file for specified regexp,
+	and select the node in which the next occurrence is found."
   (kill-all-local-variables)
   (setq major-mode 'Info-mode)
   (setq mode-name "Info")
@@ -845,9 +855,8 @@ s       Search through this Info file for specified regexp,
       (set (make-local-variable 'Info-current-subfile) nil)
       (set (make-local-variable 'Info-current-node) nil)
       (set (make-local-variable 'Info-tag-table-marker) (make-marker))
-    )
-  )
-;  (make-local-variable 'Info-history)
+;      (set (make-local-variable 'Info-history) nil)
+  ))
   (Info-set-mode-line))
 
 (defvar Info-edit-map nil
@@ -878,7 +887,7 @@ Allowed only if variable Info-enable-edit is non-nil."
   ;; Make mode line update.
   (set-buffer-modified-p (buffer-modified-p))
   (message (substitute-command-keys
-             "Editing: Type \\[Info-cease-edit] to return to info")))
+	     "Editing: Type \\[Info-cease-edit] to return to info")))
 
 (defun Info-cease-edit ()
   "Finish editing Info node; switch back to Info proper."
@@ -926,3 +935,4 @@ Allowed only if variable Info-enable-edit is non-nil."
     )
   )
 )
+
