@@ -3,20 +3,19 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY.  No author or distributor
-;; accepts responsibility to anyone for the consequences of using it
-;; or for whether it serves any particular purpose or works at all,
-;; unless he says so in writing.  Refer to the GNU Emacs General Public
-;; License for full details.
+;; GNU Emacs is free software; you can redistribute it and/or modify
+;; it under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 1, or (at your option)
+;; any later version.
 
-;; Everyone is granted permission to copy, modify and redistribute
-;; GNU Emacs, but only under the conditions described in the
-;; GNU Emacs General Public License.   A copy of this license is
-;; supposed to have been given to you along with GNU Emacs so you
-;; can know your rights and responsibilities.  It should be in a
-;; file named COPYING.  Among other things, the copyright notice
-;; and this notice must be preserved on all copies.
+;; GNU Emacs is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+;; GNU General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with GNU Emacs; see the file COPYING.  If not, write to
+;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 ;;; Special formatting conventions are used in this file!
@@ -42,17 +41,10 @@
 ;; Know which function the debugger is!
 (setq debugger 'debug)
 
-(if (boundp 'epoch::version)
-    (defconst mode-line-buffer-identification (purecopy '("Epoch: %17b")) "\
-Mode-line control for identifying the buffer being displayed.
-Its default value is \"Epoch: %17b\".  Major modes that edit things
-other than ordinary files may change this (e.g. Info, Dired,...)")
-  (defconst mode-line-buffer-identification (purecopy '("Emacs: %17b")) "\
+(defconst mode-line-buffer-identification (purecopy '("Emacs: %17b")) "\
 Mode-line control for identifying the buffer being displayed.
 Its default value is \"Emacs: %17b\".  Major modes that edit things
 other than ordinary files may change this (e.g. Info, Dired,...)")
-  )
-
 
 (make-variable-buffer-local 'mode-line-buffer-identification)
 
@@ -192,6 +184,7 @@ Visiting a file whose name matches REGEXP causes FUNCTION to be called.")
 				("\\.article$" . text-mode)
 				("\\.letter$" . text-mode)
 				("\\.texinfo$" . texinfo-mode)
+				("\\.texi$" . texinfo-mode)
 				("\\.lsp$" . lisp-mode)
 				("\\.prolog$" . prolog-mode)
 				;; Mailer puts message to be edited in /tmp/Re.... or Message
@@ -204,7 +197,7 @@ Visiting a file whose name matches REGEXP causes FUNCTION to be called.")
 				("\\.scm.[0-9]*$" . scheme-mode)
 				;; .emacs following a directory delimiter
 				;; in either Unix or VMS syntax.
-				("[]>:/]\\..*emacs" . emacs-lisp-mode)
+				("[]>:/]\\..*emacs$" . emacs-lisp-mode)
 				("\\.ml$" . lisp-mode))))
 
 (make-variable-buffer-local 'indent-tabs-mode)
@@ -262,6 +255,26 @@ Each file will be processed even if an error occurred previously.
 For example, invoke \"emacs -batch -f batch-byte-compile $emacs/ ~/*.el\""
   nil)
 
+(autoload 'calendar "cal"
+  "\
+Display three-month calendar in another window.
+The three months appear side by side, with the current month in the middle
+surrounded by the previous and next months.  The cursor is put on today's date.
+
+An optional prefix argument ARG causes the calendar displayed to be
+ARG months in the future if ARG is positive or in the past if ARG is
+negative; in this case the cursor goes on the first day of the month.
+
+The Gregorian calendar is assumed.
+
+After preparing the calendar window, the hooks calendar-hook are run
+when the calendar is for the current month--that is, the was no prefix
+argument.  If the calendar is for a future or past month--that is, there
+was a prefix argument--the hooks offset-calendar-hook are run.  Thus, for
+example, setting calendar-hooks to 'star-date will cause today's date to be
+replaced by asterisks to highlight it in the window."
+  t)
+
 (autoload 'list-command-history "chistory"
   "\
 List history of commands typed to minibuffer.
@@ -410,6 +423,11 @@ If ARG is negative, require -ARG words of continuity.
 Default is 2."
   t)
 
+(autoload 'doctor "doctor"
+  "\
+Switch to *doctor* buffer and start giving psychotherapy."
+  t)
+
 (autoload 'disassemble "disass"
   "\
 Print disassembled code for OBJECT on (optional) STREAM.
@@ -472,6 +490,72 @@ Calls the value of  electric-command-history-hook  if that is non-nil
 The Command History listing is recomputed each time this mode is invoked."
   t)
 
+(autoload 'edt-emulation-on "edt"
+  "\
+Begin emulating DEC's EDT editor.
+Certain keys are rebound; including nearly all keypad keys.
+Use \\[edt-emulation-off] to undo all rebindings except the keypad keys.
+Note that this function does not work if called directly from the .emacs file.
+Instead, the .emacs file should do (setq term-setup-hook 'edt-emulation-on)
+Then this function will be called at the time when it will work."
+  t)
+
+(autoload 'fortran-mode "fortran"
+  "\
+Major mode for editing fortran code.
+Tab indents the current fortran line correctly. 
+`do' statements must not share a common `continue'.
+
+Type `;?' or `;\\[help-command]' to display a list of built-in abbrevs for Fortran keywords.
+
+Variables controlling indentation style and extra features:
+
+ comment-start
+    Normally nil in Fortran mode.  If you want to use comments
+    starting with `!', set this to the string \"!\".
+ fortran-do-indent
+    Extra indentation within do blocks.  (default 3)
+ fortran-if-indent
+    Extra indentation within if blocks.  (default 3)
+ fortran-continuation-indent
+    Extra indentation appled to continuation statements.  (default 5)
+ fortran-comment-line-column
+    Amount of indentation for text within full-line comments. (default 6)
+ fortran-comment-indent-style
+    nil    means don't change indentation of text in full-line comments,
+    fixed  means indent that text at column fortran-comment-line-column
+    relative  means indent at fortran-comment-line-column beyond the
+ 	      indentation for a line of code.
+    Default value is fixed.
+ fortran-comment-indent-char
+    Character to be inserted instead of space for full-line comment
+    indentation.  (default is a space)
+ fortran-minimum-statement-indent
+    Minimum indentation for fortran statements. (default 6)
+ fortran-line-number-indent
+    Maximum indentation for line numbers.  A line number will get
+    less than this much indentation if necessary to avoid reaching
+    column 5.  (default 1)
+ fortran-check-all-num-for-matching-do
+    Non-nil causes all numbered lines to be treated as possible 'continue'
+    statements.  (default nil)
+ fortran-continuation-char
+    character to be inserted in column 5 of a continuation line.
+    (default $)
+ fortran-comment-region
+    String inserted by \\[fortran-comment-region] at start of each line in 
+    region.  (default \"c$$$\")
+ fortran-electric-line-number
+    Non-nil causes line number digits to be moved to the correct column 
+    as typed.  (default t)
+ fortran-startup-message
+    Set to nil to inhibit message first time fortran-mode is used.
+
+Turning on Fortran mode calls the value of the variable fortran-mode-hook 
+with no args, if that value is non-nil.
+\\{fortran-mode-map}"
+  t)
+
 (autoload 'ftp-find-file "ftp"
   "\
 FTP to HOST to get FILE, logging in as USER with password PASSWORD.
@@ -502,6 +586,17 @@ and source-file directory for GDB.  If you wish to change this, use
 the GDB commands `cd DIR' and `directory'."
   t)
 
+(autoload 'set-gosmacs-bindings "gosmacs"
+  "\
+Rebind some keys globally to make GNU Emacs resemble Gosling Emacs.
+Use \\[set-gnu-bindings] to restore previous global bindings."
+  t)
+
+(autoload 'hanoi "hanoi"
+  "\
+Towers of Hanoi diversion.  Argument is number of rings."
+  t)
+
 (autoload 'Helper-help "helper"
   "\
 Provide help for current mode."
@@ -664,6 +759,37 @@ from the usual mail system."
 Send mail using the MH mail system."
   t)
 
+(autoload 'convert-mocklisp-buffer "mlconvert"
+  "\
+Convert buffer of Mocklisp code to real Lisp that GNU Emacs can run."
+  t)
+
+(autoload 'modula-2-mode "modula2"
+  "\
+This is a mode intended to support program development in Modula-2.
+All control constructs of Modula-2 can be reached by typing
+Control-C followed by the first character of the construct.
+\\{m2-mode-map}
+  Control-c b begin         Control-c c case
+  Control-c d definition    Control-c e else
+  Control-c f for           Control-c h header
+  Control-c i if            Control-c m module
+  Control-c l loop          Control-c o or
+  Control-c p procedure     Control-c Control-w with
+  Control-c r record        Control-c s stdio
+  Control-c t type          Control-c u until
+  Control-c v var           Control-c w while
+  Control-c x export        Control-c y import
+  Control-c { begin-comment Control-c } end-comment
+  Control-c Control-z suspend-emacs     Control-c Control-t toggle
+  Control-c Control-c compile           Control-x ` next-error
+  Control-c Control-l link
+
+   m2-indent controls the number of spaces for each indentation.
+   m2-compile-command holds the command to compile a Modula-2 program.
+   m2-link-command holds the command to link a Modula-2 program."
+  t)
+
 (setq disabled-command-hook 'disabled-command-hook)
 
 (autoload 'disabled-command-hook "novice")
@@ -679,6 +805,15 @@ Require special confirmation to execute COMMAND from now on.
 The user's .emacs file is altered so that this will apply
 to future sessions." t)
 
+(autoload 'nroff-mode "nroff-mode"
+  "\
+Major mode for editing text intended for nroff to format.
+\\{nroff-mode-map}
+Turning on Nroff mode runs text-mode-hook, then nroff-mode-hook.
+Also, try nroff-electric-mode, for automatically inserting
+closing requests for requests that are used in matched pairs."
+  t)
+
 (autoload 'list-options "options"
   "\
 Display a list of Emacs user options, with values and documentation."
@@ -795,6 +930,21 @@ they are not defaultly assigned to keys."
 
 (fset 'picture-mode 'edit-picture)
 
+(autoload 'prolog-mode "prolog"
+  "\
+Major mode for editing Prolog code for Prologs.
+Blank lines and `%%...' separate paragraphs.  `%'s start comments.
+Commands:
+\\{prolog-mode-map}
+Entry to this mode calls the value of prolog-mode-hook
+if that value is non-nil."
+  t)
+
+(autoload 'run-prolog "prolog"
+  "\
+Run an inferior Prolog process, input and output via buffer *prolog*."
+  t)
+
 
 (autoload 'clear-rectangle "rect"
   "\
@@ -904,6 +1054,51 @@ It is useful to set this variable in the site customisation file.")
 
 ;;; Others are in paths.el.
 
+(autoload 'run-scheme "xscheme"
+  "\
+Run an inferior Scheme process.
+Output goes to the buffer `*scheme*'.
+With argument, asks for a command line."
+  t)
+
+(autoload 'scheme-mode "scheme"
+  "\
+Major mode for editing Scheme code.
+Editing commands are similar to those of lisp-mode.
+
+In addition, if an inferior Scheme process is running, some additional
+commands will be defined, for evaluating expressions and controlling
+the interpreter, and the state of the process will be displayed in the
+modeline of all Scheme buffers.  The names of commands that interact
+with the Scheme process start with \"xscheme-\".  For more information
+see the documentation for xscheme-interaction-mode.
+
+Commands:
+Delete converts tabs to spaces as it moves back.
+Blank lines separate paragraphs.  Semicolons start comments.
+\\{scheme-mode-map}
+Entry to this mode calls the value of scheme-mode-hook
+if that value is non-nil."
+  t)
+
+(autoload 'scribe-mode "scribe"
+  "\
+Major mode for editing files of Scribe (a text formatter) source.
+Scribe-mode is similar text-mode, with a few extra commands added.
+\\{scribe-mode-map}
+
+Interesting variables:
+
+scribe-fancy-paragraphs
+  Non-nil makes Scribe mode use a different style of paragraph separation.
+
+scribe-electric-quote
+  Non-nil makes insert of double quote use `` or '' depending on context.
+
+scribe-electric-parenthesis
+  Non-nil makes an open-parenthesis char (one of `([<{')
+  automatically insert its close if typed after an @Command form."
+  t)
 
 ;; Useful to set in site-init.el
 (defconst send-mail-function 'sendmail-send-it "\
