@@ -3,21 +3,29 @@
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.3 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/alloc.c,v $
+ * $Date: 92/02/23 11:03:00 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:03:00 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/alloc.c,v $ $Revision: 1.3 $";
+#endif
 
 #include "config.h"
 #include "lisp.h"
@@ -28,12 +36,31 @@ and this notice must be preserved on all copies.  */
 
 #define max(A,B) ((A) > (B) ? (A) : (B))
 
+/* Macro to verify that storage intended for Lisp objects is not
+   out of range to fit in the space for a pointer.
+   ADDRESS is the start of the block, and SIZE
+   is the amount of space within which objects can start.  */
+#define VALIDATE_LISP_STORAGE(address, size)			\
+do								\
+  {								\
+    Lisp_Object val;						\
+    XSET (val, Lisp_Cons, (char *) address + size);		\
+    if ((char *) XCONS (val) != (char *) address + size)	\
+      {								\
+	free (address);						\
+	memory_full ();						\
+      }								\
+  } while (0)
+
 /* Number of bytes of consing done since the last gc */
 int consing_since_gc;
 
 /* Number of bytes of consing since gc before another gc should be done. */
 int gc_cons_threshold;
 
+/* value of consing_since_gc when undos were last truncated.  */
+int consing_at_last_truncate;
+
 /* Nonzero during gc */
 int gc_in_progress;
 
@@ -47,16 +74,35 @@ extern
 #endif /* VIRT_ADDR_VARIES */
  int malloc_sbrk_unused;
 
+/* Two thresholds controlling how much undo information to keep.  */
+int undo_threshold;
+int undo_high_threshold;
+
 /* Non-nil means defun should do purecopy on the function definition */
 Lisp_Object Vpurify_flag;
 
-int pure[PURESIZE / sizeof (int)] = {0,};   /* Force it into data space! */
+/* Argument we give to Fsignal when memory is full.
+   Preallocated since perhaps we can't allocate it when memory is full.  */
+Lisp_Object memory_exhausted_message;
 
+#ifndef HAVE_SHM
+#ifdef VMS
+int pure[PURESIZE / sizeof (int)]; /* no need to initialize - wasted space */
+#else     
+int pure[PURESIZE / sizeof (int)] = {0,};   /* Force it into data space! */
+#endif /* not VMS */     
 #define PUREBEG (char *) pure
+#else
+#define pure PURE_SEG_BITS   /* Use shared memory segment */
+#define PUREBEG (char *)PURE_SEG_BITS
+#endif /* not HAVE_SHM */
 
 /* Index in pure at which next pure object will be allocated. */
 int pureptr;
 
+/* If nonzero, this is a warning delivered by malloc and not yet displayed.  */
+char *pending_malloc_warning;
+
 Lisp_Object
 malloc_warning_1 (str)
      Lisp_Object str;
@@ -71,17 +117,24 @@ malloc_warning_1 (str)
 /* malloc calls this if it finds we are near exhausting storage */
 malloc_warning (str)
      char *str;
+{
+  pending_malloc_warning = str;
+}
+
+display_malloc_warning ()
 {
   register Lisp_Object val;
 
-  val = build_string (str);
+  val = build_string (pending_malloc_warning);
+  pending_malloc_warning = 0;
   internal_with_output_to_temp_buffer (" *Danger*", malloc_warning_1, val);
 }
 
 /* Called if malloc returns zero */
 memory_full ()
 {
-  error ("Memory exhausted");
+  while (1)
+    Fsignal (Qerror, memory_exhausted_message);
 }
 
 /* like malloc and realloc but check for no memory left */
@@ -90,7 +143,11 @@ long *
 xmalloc (size)
      int size;
 {
-  register long *val = (long *) malloc (size);
+  register long *val;
+  /* Avoid failure if malloc (0) returns 0.  */
+  if (size == 0)
+    size = 1;
+  val = (long *) malloc (size);
   if (!val) memory_full ();
   return val;
 }
@@ -100,7 +157,11 @@ xrealloc (block, size)
      long *block;
      int size;
 {
-  register long *val = (long *) realloc ((char *) block, size);
+  register long *val;
+  /* Avoid failure if malloc (0) returns 0.  */
+  if (size == 0)
+    size = 1;
+  val = (long *) realloc (block, size);
   if (!val) memory_full ();
   return val;
 }
@@ -165,9 +226,11 @@ DEFUN ("cons", Fcons, Scons, 2, 2, 0,
 	{
 	  register struct cons_block *new = (struct cons_block *) malloc (sizeof (struct cons_block));
 	  if (!new) memory_full ();
+	  VALIDATE_LISP_STORAGE (new, sizeof *new);
 	  new->next = cons_block;
 	  cons_block = new;
 	  cons_block_index = 0;
+	  XSET (val, Lisp_Cons, &cons_block->conses[CONS_BLOCK_SIZE - 1]);
 	}
       XSET (val, Lisp_Cons, &cons_block->conses[cons_block_index++]);
     }
@@ -234,6 +297,7 @@ DEFUN ("make-vector", Fmake_vector, Smake_vector, 2, 2, 0,
   p = (struct Lisp_Vector *) malloc (sizeof (struct Lisp_Vector) + (sizei - 1) * sizeof (Lisp_Object));
   if (p == 0)
     memory_full ();
+  VALIDATE_LISP_STORAGE (p, 0);
 
   XSET (vector, Lisp_Vector, p);
   consing_since_gc += sizeof (struct Lisp_Vector) + (sizei - 1) * sizeof (Lisp_Object);
@@ -320,6 +384,7 @@ Its value and function definition are void, and its property list is NIL.")
 	{
 	  struct symbol_block *new = (struct symbol_block *) malloc (sizeof (struct symbol_block));
 	  if (!new) memory_full ();
+	  VALIDATE_LISP_STORAGE (new, sizeof *new);
 	  new->next = symbol_block;
 	  symbol_block = new;
 	  symbol_block_index = 0;
@@ -382,6 +447,7 @@ DEFUN ("make-marker", Fmake_marker, Smake_marker, 0, 0, 0,
 	{
 	  struct marker_block *new = (struct marker_block *) malloc (sizeof (struct marker_block));
 	  if (!new) memory_full ();
+	  VALIDATE_LISP_STORAGE (new, sizeof *new);
 	  new->next = marker_block;
 	  marker_block = new;
 	  marker_block_index = 0;
@@ -391,7 +457,6 @@ DEFUN ("make-marker", Fmake_marker, Smake_marker, 0, 0, 0,
   p = XMARKER (val);
   p->buffer = 0;
   p->bufpos = 0;
-  p->modified = 0;
   p->chain = Qnil;
   consing_since_gc += sizeof (struct Lisp_Marker);
   return val;
@@ -514,7 +579,6 @@ make_uninit_string (length)
   register int fullsize = STRING_FULLSIZE (length);
 
   if (length < 0) abort ();
-  if (XTYPE(length) != 0) abort(); /* bug causes bad length value */
 
   if (fullsize <= STRING_BLOCK_SIZE - current_string_block->pos)
     /* This string can fit in the current string block */
@@ -529,6 +593,7 @@ make_uninit_string (length)
       register struct string_block *new
 	= (struct string_block *) malloc (sizeof (struct string_block_head) + fullsize);
       if (!new) memory_full ();
+      VALIDATE_LISP_STORAGE (new, 0);
       consing_since_gc += sizeof (struct string_block_head) + fullsize;
       new->pos = fullsize;
       new->next = large_string_blocks;
@@ -542,6 +607,7 @@ make_uninit_string (length)
       register struct string_block *new
 	= (struct string_block *) malloc (sizeof (struct string_block));
       if (!new) memory_full ();
+      VALIDATE_LISP_STORAGE (new, sizeof *new);
       consing_since_gc += sizeof (struct string_block);
       current_string_block->next = new;
       new->prev = current_string_block;
@@ -731,9 +797,7 @@ you lose
 
 int total_conses, total_markers, total_symbols, total_string_size, total_vector_size;
 int total_free_conses, total_free_markers, total_free_symbols;
-#ifdef CB
 int total_Xresources, total_free_Xresources;
-#endif
 
 static void mark_object (), mark_buffer ();
 static void clear_marks (), gc_sweep ();
@@ -743,8 +807,7 @@ DEFUN ("garbage-collect", Fgarbage_collect, Sgarbage_collect, 0, 0, "",
   "Reclaim storage for Lisp objects no longer needed.\n\
 Returns info on amount of space in use:\n\
  ((USED-CONSES . FREE-CONSES) (USED-SYMS . FREE-SYMS)\n\
-  (USED-MARKERS . FREE-MARKERS) USED-STRING-CHARS USED-VECTOR-SLOTS\n\
-  (USED-RESOURCES . FREE-RESOURCES))\n\
+  (USED-MARKERS . FREE-MARKERS) USED-STRING-CHARS USED-VECTOR-SLOTS)\n\
 Garbage collection happens automatically if you cons more than\n\
 gc-cons-threshold  bytes of Lisp data since previous garbage collection.")
   ()
@@ -755,7 +818,7 @@ gc-cons-threshold  bytes of Lisp data since previous garbage collection.")
   struct handler *handler;
   register struct backtrace *backlist;
   register Lisp_Object tem;
-  char *omessage = minibuf_message;
+  char *omessage = echo_area_contents;
 
   register int i;
 
@@ -766,6 +829,8 @@ gc-cons-threshold  bytes of Lisp data since previous garbage collection.")
   tem = Fnthcdr (make_number (30), Vcommand_history);
   if (CONSP (tem))
     XCONS (tem)->cdr = Qnil;
+  /* Likewise for undo information.  */
+  truncate_all_undos ();
 
   gc_in_progress = 1;
 
@@ -853,6 +918,7 @@ gc-cons-threshold  bytes of Lisp data since previous garbage collection.")
   gc_in_progress = 0;
 
   consing_since_gc = 0;
+  consing_at_last_truncate = 0;
   if (gc_cons_threshold < 10000)
     gc_cons_threshold = 10000;
 
@@ -861,20 +927,28 @@ gc-cons-threshold  bytes of Lisp data since previous garbage collection.")
   else if (!noninteractive)
     message1 ("Garbage collecting...done");
 
-  return Fcons (Fcons (make_number (total_conses),
-		       make_number (total_free_conses)),
-		Fcons (Fcons (make_number (total_symbols),
-			      make_number (total_free_symbols)),
-		       Fcons (Fcons (make_number (total_markers),
-				     make_number (total_free_markers)),
-			      Fcons (make_number (total_string_size),
-				     Fcons (make_number (total_vector_size),
-#ifdef CB
-					    Fcons(Fcons(make_number(total_Xresources),make_number(total_free_Xresources)),
-						  Qnil))))));
-#else
-					    Qnil)))));
-#endif
+  if (inhibit_window_system)
+    return Fcons (Fcons (make_number (total_conses),
+			 make_number (total_free_conses)),
+		  Fcons (Fcons (make_number (total_symbols),
+				make_number (total_free_symbols)),
+			 Fcons (Fcons (make_number (total_markers),
+				       make_number (total_free_markers)),
+				Fcons (make_number (total_string_size),
+				       Fcons (make_number (total_vector_size),
+					      Qnil)))));
+  else
+    return Fcons (Fcons (make_number (total_conses),
+			 make_number (total_free_conses)),
+		  Fcons (Fcons (make_number (total_symbols),
+				make_number (total_free_symbols)),
+			 Fcons (Fcons (make_number (total_markers),
+				       make_number (total_free_markers)),
+				Fcons (make_number (total_string_size),
+				       Fcons (make_number (total_vector_size),
+					      Fcons(Fcons(make_number(total_Xresources),make_number(total_free_Xresources)),
+						    Qnil))))));
+
 }
 
 #if 0
@@ -999,11 +1073,9 @@ mark_object (objptr)
     case Lisp_Window:
     case Lisp_Process:
     case Lisp_Window_Configuration:
-    case Lisp_Root_Block:	/* [amc] */
-#ifdef BUTTON
+    case Lisp_Root_Block:	/* Epoch */
     case Lisp_Button:
     case Lisp_Style:
-#endif
       {
 	register struct Lisp_Vector *ptr = XVECTOR (obj);
 	register int size = ptr->size;
@@ -1016,6 +1088,19 @@ mark_object (objptr)
       }
       break;
 
+#if 0
+    case Lisp_Temp_Vector:
+      {
+	register struct Lisp_Vector *ptr = XVECTOR (obj);
+	register int size = ptr->size;
+	register int i;
+
+	for (i = 0; i < size; i++)     /* and then mark its elements */
+	  mark_object (&ptr->contents[i]);
+      }
+      break;
+#endif 0
+
     case Lisp_Symbol:
       {
 	register struct Lisp_Symbol *ptr = XSYMBOL (obj);
@@ -1071,18 +1156,16 @@ mark_object (objptr)
     case Lisp_Objfwd:
     case Lisp_Buffer_Objfwd:
     case Lisp_Internal_Stream:
-    case Lisp_Raw_Data:		/* [amc] */
+    case Lisp_Raw_Data:		/* Epoch */
     /* Don't bother with Lisp_Buffer_Objfwd,
        since all markable slots in current buffer marked anyway.  */
     /* Don't need to do Lisp_Objfwd, since the places they point
        are protected with staticpro.  */
-
       break;
-#ifdef CB
-  case Lisp_Xresource : /* [amc] */
-      mark_Xresource(obj); break;
-#endif
-
+    case Lisp_Xresource:	/* Epoch */
+      if (!inhibit_window_system)
+	mark_Xresource(obj);
+      break;
     default:
       abort ();
     }
@@ -1102,10 +1185,6 @@ mark_buffer (buf)
   mark_object (&buffer->name);
   XMARK (buffer->name);
 
-  XSET (tem, Lisp_Vector, buffer->syntax_table_v);
-  if (buffer->syntax_table_v)
-    mark_object (&tem);
-
   for (ptr = &buffer->name + 1;
        (char *)ptr < (char *)buffer + sizeof (struct buffer);
        ptr++)
@@ -1200,7 +1279,8 @@ gc_sweep ()
 	      Lisp_Object tem;
 	      tem1 = &mblk->markers[i];  /* tem1 avoids Sun compiler bug */
 	      XSET (tem, Lisp_Marker, tem1);
-	      unchain_marker (tem);
+	      if (tem1->buffer)
+		unchain_marker (tem);
 	      XFASTINT (mblk->markers[i].chain) = (int) marker_free_list;
 	      marker_free_list = &mblk->markers[i];
 	      num_free++;
@@ -1239,7 +1319,7 @@ gc_sweep ()
 	}
   }
 
-#endif /* standalone */
+#endif standalone
 
   /* Free all unmarked vectors */
   {
@@ -1288,9 +1368,8 @@ gc_sweep ()
 	  prev = sb, sb = sb->next;
 	}
   }
-#ifdef CB
-  Xresource_sweep(&total_Xresources,&total_free_Xresources); /* [amc] */
-#endif /* CB */
+  if (!inhibit_window_system)
+    Xresource_sweep(&total_Xresources,&total_free_Xresources);
 }
 
 /* Compactify strings, relocate references to them, and
@@ -1416,6 +1495,21 @@ compact_strings ()
 	from_sb = to_sb;
     }
 }
+
+truncate_all_undos ()
+{
+  register struct buffer *nextb = all_buffers;
+
+  consing_at_last_truncate = consing_since_gc;
+
+  while (nextb)
+    {
+      nextb->undo_list 
+	= truncate_undo_list (nextb->undo_list, undo_threshold,
+			      undo_high_threshold);
+      nextb = nextb->next;
+    }
+}
 
 /* Initialization */
 
@@ -1446,6 +1540,9 @@ init_alloc ()
 void
 syms_of_alloc ()
 {
+  memory_exhausted_message = Fcons (build_string ("Memory exhausted"), Qnil);
+  staticpro (&memory_exhausted_message);
+
   DEFVAR_INT ("gc-cons-threshold", &gc_cons_threshold,
     "*Number of bytes of consing between garbage collections.");
 
@@ -1463,6 +1560,20 @@ syms_of_alloc ()
   DEFVAR_LISP ("purify-flag", &Vpurify_flag,
     "Non-nil means loading Lisp code in order to dump an executable.");
 
+  DEFVAR_INT ("undo-threshold", &undo_threshold,
+    "Keep no more undo information once it exceeds this size.\n\
+This threshold is applied when garbage collection happens.\n\
+The size is counted as the number of bytes occupied,\n\
+which includes both saved text and other data.");
+  undo_threshold = 15000;
+  DEFVAR_INT ("undo-high-threshold", &undo_high_threshold,
+    "Don't keep more than this much size of undo information.\n\
+A command which pushes past this size is itself forgotten.\n\
+This threshold is applied when garbage collection happens.\n\
+The size is counted as the number of bytes occupied,\n\
+which includes both saved text and other data.");
+  undo_high_threshold = 20000;
+
   defsubr (&Scons);
   defsubr (&Slist);
   defsubr (&Svector);
