@@ -1,23 +1,31 @@
 /* Synchronous subprocess invocation for GNU Emacs.
-   Copyright (C) 1985, 1986, 1987, 1988 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1988, 1990 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.5 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/callproc.c,v $
+ * $Author: love $
+ * $Date: 92/02/23 11:01:33 $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:01:33 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/callproc.c,v $ $Revision: 1.5 $";
+#endif
 
 #include <signal.h>
 
@@ -62,6 +70,18 @@ Lisp_Object Vprocess_environment;
    We wait for this to be zero in order to wait for termination.  */
 int synch_process_pid;
 #endif /* BSD4_1 */
+
+/* True iff we are about to fork off a synchronous process or if we
+   are waiting for it.  */
+int synch_process_alive;
+
+/* Nonzero => this is a string explaining death of synchronous subprocess.  */
+char *synch_process_death;
+
+/* Exit code of synchronous subprocess if positive,
+   minus the signal number if negative.  */
+int synch_process_retcode;
+
 
 Lisp_Object
 call_process_cleanup (fdpid)
@@ -76,7 +96,12 @@ call_process_cleanup (fdpid)
 }
 
 #ifdef VMS
+#ifdef __GNUC__
+#define       environ $$PsectAttributes_NOSHR$$environ
+extern char **environ;
+#else
 extern noshare char **environ;
+#endif
 #else
 extern char **environ;
 #endif
@@ -88,8 +113,9 @@ Insert output in BUFFER before point; t means current buffer;\n\
  nil for BUFFER means discard it; 0 means discard and don't wait.\n\
 Fourth arg DISPLAY non-nil means redisplay buffer as output is inserted.\n\
 Remaining arguments are strings passed as command arguments to PROGRAM.\n\
-This function waits for PROGRAM to terminate;\n\
-if you quit, the process is killed.")
+Otherwise waits for PROGRAM to terminate\n\
+and returns a numeric exit status or a signal name as a string.\n\
+If you quit, the process is killed.")
   (nargs, args)
      int nargs;
      register Lisp_Object *args;
@@ -102,7 +128,7 @@ if you quit, the process is killed.")
   int count = specpdl_ptr - specpdl;
   register unsigned char **new_argv
     = (unsigned char **) alloca ((max (2, nargs - 2)) * sizeof (char *));
-  struct buffer *old = bf_cur;
+  struct buffer *old = current_buffer;
 
   CHECK_STRING (args[0], 0);
 
@@ -113,7 +139,7 @@ if you quit, the process is killed.")
     args[1] = build_string ("/dev/null");
 #endif /* not VMS */
   else
-    args[1] = Fexpand_file_name (args[1], bf_cur->directory);
+    args[1] = Fexpand_file_name (args[1], current_buffer->directory);
 
   CHECK_STRING (args[1], 1);
 
@@ -173,6 +199,9 @@ if you quit, the process is killed.")
 #endif
     }
 
+  synch_process_death = 0;
+  synch_process_retcode = 0;
+
   {
     /* child_setup must clobber environ in systems with true vfork.
        Protect it from permanent change.  */
@@ -221,7 +250,7 @@ if you quit, the process is killed.")
     {
 #ifndef subprocesses
       wait_without_blocking ();
-#endif /* subprocesses */
+#endif subprocesses
       return Qnil;
     }
 
@@ -242,9 +271,9 @@ if you quit, the process is killed.")
       {
 	immediate_quit = 0;
 	if (!NULL (buffer))
-	  InsCStr (buf, nread);
-	if (!NULL (display) && INTERACTIVE)
-	  DoDsp (1);
+	  insert (buf, nread);
+	if (!NULL (display) && FROM_KBD)
+	  redisplay_preserve_echo_area ();
 	immediate_quit = 1;
 	QUIT;
       }
@@ -255,18 +284,22 @@ if you quit, the process is killed.")
 
   immediate_quit = 0;
 
-  SetBfp (old);
+  set_buffer_internal (old);
 
   unbind_to (count);
 
-  return Qnil;
+  if (synch_process_death)
+    return build_string (synch_process_death);
+  return make_number (synch_process_retcode);
+
 }
 
 DEFUN ("call-process-region", Fcall_process_region, Scall_process_region,
   3, MANY, 0,
   "Send text from START to END to a process running PROGRAM.\n\
 Delete the text if DELETE is non-nil.\n\
-Put output in BUFFER, before point.  nil => discard it, t => current buffer.\n\
+Insert output in BUFFER before point; t means current buffer;\n\
+nil for BUFFER means discard it; 0 means discard and don't wait.\n\
 Sixth arg DISPLAY non-nil means redisplay buffer as output is inserted.\n\
 Remaining args are passed to PROGRAM at startup as command args.\n\
 This function normally waits for the process to terminate;\n\
@@ -327,14 +360,14 @@ child_setup (in, out, err, new_argv, env)
      the superior's static variables as if the superior had done alloca
      and will be cleaned up in the usual way.  */
 
-  if (XTYPE (bf_cur->directory) == Lisp_String)
+  if (XTYPE (current_buffer->directory) == Lisp_String)
     {
       register unsigned char *temp;
       register int i;
 
-      i = XSTRING (bf_cur->directory)->size;
+      i = XSTRING (current_buffer->directory)->size;
       temp = (unsigned char *) alloca (i + 2);
-      bcopy (XSTRING (bf_cur->directory)->data, temp, i);
+      bcopy (XSTRING (current_buffer->directory)->data, temp, i);
       if (temp[i - 1] != '/') temp[i++] = '/';
       temp[i] = 0;
       chdir (temp);
@@ -389,7 +422,7 @@ child_setup (in, out, err, new_argv, env)
 
 #ifdef vipc
   something missing here;
-#endif /* vipc */
+#endif vipc
 
   /* execvp does not accept an environment arg so the only way
      to pass this environment is to set environ.  Our caller
@@ -409,9 +442,8 @@ init_callproc ()
   register char **envp;
   Lisp_Object execdir;
 
-  /* Turn PATH_EXEC into a path.  `==' is just a string which we know
-     will not be the name of an environment variable.  */
-  Vexec_path = decode_env_path ("==", PATH_EXEC);
+  /* Turn PATH_EXEC into a path.  Don't look at environment */
+  Vexec_path = decode_env_path ((char *) 0, PATH_EXEC);
   Vexec_directory = Ffile_name_as_directory (Fcar (Vexec_path));
   Vexec_path = nconc2 (decode_env_path ("PATH", ""), Vexec_path);
 
