@@ -1,32 +1,38 @@
 /* GNU Emacs case conversion functions.
-   Copyright (C) 1985 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1990 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+GNU Emacs is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /*
- * Includes local changes to support Leif.
+ * $Revision: 1.4 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/casefiddle.c,v $
+ * $Date: 91/12/05 13:31:23 $
+ * $Author: love $
  */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 91/12/05 13:31:23 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/casefiddle.c,v $ $Revision: 1.4 $";
+#endif
 
 #include "config.h"
 #include "lisp.h"
 #include "buffer.h"
 #include "commands.h"
 #include "syntax.h"
+#include "button.h"
 
 enum case_action {CASE_UP, CASE_DOWN, CASE_CAPITALIZE, CASE_CAPITALIZE_UP};
 
@@ -108,29 +114,31 @@ casify_region (flag, b, e)
   register int i;
   register int c;
   register int inword = flag == CASE_DOWN;
-  int flag2;
-  
+
   validate_region (&b, &e);
-  modify_region (XFASTINT (b), XFASTINT (e)); /* if modification allowed */
+/* prepare_to_modify_buffer (); */
+  modify_region(XFASTINT(b),XFASTINT(e));
   record_change (XFASTINT (b), XFASTINT (e) - XFASTINT (b));
 
   for (i = XFASTINT (b); i < XFASTINT (e); i++)
     {
-      c = CharAt (i);
+      c = FETCH_CHAR (i);
       if (inword && flag != CASE_CAPITALIZE_UP)
 	c = DOWNCASE (c);
       else if (!UPPERCASEP (c)
 	       && (!inword || flag != CASE_CAPITALIZE_UP))
 	c = UPCASE1 (c);
-      CharAt (i) = c;
+      FETCH_CHAR (i) = c;
       if ((int) flag >= (int) CASE_CAPITALIZE)
 	inword = SYNTAX (c) == Sword;
     }
-#ifdef HAVE_LEIF
-  signal_after_change (XFASTINT (b),
-                       XFASTINT (e),
-                       XFASTINT (e) - XFASTINT (b));
-#endif
+#ifdef DEFINE_CHANGE_FUNCTIONS
+  signal_after_change(XFASTINT(b),
+		      XFASTINT(e),
+		      XFASTINT(e) - XFASTINT(b));
+#endif  
+
+  modify_region (XFASTINT (b), XFASTINT (e));
 }
 
 DEFUN ("upcase-region", Fupcase_region, Supcase_region, 2, 2, "r",
@@ -181,21 +189,21 @@ upcase_initials_region (b, e)
   return Qnil;
 }
 
-void
-operate_on_word (flag, arg)
-     int flag;
+Lisp_Object
+operate_on_word (arg)
      Lisp_Object arg;
 {
-  Lisp_Object beg, end;
+  Lisp_Object end, val;
   int farend;
+
   CHECK_NUMBER (arg, 0);
   farend = scan_words (point, XINT (arg));
   if (!farend)
-    farend = XINT (arg) > 0 ? NumCharacters + 1 : FirstCharacter;
-  XFASTINT (beg) = point < farend ? point : farend;
-  XFASTINT (end) = point > farend ? point : farend;
-  casify_region (flag, beg, end);
-  SetPoint (XFASTINT (end));
+    farend = XINT (arg) > 0 ? ZV : BEGV;
+  end = point > farend ? point : farend;
+  SET_PT (end);
+  XFASTINT (val) = farend;
+  return val;
 }
 
 DEFUN ("upcase-word", Fupcase_word, Supcase_word, 1, 1, "p",
@@ -204,7 +212,9 @@ With negative argument, convert previous words but do not move.")
   (arg)
      Lisp_Object arg;
 {
-  operate_on_word (CASE_UP, arg);
+  Lisp_Object opoint;
+  XFASTINT (opoint) = point;
+  casify_region (CASE_UP, opoint, operate_on_word (arg));
   return Qnil;
 }
 
@@ -214,7 +224,9 @@ With negative argument, convert previous words but do not move.")
   (arg)
      Lisp_Object arg;
 {
-  operate_on_word (CASE_DOWN, arg);
+  Lisp_Object opoint;
+  XFASTINT (opoint) = point;
+  casify_region (CASE_DOWN, opoint, operate_on_word (arg));
   return Qnil;
 }
 
@@ -226,7 +238,9 @@ With negative argument, capitalize previous words but do not move.")
   (arg)
      Lisp_Object arg;
 {
-  operate_on_word (CASE_CAPITALIZE, arg);
+  Lisp_Object opoint;
+  XFASTINT (opoint) = point;
+  casify_region (CASE_CAPITALIZE, opoint, operate_on_word (arg));
   return Qnil;
 }
 
@@ -245,9 +259,9 @@ syms_of_casefiddle ()
 
 keys_of_casefiddle ()
 {
-  defkey (CtlXmap, Ctl('U'), "upcase-region");
-  defkey (CtlXmap, Ctl('L'), "downcase-region");
-  defkey (ESCmap, 'u', "upcase-word");
-  defkey (ESCmap, 'l', "downcase-word");
-  defkey (ESCmap, 'c', "capitalize-word");
+  ndefkey (Vctl_x_map, Ctl('U'), "upcase-region");
+  ndefkey (Vctl_x_map, Ctl('L'), "downcase-region");
+  ndefkey (Vesc_map, 'u', "upcase-word");
+  ndefkey (Vesc_map, 'l', "downcase-word");
+  ndefkey (Vesc_map, 'c', "capitalize-word");
 }
