@@ -3,21 +3,29 @@
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.4 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/data.c,v $
+ * $Date: 92/02/23 11:02:48 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:02:48 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/data.c,v $ $Revision: 1.4 $";
+#endif
 
 #include <signal.h>
 
@@ -28,30 +36,8 @@ and this notice must be preserved on all copies.  */
 #include "buffer.h"
 #endif
 
-#ifdef USG
-#ifndef USG_SIGNAL
-#define USG_SIGNAL		/* reset signal handler on System V */
-#endif
-#endif
+#include "emacssignal.h"
 
-#ifdef POSIX_SIGNAL
-extern sigset_t zero_sigmask;		/* sigmask with all 0's */
-extern struct sigaction zero_sigaction;	/* sigaction with all 0's */
-
-#ifndef signal
-/* Rather than changing all of the signal calls directly, do so with
-   the preprocessor.  This macro does not return anything.  */
-
-#define signal(num, func) \
-  { \
-    struct sigaction sa; \
-    sa = zero_sigaction; \
-    sa.sa_handler = (void (*)()) func; \
-    sigaction (num, &sa, (struct sigaction *)0); \
-  }
-#endif /* signal */
-#endif /* POSIX_SIGNAL */
- 
 Lisp_Object Qnil, Qt, Qquote, Qlambda, Qsubr, Qunbound;
 Lisp_Object Qerror_conditions, Qerror_message, Qtop_level;
 Lisp_Object Qerror, Qquit, Qwrong_type_argument, Qargs_out_of_range;
@@ -507,7 +493,7 @@ do_symval_forwarding (valcontents)
       return *XOBJFWD (valcontents);
 
     case Lisp_Buffer_Objfwd:
-      return *(Lisp_Object *)(XUINT (valcontents) + (char *)bf_cur);
+      return *(Lisp_Object *)(XUINT (valcontents) + (char *)current_buffer);
     }
   return valcontents;
 }
@@ -537,13 +523,13 @@ store_symval_forwarding (sym, valcontents, newval)
       break;
 
     case Lisp_Buffer_Objfwd:
-      *(Lisp_Object *)(XUINT (valcontents) + (char *)bf_cur) = newval;
+      *(Lisp_Object *)(XUINT (valcontents) + (char *)current_buffer) = newval;
       break;
 
     default:
       valcontents = XSYMBOL (sym)->value;
-      if (XTYPE (valcontents) == Lisp_Buffer_Local_Value ||
-	  XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
+      if (XTYPE (valcontents) == Lisp_Buffer_Local_Value
+	  || XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
 	XCONS (XSYMBOL (sym)->value)->car = newval;
       else
 	XSYMBOL (sym)->value = newval;
@@ -586,15 +572,15 @@ DEFUN ("symbol-value", Fsymbol_value, Ssymbol_value, 1, 1, 0, "Return SYMBOL's v
  	Note that REALVALUE can be a forwarding pointer. */
 
       tem1 = XCONS (XCONS (valcontents)->cdr)->car;
-      if (NULL (tem1) || bf_cur != XBUFFER (tem1))
+      if (NULL (tem1) || current_buffer != XBUFFER (tem1))
 	{
 	  tem1 = XCONS (XCONS (XCONS (valcontents)->cdr)->cdr)->car;
           Fsetcdr (tem1, do_symval_forwarding (XCONS (valcontents)->car));
-	  tem1 = assq_no_quit (sym, bf_cur->local_var_alist);
+	  tem1 = assq_no_quit (sym, current_buffer->local_var_alist);
 	  if (NULL (tem1))
 	    tem1 = XCONS (XCONS (valcontents)->cdr)->cdr;
 	  XCONS (XCONS (XCONS (valcontents)->cdr)->cdr)->car = tem1;
-	  XSET (XCONS (XCONS (valcontents)->cdr)->car, Lisp_Buffer, bf_cur);
+	  XSET (XCONS (XCONS (valcontents)->cdr)->car, Lisp_Buffer, current_buffer);
 	  store_symval_forwarding (sym, XCONS (valcontents)->car, Fcdr (tem1));
 	}
       valcontents = XCONS (valcontents)->car;
@@ -613,7 +599,7 @@ DEFUN ("symbol-value", Fsymbol_value, Ssymbol_value, 1, 1, 0, "Return SYMBOL's v
       return *XOBJFWD (valcontents);
 
     case Lisp_Buffer_Objfwd:
-      return *(Lisp_Object *)(XUINT (valcontents) + (char *)bf_cur);
+      return *(Lisp_Object *)(XUINT (valcontents) + (char *)current_buffer);
 
     case Lisp_Symbol:
       /* For a symbol, check whether it is 'unbound. */
@@ -651,7 +637,6 @@ for this variable.")
   return Fsymbol_value (sym);
 }
 
-#ifdef CB
 /* Access to variable value in another buffer */
 DEFUN ("symbol-buffer-value",Fsymbol_buffer_value,Ssymbol_buffer_value,2,2,0,
        "Return the value of SYMBOL in BUFFER")
@@ -693,7 +678,7 @@ DEFUN ("symbol-buffer-value",Fsymbol_buffer_value,Ssymbol_buffer_value,2,2,0,
 
   return result;
 }
-#endif
+
 
 DEFUN ("set", Fset, Sset, 2, 2, 0,
   "Set SYMBOL's value to NEWVAL, and return NEWVAL.")
@@ -717,19 +702,19 @@ DEFUN ("set", Fset, Sset, 2, 2, 0,
       register int idx = XUINT (valcontents);
       register int mask = *(int *)(idx + (char *) &buffer_local_flags);
       if (mask > 0)
-	bf_cur->local_var_flags |= mask;
+	current_buffer->local_var_flags |= mask;
     }
 
-  if (XTYPE (valcontents) == Lisp_Buffer_Local_Value ||
-      XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
+  if (XTYPE (valcontents) == Lisp_Buffer_Local_Value
+      || XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
     {
       /* valcontents is a list
         (REALVALUE BUFFER CURRENT-ALIST-ELEMENT . DEFAULT-VALUE)).
 
         CURRENT-ALIST-ELEMENT is a pointer to an element of BUFFER's
 	local_var_alist, that being the element whose car is this variable.
-        Or it can be a pointer to the (CURRENT-ALIST-ELEMENT . DEFAULT-VALUE), if BUFFER
-	does not have an element in its alist for this variable.
+        Or it can be a pointer to the (CURRENT-ALIST-ELEMENT . DEFAULT-VALUE),
+	if BUFFER does not have an element in its alist for this variable.
 
 	If the current buffer is not BUFFER, we store the current REALVALUE value into
 	CURRENT-ALIST-ELEMENT, then find the appropriate alist element for
@@ -738,28 +723,50 @@ DEFUN ("set", Fset, Sset, 2, 2, 0,
 	Note that REALVALUE can be a forwarding pointer. */
 
       current_alist_element = XCONS (XCONS (XCONS (valcontents)->cdr)->cdr)->car;
-      if (bf_cur != ((XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
-		     ? XBUFFER (XCONS (XCONS (valcontents)->cdr)->car)
-		     : XBUFFER (XCONS (current_alist_element)->car)))
+      if (current_buffer
+	  != ((XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
+	      /* If the current buffer is already cached,
+		 we don't need to change that fact. */
+	      ? XBUFFER (XCONS (XCONS (valcontents)->cdr)->car)
+	      /* If setting the variable should make it local,
+		 it's not enough if the current buffer is already cached.
+		 We need to make sure it has a local value for the var. */
+	      : XBUFFER (XCONS (current_alist_element)->car)))
 	{
-          Fsetcdr (current_alist_element, do_symval_forwarding (XCONS (valcontents)->car));
+	  /* "Swap out" the buffer that was previously cached. */
+          Fsetcdr (current_alist_element,
+		   do_symval_forwarding (XCONS (valcontents)->car));
 
-	  tem1 = Fassq (sym, bf_cur->local_var_alist);
+	  /* Encache the current buffer. */
+	  tem1 = Fassq (sym, current_buffer->local_var_alist);
 	  if (NULL (tem1))
-	    /* This buffer sees the default value still.
-	       If type is Lisp_Some_Buffer_Local_Value, set the default value.
-	       If type is Lisp_Buffer_Local_Value, give this buffer a local value
-		and set that.  */
-	    if (XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
-	      tem1 = XCONS (XCONS (valcontents)->cdr)->cdr;
-	    else
-	      {
-		tem1 = Fcons (sym, Fcdr (current_alist_element));
-		bf_cur->local_var_alist = Fcons (tem1, bf_cur->local_var_alist);
-	      }
+	    {
+	      /* This buffer sees the default value still.
+		 If type is Lisp_Some_Buffer_Local_Value, set the default value.
+		 If type is Lisp_Buffer_Local_Value, give this buffer a local value
+		 and set that.  */
+	      if (XTYPE (valcontents) == Lisp_Some_Buffer_Local_Value)
+		tem1 = XCONS (XCONS (valcontents)->cdr)->cdr;
+	      else
+		{
+		  tem1 = Fcons (sym, Fcdr (current_alist_element));
+		  current_buffer->local_var_alist = Fcons (tem1, current_buffer->local_var_alist);
+		}
+	    }
+	  /* Set the CURRENT-ALIST-ELEMENT slot to fit this buffer. */
+	  current_alist_element = tem1;
 	  XCONS (XCONS (XCONS (valcontents)->cdr)->cdr)->car = tem1;
-	  XSET (XCONS (XCONS (valcontents)->cdr)->car, Lisp_Buffer, bf_cur);
+	  /* Record this buffer in the BUFFER slot. */
+	  XSET (XCONS (XCONS (valcontents)->cdr)->car,
+		Lisp_Buffer, current_buffer);
+	  /* Now the CURRENT-ALIST-ELEMENT slot in VALCONTENTS
+	     is correct for the current buffer. */
 	}
+      /* If the current buffer sees the default value,
+	 set the default value in the CURRENT-ALIST-ELEMENT right now.
+	 Don't wait till some other buffer gets cached. */
+      if (XCONS(XCONS(valcontents)->cdr)->cdr == current_alist_element)
+	XCONS(current_alist_element)->cdr = newval;
       valcontents = XCONS (valcontents)->car;
     }
   store_symval_forwarding (sym, valcontents, newval);
@@ -791,8 +798,16 @@ for this variable.")
 #endif
       register int mask = *(int *) (idx + (char *) &buffer_local_flags);
 
+      if (mask < 0)
+	/* -1 means the variable is alwas local in every buffer.
+	   Set the default value, but not any of the local values. */
+	*(Lisp_Object *)(idx + (char *) &buffer_defaults) = value;
+
       if (mask > 0)
 	{
+	  /* Positive means a value that is local in some buffers.
+	     Set the default value, and also the corresponding slot
+	     in buffers where the variable is not local. */
 	  *(Lisp_Object *)(idx + (char *) &buffer_defaults) = value;
 	  for (b = all_buffers; b; b = b->next)
 	    if (!(b->local_var_flags & mask))
@@ -900,12 +915,12 @@ See also `make-variable-buffer-local'.")
       XSETTYPE (XSYMBOL (sym)->value, Lisp_Some_Buffer_Local_Value);
     }
   /* Make sure this buffer has its own value of sym */
-  tem = Fassq (sym, bf_cur->local_var_alist);
+  tem = Fassq (sym, current_buffer->local_var_alist);
   if (NULL (tem))
     {
-      bf_cur->local_var_alist
+      current_buffer->local_var_alist
         = Fcons (Fcons (sym, XCONS (XCONS (XCONS (XSYMBOL (sym)->value)->cdr)->cdr)->cdr),
-		 bf_cur->local_var_alist);
+		 current_buffer->local_var_alist);
 
       /* Make sure symbol does not think it is set up for this buffer;
 	 force it to look once again for this buffer's value */
@@ -914,7 +929,7 @@ See also `make-variable-buffer-local'.")
 	Lisp_Object xs;
     
 	xs = XSYMBOL (sym)->value;
-	if (bf_cur == XBUFFER (XCONS (XCONS (xs)->cdr)->car))
+	if (current_buffer == XBUFFER (XCONS (XCONS (xs)->cdr)->car))
 	  XCONS (XCONS (XSYMBOL (sym)->value)->cdr)->car = Qnil; 
       }
     }
@@ -941,9 +956,9 @@ From now on the default value will apply in this buffer.")
 
       if (mask > 0)
 	{
-	  *(Lisp_Object *)(idx + (char *) bf_cur)
+	  *(Lisp_Object *)(idx + (char *) current_buffer)
 	    = *(Lisp_Object *)(idx + (char *) &buffer_defaults);
-	  bf_cur->local_var_flags &= ~mask;
+	  current_buffer->local_var_flags &= ~mask;
 	}
       return sym;
     }
@@ -954,16 +969,16 @@ From now on the default value will apply in this buffer.")
 
   /* Get rid of this buffer's alist element, if any */
 
-  tem = Fassq (sym, bf_cur->local_var_alist);
+  tem = Fassq (sym, current_buffer->local_var_alist);
   if (!NULL (tem))
-    bf_cur->local_var_alist = Fdelq (tem, bf_cur->local_var_alist);
+    current_buffer->local_var_alist = Fdelq (tem, current_buffer->local_var_alist);
 
   /* Make sure symbol does not think it is set up for this buffer;
      force it to look once again for this buffer's value */
   {
     Lisp_Object sv;
     sv = XSYMBOL (sym)->value;
-    if (bf_cur == XBUFFER (XCONS (XCONS (sv)->cdr)->car))
+    if (current_buffer == XBUFFER (XCONS (XCONS (sv)->cdr)->car))
       XCONS (XCONS (sv)->cdr)->car = Qnil;
   }
 
@@ -1137,16 +1152,11 @@ DEFUN ("int-to-string", Fint_to_string, Sint_to_string, 1, 1, 0,
 }
 
 DEFUN ("string-to-int", Fstring_to_int, Sstring_to_int, 1, 1, 0,
-  "Convert STRING to an integer by parsing it as a decimal number.\n\
-Optional second arg FLAG non-nil means also convert \"yes\" to 1, \"no\" to 0.")
-  (str, flag)
-     register Lisp_Object str, flag;
+  "Convert STRING to an integer by parsing it as a decimal number.")
+  (str)
+     register Lisp_Object str;
 {
   CHECK_STRING (str, 0);
-  if (!NULL (flag) && !strcmp (XSTRING (str)->data, "yes"))
-    return make_number (1);
-  if (!NULL (flag) && !strcmp (XSTRING (str)->data, "no"))
-    return make_number (0);
   return make_number (atoi (XSTRING (str)->data));
 }
   
@@ -1162,7 +1172,7 @@ arith_driver
 {
   register Lisp_Object val;
   register int argnum;
-  register int accum = 0;
+  register int accum;
   register int next;
 
 #ifdef SWITCH_ENUM_BUG
@@ -1588,6 +1598,7 @@ syms_of_data ()
   defsubr (&Sfset);
   defsubr (&Ssetplist);
   defsubr (&Ssymbol_value);
+  defsubr (&Ssymbol_buffer_value);  
   defsubr (&Sset);
   defsubr (&Sdefault_value);
   defsubr (&Sset_default);
@@ -1621,30 +1632,24 @@ syms_of_data ()
   defsubr (&Sadd1);
   defsubr (&Ssub1);
   defsubr (&Slognot);
-
-#ifdef CB
-  defsubr (&Ssymbol_buffer_value);
-#endif
 }
 
 arith_error (signo)
      int signo;
 {
-#ifdef USG_SIGNAL
+#ifdef USG
   /* USG systems forget handlers when they are used;
      must reestablish each time */
   signal (signo, arith_error);
 #endif /* USG */
+#ifdef VMS
+  /* VMS systems are like USG.  */
+  signal (signo, arith_error);
+#endif /* VMS */
 #ifdef BSD4_1
   sigrelse (SIGFPE);
 #else /* not BSD4_1 */
-#ifdef POSIX_SIGNAL
-  sigset_t mask;
-  mask = zero_sigmask;
-  sigprocmask (SIG_SETMASK, &mask, (sigset_t *)0);
-#else  /* not POSIX */
-  sigsetmask (0);
-#endif /* not POSIX */
+  sigsetmask (SIGEMPTYMASK);
 #endif /* not BSD4_1 */
 
   Fsignal (Qarith_error, Qnil);
