@@ -1,23 +1,31 @@
 /* Fully extensible Emacs, running on Unix, intended for GNU.
-   Copyright (C) 1985, 1986, 1987 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1990 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.5 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/emacs.c,v $
+ * $Date: 91/09/08 22:08:48 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 91/09/08 22:08:48 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/emacs.c,v $ $Revision: 1.5 $";
+#endif
 
 #include <signal.h>
 #include <errno.h>
@@ -55,24 +63,6 @@ and this notice must be preserved on all copies.  */
 
 #define PRIO_PROCESS 0
 
-#ifdef POSIX_SIGNAL
-sigset_t zero_sigmask;			/* sigmask with all 0's */
-struct sigaction zero_sigaction;	/* sigaction with all 0's */
- 
-#ifndef signal
-/* Rather than changing all of the signal calls directly, do so with
-   the preprocessor.  This macro does not return anything.  */
-
-#define signal(num, func) \
-  { \
-    struct sigaction sa; \
-    sa = zero_sigaction; \
-    sa.sa_handler = (void (*)()) func; \
-    sigaction (num, &sa, (struct sigaction *)0); \
-  }
-#endif /* signal */
-#endif /* POSIX_SIGNAL */
-
 /* Command line args from shell, as list of strings */
 Lisp_Object Vcommand_line_args;
 
@@ -91,6 +81,10 @@ Lisp_Object Vsystem_type;
    but instead should use the virtual terminal under which it was started */
 int inhibit_window_system;
 
+/* If non-zero, we are running under X and want a distinct screen for the
+   minibuffer. */
+int distinct_minibuffer;
+
 #ifdef HAVE_X_WINDOWS
 /* If -d option is used, this variable points to the name of
    the display to use.  */
@@ -100,6 +94,13 @@ char **xargv;
 int xargc;
 #endif /* HAVE_X_WINDOWS */
 
+#ifdef USG_SHARED_LIBRARIES
+/* If nonzero, this is the place to put the end of the writable segment
+   at startup.  */
+
+unsigned int bss_end = 0;
+#endif
+
 /* Nonzero means running Emacs without interactive terminal.  */
 
 int noninteractive;
@@ -172,20 +173,15 @@ init_cmdargs (argc, argv, skip_args)
      int skip_args;
 {
   register int i;
-
-#ifdef HAVE_X_WINDOWS
   extern char *XD_command_line,*x_make_command_line();
 
   if (!inhibit_window_system && !noninteractive)
-      {
+    {
       XD_command_line = x_make_command_line(argc,argv);
-      /* load up the defaults, and snarf all the X stuff out of the command
-       * line
+      /* Load defaults, and snarf X stuff out of command line.
        */
       x_load_defaults(&argc,argv);
-      }
-#endif
-
+    }
 
   Vcommand_line_args = Qnil;
 
@@ -197,7 +193,16 @@ init_cmdargs (argc, argv, skip_args)
     }
 }
 
+#ifdef VMS
+#ifdef LINK_CRTL_SHARE
+#ifdef SHAREABLE_LIB_BUG
+extern noshare char **environ;
+#endif /* SHAREABLE_LIB_BUG */
+#endif /* LINK_CRTL_SHARE */
+#endif /* VMS */
+
 /* ARGSUSED */
+int malloc_cookie;
 main (argc, argv, envp)
      int argc;
      char **argv;
@@ -206,14 +211,31 @@ main (argc, argv, envp)
   int skip_args = 0;
   extern int errno;
   extern void malloc_warning ();
+#ifdef NeXT
+  if (initialized)
+    if (malloc_jumpstart(malloc_cookie) != 0)
+      printf("malloc jumpstart failed\n");
+#endif  
+
+  distinct_minibuffer = 1;	/* Default */
+
+/* Map in shared memory, if we are using that.  */
+#ifdef HAVE_SHM
+  if (argc > 1 && !strcmp (argv[1], "-nl"))
+    {
+      map_in_data (0);
+      /* The shared momory was just restored, which clobbered this.  */
+      skip_args = 1;
+    }
+  else
+    {
+      map_in_data (1);
+      /* The shared momory was just restored, which clobbered this.  */
+      skip_args = 0;
+    }
+#endif
 
 #ifdef VMS
-#ifdef LINK_CRTL_SHARE
-#ifdef SHAREABLE_LIB_BUG
-  extern noshare char **environ;
-#endif /* SHAREABLE_LIB_BUG */
-#endif /* LINK_CRTL_SHARE */
-
   /* If -map specified, map the data file in */
   if (argc > 2 && ! strcmp (argv[1], "-map"))
     {
@@ -236,17 +258,9 @@ main (argc, argv, envp)
 #endif /* LINK_CRTL_SHARE */
 #endif /* VMS */
 
-#ifdef TRACE
-  {
-    extern time_t time ();
-    extern char *ctime ();
-    int i;
-    time_t t = time ((time_t *)0);
-    init_trace ();
-    trace ("Trace started: %s", ctime (&t));
-    for (i = 0; i < argc; i++)
-      trace ("Argument %2d = %s\n", i, argv[i]);
-  }
+#ifdef USG_SHARED_LIBRARIES
+  if (bss_end)
+    brk (bss_end);
 #endif
 
   clearerr (stdin);
@@ -268,46 +282,39 @@ main (argc, argv, envp)
 #ifdef HIGHPRI
   setpriority (PRIO_PROCESS, getpid (), HIGHPRI);
   setuid (getuid ());
-#endif /* HIGHPRI */
+#endif HIGHPRI
 
   inhibit_window_system = 0;
 
-#ifdef HAVE_X_WINDOWS
   xargv = argv;
   xargc = argc;
-#endif
 
 /* Handle the -t switch, which specifies filename to use as terminal */
   if (skip_args + 2 < argc && !strcmp (argv[skip_args + 1], "-t"))
-    {
-      /*
-       * [cjl] Epoch does not support the -t switch
-       */
-      fprintf(stderr,"Sorry, Epoch does not support the -t command line option\n");
-      exit(1);
-    }
-#ifdef HAVE_X_WINDOWS
-#ifndef CB
-/* Handle the -d switch, which means use a different display for X */
-  if (skip_args + 2 < argc && (!strcmp (argv[skip_args + 1], "-d") ||
-			       !strcmp (argv[skip_args + 1], "-display")))
     {
       skip_args += 2;
-      XD_display_name = argv[skip_args];
-    } 
-  else
-    XD_display_name = 0;
-#endif
-#endif	/* HAVE_X_WINDOWS */
+      close (0);
+      close (1);
+      open (argv[skip_args], O_RDWR, 2 );
+      dup (0);
+      fprintf (stderr, "Using %s\n", argv[skip_args]);
+      inhibit_window_system = 1;	/* -t => -nw */
+      distinct_minibuffer = 0;	/* no distinct minibuffer */
+    }
 
   if (skip_args + 1 < argc
       && (!strcmp (argv[skip_args + 1], "-nw")))
     {
-      /*
-       * [cjl] Epoch does not support the -nw switch
-       */
-      fprintf(stderr,"Sorry, Epoch does not support the -nw command line option\n");
-      exit(1);
+      skip_args += 1;
+      inhibit_window_system = 1; /* no X window system */
+      distinct_minibuffer = 0;	/* No distinct minibuffer */
+    }
+
+  if (skip_args + 1 < argc
+      && (!strcmp (argv[skip_args +1], "-nm")))
+    {
+      skip_args += 1;
+      distinct_minibuffer = 0;
     }
 
 /* Handle the -batch switch, which means don't do interactive display.  */
@@ -318,6 +325,13 @@ main (argc, argv, envp)
       noninteractive = 1;
     }
 
+#ifdef POSIX_SIGNALS
+  /* initialize some empty and full sigsets - DJB */
+  /* these are declared in sysdep.c */
+  sigemptyset (&empty_mask);
+  sigfillset (&full_mask);
+#endif /* POSIX_SIGNALS */
+
   if (
 #ifndef CANNOT_DUMP
       ! noninteractive || initialized
@@ -347,9 +361,9 @@ main (argc, argv, envp)
 #endif
 #ifdef SIGXFSZ
       signal (SIGXFSZ, fatal_error_signal);
-#endif /* SIGXFSZ */
+#endif SIGXFSZ
 
-#ifdef IBMRTAIX
+#ifdef AIX
       signal (SIGDANGER, fatal_error_signal);
       signal (20, fatal_error_signal);
       signal (21, fatal_error_signal);
@@ -363,7 +377,7 @@ main (argc, argv, envp)
       signal (SIGRETRACT, fatal_error_signal);
       signal (SIGSOUND, fatal_error_signal);
       signal (SIGMSG, fatal_error_signal);
-#endif
+#endif /* AIX */
     }
 
   noninteractive1 = noninteractive;
@@ -424,7 +438,7 @@ main (argc, argv, envp)
       syms_of_alloc ();
 #ifdef MAINTAIN_ENVIRONMENT
       syms_of_environ ();
-#endif /* MAINTAIN_ENVIRONMENT */
+#endif MAINTAIN_ENVIRONMENT
       syms_of_read ();
       syms_of_print ();
       syms_of_eval ();
@@ -440,7 +454,6 @@ main (argc, argv, envp)
 #ifndef NO_DIR_LIBRARY
       syms_of_dired ();
 #endif /* not NO_DIR_LIBRARY */
-      syms_of_screen () ;
       syms_of_display ();
       syms_of_doc ();
       syms_of_editfns ();
@@ -463,12 +476,10 @@ main (argc, argv, envp)
       syms_of_syntax ();
       syms_of_undo ();
       syms_of_window ();
+      syms_of_screen (); 
       syms_of_xdisp ();
 #ifdef HAVE_X_WINDOWS
       syms_of_xfns ();
-#ifdef HAVE_X_MENU
-      syms_of_xmenu ();
-#endif /* HAVE_X_MENU */
 #endif /* HAVE_X_WINDOWS */
 
 #ifdef SYMS_SYSTEM
@@ -519,30 +530,15 @@ return ARG as the exit program code.")
 {
   Lisp_Object answer;
   int i;
+  struct gcpro gcpro1;
+
+  GCPRO1 (arg);
 
   if (!NULL (Vkill_emacs_hook))
     call0 (Vkill_emacs_hook);
 
   if (feof (stdin))
     arg = Qt;
-  if (!noninteractive && NULL (arg))
-    {
-      if ((i = ModExist())
-	  && (answer = Fyes_or_no_p (format1 (
-"%d modified buffer%s exist%s, do you really want to exit? ",
-					      i, i == 1 ? "" : "s",
-					      i == 1 ? "s" : "")),
-	      NULL (answer)))
-	return Qnil;
-
-#ifdef subprocesses
-      if (count_active_processes()
-	  && (answer = Fyes_or_no_p (format1 (
-"Subprocesses are executing; kill them and exit? ")),
-	      NULL (answer)))
-	return Qnil;
-#endif /* subprocesses */
-    }
 
 #ifdef subprocesses
   kill_buffer_processes (Qnil);
@@ -560,12 +556,20 @@ return ARG as the exit program code.")
 
   fflush (stdout);
   reset_sys_modes ();
+  UNGCPRO;
+
 /* Is it really necessary to do this deassign
    when we are going to exit anyway?  */
 /* #ifdef VMS
   stop_vms_input ();
  #endif  */
   stuff_buffered_input (arg);
+#ifdef SIGIO
+  /* There is a tendency for a SIGIO signal to arrive within exit,
+     and cause a SIGHUP because the input descriptor is already closed.  */
+  unrequest_sigio ();
+  signal (SIGIO, SIG_IGN);
+#endif
   exit ((XTYPE (arg) == Lisp_Int) ? XINT (arg)
 #ifdef VMS
 	: 1
@@ -580,13 +584,45 @@ return ARG as the exit program code.")
 /* Nothing like this can be implemented on an Apollo.
    What a loss!  */
 
+#ifdef HAVE_SHM
+
+DEFUN ("dump-emacs-data", Fdump_emacs_data, Sdump_emacs_data, 1, 1, 0,
+  "Dump current state of Emacs into data file FILENAME.\n\
+This function exists on systems that use HAVE_SHM.")
+  (intoname)
+     Lisp_Object intoname;
+{
+  extern int my_edata;
+  Lisp_Object tem;
+  extern void malloc_warning ();
+
+  CHECK_STRING (intoname, 0);
+  intoname = Fexpand_file_name (intoname, Qnil);
+
+  tem = Vpurify_flag;
+  Vpurify_flag = Qnil;
+
+  fflush (stdout);
+  /* Tell malloc where start of impure now is */
+  /* Also arrange for warnings when nearly out of space.  */
+#ifndef SYSTEM_MALLOC
+  malloc_init (&my_edata, malloc_warning);
+#endif
+  map_out_data (XSTRING (intoname)->data);
+
+  Vpurify_flag = tem;
+
+  return Qnil;
+}
+
+#else /* not HAVE_SHM */
+
 DEFUN ("dump-emacs", Fdump_emacs, Sdump_emacs, 2, 2, 0,
   "Dump current state of Emacs into executable file FILENAME.\n\
 Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
   (intoname, symname)
      Lisp_Object intoname, symname;
 {
-  register unsigned char *a_name = 0;
   extern int my_edata;
   Lisp_Object tem;
   extern void malloc_warning ();
@@ -597,10 +633,7 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
     {
       CHECK_STRING (symname, 0);
       if (XSTRING (symname)->size)
-	{
-	  symname = Fexpand_file_name (symname, Qnil);
-	  a_name = XSTRING (symname)->data;
-	}
+	symname = Fexpand_file_name (symname, Qnil);
     }
 
   tem = Vpurify_flag;
@@ -615,7 +648,8 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
 #ifndef SYSTEM_MALLOC
   malloc_init (&my_edata, malloc_warning);
 #endif
-  unexec (XSTRING (intoname)->data, a_name, &my_edata, 0, 0);
+  unexec (XSTRING (intoname)->data,
+	  !NULL (symname) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
 #endif /* not VMS */
 
   Vpurify_flag = tem;
@@ -623,6 +657,8 @@ Take symbols from SYMFILE (presumably the file you executed to run Emacs).")
   return Qnil;
 }
 
+#endif /* not HAVE_SHM */
+
 #endif /* not CANNOT_DUMP */
 
 #ifdef VMS
@@ -635,12 +671,13 @@ Lisp_Object
 decode_env_path (evarname, defalt)
      char *evarname, *defalt;
 {
-  register char *path, *p;
+  register char *path=0, *p;
   extern char *index ();
 
   Lisp_Object lpath;
 
-  path = (char *) egetenv (evarname);
+  if (evarname)
+    path = (char *) egetenv (evarname);
   if (!path)
     path = defalt;
   lpath = Qnil;
@@ -661,7 +698,11 @@ decode_env_path (evarname, defalt)
 syms_of_emacs ()
 {
 #ifndef CANNOT_DUMP
+#ifdef HAVE_SHM
+  defsubr (&Sdump_emacs_data);
+#else
   defsubr (&Sdump_emacs);
+#endif
 #endif /* not CANNOT_DUMP */
 
   defsubr (&Skill_emacs);
