@@ -1,23 +1,31 @@
 /* Evaluator for GNU Emacs Lisp interpreter.
-   Copyright (C) 1985, 1986, 1987 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1990 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.4 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/eval.c,v $
+ * $Date: 92/02/23 11:00:55 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:00:55 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/eval.c,v $ $Revision: 1.4 $";
+#endif
 
 #include "config.h"
 #include "lisp.h"
@@ -25,7 +33,7 @@ and this notice must be preserved on all copies.  */
 #ifndef standalone
 #include "commands.h"
 #else
-#define INTERACTIVE 1
+#define FROM_KBD 1
 #endif
 
 #include <setjmp.h>
@@ -55,14 +63,16 @@ struct catchtag
     struct gcpro *gcpro;
     jmp_buf jmp;
     struct backtrace *backlist;
+    struct handler *handlerlist;
     int lisp_eval_depth;
     int pdlcount;
+    int poll_suppress_count;
   };
 
 struct catchtag *catchlist;
 
 Lisp_Object Qautoload, Qmacro, Qexit, Qinteractive, Qcommandp, Qdefun;
-Lisp_Object Vquit_flag, Vinhibit_quit;
+Lisp_Object Vquit_flag, Vinhibit_quit, Qinhibit_quit;
 Lisp_Object Qmocklisp_arguments, Vmocklisp_arguments, Qmocklisp;
 Lisp_Object Qand_rest, Qand_optional;
 
@@ -173,7 +183,7 @@ If all args return NIL, return NIL.")
   Lisp_Object args_left;
   struct gcpro gcpro1;
 
-  if (NULL(args))
+  if (NIL(args))
     return Qnil;
 
   args_left = args;
@@ -182,11 +192,11 @@ If all args return NIL, return NIL.")
   do
     {
       val = Feval (Fcar (args_left));
-      if (!NULL (val))
+      if (!NIL (val))
 	break;
       args_left = Fcdr (args_left);
     }
-  while (!NULL(args_left));
+  while (!NIL(args_left));
 
   UNGCPRO;
   return val;
@@ -203,7 +213,7 @@ If no arg yields NIL, return the last arg's value.")
   Lisp_Object args_left;
   struct gcpro gcpro1;
 
-  if (NULL(args))
+  if (NIL(args))
     return Qt;
 
   args_left = args;
@@ -212,11 +222,11 @@ If no arg yields NIL, return the last arg's value.")
   do
     {
       val = Feval (Fcar (args_left));
-      if (NULL (val))
+      if (NIL (val))
 	break;
       args_left = Fcdr (args_left);
     }
-  while (!NULL(args_left));
+  while (!NIL(args_left));
 
   UNGCPRO;
   return val;
@@ -236,7 +246,7 @@ There may be no E's; then if C yields NIL, the value is NIL.")
   cond = Feval (Fcar (args));
   UNGCPRO;
 
-  if (!NULL (cond))
+  if (!NIL (cond))
     return Feval (Fcar (Fcdr (args)));
   return Fprogn (Fcdr (Fcdr (args)));
 }
@@ -252,15 +262,15 @@ If no clause succeeds, cond returns nil.")
   (args)
      Lisp_Object args;
 {
-  register Lisp_Object clause, val = Qnil;
+  register Lisp_Object clause, val;
   struct gcpro gcpro1;
 
   GCPRO1 (args);
-  while (!NULL (args))
+  while (!NIL (args))
     {
       clause = Fcar (args);
       val = Feval (Fcar (clause));
-      if (!NULL (val))
+      if (!NIL (val))
 	{
 	  if (!EQ (XCONS (clause)->cdr, Qnil))
 	    val = Fprogn (XCONS (clause)->cdr);
@@ -287,14 +297,14 @@ DEFUN ("progn", Fprogn, Sprogn, 0, UNEVALLED, 0,
   if (!EQ (Vmocklisp_arguments, Qt))
     {
       val = make_number (0);
-      while (!NULL (args) && (tem = Fcar (args), XTYPE (tem) == Lisp_Symbol))
+      while (!NIL (args) && (tem = Fcar (args), XTYPE (tem) == Lisp_Symbol))
 	{
 	  QUIT;
 	  specbind (tem, val), args = Fcdr (args);
 	}
     }
 
-  if (NULL(args))
+  if (NIL(args))
     return Qnil;
 
   args_left = args;
@@ -305,7 +315,7 @@ DEFUN ("progn", Fprogn, Sprogn, 0, UNEVALLED, 0,
       val = Feval (Fcar (args_left));
       args_left = Fcdr (args_left);
     }
-  while (!NULL(args_left));
+  while (!NIL(args_left));
 
   UNGCPRO;
   return val;
@@ -323,7 +333,7 @@ whose values are discarded.")
   struct gcpro gcpro1, gcpro2;
   register int argnum = 0;
 
-  if (NULL(args))
+  if (NIL(args))
     return Qnil;
 
   args_left = args;
@@ -338,7 +348,7 @@ whose values are discarded.")
 	Feval (Fcar (args_left));
       args_left = Fcdr (args_left);
     }
-  while (!NULL(args_left));
+  while (!NIL(args_left));
 
   UNGCPRO;
   return val;
@@ -358,7 +368,7 @@ whose values are discarded.")
 
   val = Qnil;
 
-  if (NULL(args))
+  if (NIL(args))
     return Qnil;
 
   args_left = args;
@@ -373,7 +383,7 @@ whose values are discarded.")
 	Feval (Fcar (args_left));
       args_left = Fcdr (args_left);
     }
-  while (!NULL(args_left));
+  while (!NIL(args_left));
 
   UNGCPRO;
   return val;
@@ -390,7 +400,7 @@ Each SYM is set before the next VAL is computed.")
   register Lisp_Object val, sym;
   struct gcpro gcpro1;
 
-  if (NULL(args))
+  if (NIL(args))
     return Qnil;
 
   args_left = args;
@@ -403,7 +413,7 @@ Each SYM is set before the next VAL is computed.")
       Fset (sym, val);
       args_left = Fcdr (Fcdr (args_left));
     }
-  while (!NULL(args_left));
+  while (!NIL(args_left));
 
   UNGCPRO;
   return val;
@@ -438,7 +448,7 @@ and input is currently coming from the keyboard (not in keyboard macro).")
   register struct backtrace *btp;
   register Lisp_Object fun;
 
-  if (!INTERACTIVE)
+  if (!FROM_KBD)
     return Qnil;
   /* Skip the frame of interactive-p itself (if interpreted)
      or the frame of byte-code (if called from compiled function).  */
@@ -478,7 +488,7 @@ See also the function  interactive .")
 
   fn_name = Fcar (args);
   defn = Fcons (Qlambda, Fcdr (args));
-  if (!NULL (Vpurify_flag))
+  if (!NIL (Vpurify_flag))
     defn = Fpurecopy (defn);
   Ffset (fn_name, defn);
   return fn_name;
@@ -499,7 +509,7 @@ and the result should be a form to be evaluated instead of the original.")
 
   fn_name = Fcar (args);
   defn = Fcons (Qmacro, Fcons (Qlambda, Fcdr (args)));
-  if (!NULL (Vpurify_flag))
+  if (!NIL (Vpurify_flag))
     defn = Fpurecopy (defn);
   Ffset (fn_name, defn);
   return fn_name;
@@ -519,16 +529,16 @@ If INITVALUE is missing, SYMBOL's value is not set.")
 
   sym = Fcar (args);
   tem = Fcdr (args);
-  if (!NULL (tem))
+  if (!NIL (tem))
     {
       tem = Fboundp (sym);
-      if (NULL (tem))
+      if (NIL (tem))
 	Fset (sym, Feval (Fcar (Fcdr (args))));
     }
   tem = Fcar (Fcdr (Fcdr (args)));
-  if (!NULL (tem))
+  if (!NIL (tem))
     {
-      if (!NULL (Vpurify_flag))
+      if (!NIL (Vpurify_flag))
 	tem = Fpurecopy (tem);
       Fput (sym, Qvariable_documentation, tem);
     }
@@ -550,9 +560,9 @@ If DOCSTRING starts with *, this variable is identified as a user option.\n\
   sym = Fcar (args);
   Fset (sym, Feval (Fcar (Fcdr (args))));
   tem = Fcar (Fcdr (Fcdr (args)));
-  if (!NULL (tem))
+  if (!NIL (tem))
     {
-      if (!NULL (Vpurify_flag))
+      if (!NIL (Vpurify_flag))
 	tem = Fpurecopy (tem);
       Fput (sym, Qvariable_documentation, tem);
     }
@@ -594,7 +604,7 @@ Each VALUEFORM can refer to the symbols already bound by this VARLIST.")
   GCPRO3 (args, elt, varlist);
 
   varlist = Fcar (args);
-  while (!NULL (varlist))
+  while (!NIL (varlist))
     {
       QUIT;
       elt = Fcar (varlist);
@@ -639,7 +649,7 @@ All the VALUEFORMs are evalled before any symbols are bound.")
   GCPRO2 (args, *temps);
   gcpro2.nvars = 0;
 
-  for (argnum = 0; !NULL (varlist); varlist = Fcdr (varlist))
+  for (argnum = 0; !NIL (varlist); varlist = Fcdr (varlist))
     {
       QUIT;
       elt = Fcar (varlist);
@@ -652,7 +662,7 @@ All the VALUEFORMs are evalled before any symbols are bound.")
   UNGCPRO;
 
   varlist = Fcar (args);
-  for (argnum = 0; !NULL (varlist); varlist = Fcdr (varlist))
+  for (argnum = 0; !NIL (varlist); varlist = Fcdr (varlist))
     {
       elt = Fcar (varlist);
       tem = temps[argnum++];
@@ -679,7 +689,7 @@ DEFUN ("while", Fwhile, Swhile, 1, UNEVALLED, 0,
 
   test = Fcar (args);
   body = Fcdr (args);
-  while (tem = Feval (test), !NULL (tem))
+  while (tem = Feval (test), !NIL (tem))
     {
       QUIT;
       Fprogn (body);
@@ -699,7 +709,7 @@ definitions to shadow the loaded ones for use in file byte-compilation.")
      register Lisp_Object form;
      Lisp_Object env;
 {
-  register Lisp_Object expander, sym, def = Qnil, tem;
+  register Lisp_Object expander, sym, def, tem;
 
   while (1)
     {
@@ -716,7 +726,7 @@ definitions to shadow the loaded ones for use in file byte-compilation.")
 	{
 	  QUIT;
 	  tem = Fassq (sym, env);
-	  if (NULL (tem))
+	  if (NIL (tem))
 	    {
 	      def = XSYMBOL (sym)->function;
 	      if (XTYPE (def) == Lisp_Symbol && !EQ (def, Qunbound))
@@ -735,7 +745,7 @@ definitions to shadow the loaded ones for use in file byte-compilation.")
 	}
       /* Right now TEM is the result from SYM in ENV,
 	 and if TEM is nil then DEF is SYM's function definition.  */
-      if (NULL (tem))
+      if (NIL (tem))
 	{
 	  /* SYM is not mentioned in ENV.
 	     Look at its function definition.  */
@@ -747,7 +757,7 @@ definitions to shadow the loaded ones for use in file byte-compilation.")
 	    {
 	      /* Autoloading function: will it be a macro when loaded?  */
 	      tem = Fcar (Fnthcdr (make_number (4), def));
-	      if (NULL (tem))
+	      if (NIL (tem))
 		break;
 	      /* Yes, load it and try again.  */
 	      do_autoload (def, sym);
@@ -760,7 +770,7 @@ definitions to shadow the loaded ones for use in file byte-compilation.")
       else
 	{
 	  expander = XCONS (tem)->cdr;
-	  if (NULL (expander))
+	  if (NIL (expander))
 	    break;
 	}
       form = apply1 (expander, XCONS (form)->cdr);
@@ -804,7 +814,9 @@ internal_catch (tag, func, arg)
   c.tag = tag;
   c.val = Qnil;
   c.backlist = backtrace_list;
+  c.handlerlist = handlerlist;
   c.lisp_eval_depth = lisp_eval_depth;
+  c.poll_suppress_count = poll_suppress_count;
   c.pdlcount = specpdl_ptr - specpdl;
   c.gcpro = gcprolist;
   catchlist = &c;
@@ -839,8 +851,7 @@ unbind_catch (catch)
     {
       last_time = catchlist == catch;
       unbind_to (catchlist->pdlcount);
-      if (handlerlist != 0 && handlerlist->tag == catchlist)
-	handlerlist = handlerlist->next;
+      handlerlist = catchlist->handlerlist;
       catchlist = catchlist->next;
     }
   while (! last_time);
@@ -849,31 +860,7 @@ unbind_catch (catch)
   backtrace_list = catch->backlist;
   lisp_eval_depth = catch->lisp_eval_depth;
 }
-#ifdef DGUX
-DEFUN2 ("throw", Fthrow, Sthrow, 2, 2, 0,
- "(throw TAG VALUE): throw to the catch for TAG and return VALUE from it.\n\
-Both TAG and VALUE are evalled.")
-  (tag, val)
-     register Lisp_Object tag, val;
-{
-  register struct catchtag *c;
 
-  while (1)
-    {
-      if (!NULL (tag))
-	for (c = catchlist; c; c = c->next)
-	  {
-	    if (EQ (c->tag, tag))
-	      {
-		c->val = val;
-		unbind_catch (c);
-		_longjmp (c->jmp, 1);
-	      }
-	  }
-      tag = Fsignal (Qno_catch, Fcons (tag, Fcons (val, Qnil)));
-    }
-}
-#else
 DEFUN ("throw", Fthrow, Sthrow, 2, 2, 0,
   "(throw TAG VALUE): throw to the catch for TAG and return VALUE from it.\n\
 Both TAG and VALUE are evalled.")
@@ -884,11 +871,16 @@ Both TAG and VALUE are evalled.")
 
   while (1)
     {
-      if (!NULL (tag))
+      if (!NIL (tag))
 	for (c = catchlist; c; c = c->next)
 	  {
 	    if (EQ (c->tag, tag))
 	      {
+		/* Restore the polling-suppression count.  */
+		if (c->poll_suppress_count > poll_suppress_count)
+		  abort ();
+		while (c->poll_suppress_count < poll_suppress_count)
+		  start_polling ();		
 		c->val = val;
 		unbind_catch (c);
 		_longjmp (c->jmp, 1);
@@ -897,8 +889,6 @@ Both TAG and VALUE are evalled.")
       tag = Fsignal (Qno_catch, Fcons (tag, Fcons (val, Qnil)));
     }
 }
-#endif
-
 
 
 DEFUN ("unwind-protect", Funwind_protect, Sunwind_protect, 1, UNEVALLED, 0,
@@ -957,12 +947,14 @@ See SIGNAL for more info.")
   c.tag = Qnil;
   c.val = Qnil;
   c.backlist = backtrace_list;
+  c.handlerlist = handlerlist;
   c.lisp_eval_depth = lisp_eval_depth;
+  c.poll_suppress_count = poll_suppress_count;
   c.pdlcount = specpdl_ptr - specpdl;
   c.gcpro = gcprolist;
   if (_setjmp (c.jmp))
     {
-      if (!NULL (h.var))
+      if (!NIL (h.var))
         specbind (h.var, Fcdr (c.val));
       val = Fprogn (Fcdr (Fcar (c.val)));
       unbind_to (c.pdlcount);
@@ -973,15 +965,16 @@ See SIGNAL for more info.")
   h.var = Fcar (args);
   h.handler = Fcdr (Fcdr (args));
   
-  for (val = h.handler; ! NULL (val); val = Fcdr (val))
+  for (val = h.handler; ! NIL (val); val = Fcdr (val))
     {
       tem = Fcar (val);
-      if ((!NULL (tem)) &&
+      if ((!NIL (tem)) &&
 	  (!CONSP (tem) || (XTYPE (XCONS (tem)->car) != Lisp_Symbol)))
 	error ("Invalid condition handler", tem);
     }
   
   h.next = handlerlist;
+  h.poll_suppress_count = poll_suppress_count;
   h.tag = &c;
   handlerlist = &h;
 
@@ -1004,7 +997,9 @@ internal_condition_case (bfun, handlers, hfun)
   c.tag = Qnil;
   c.val = Qnil;
   c.backlist = backtrace_list;
+  c.handlerlist = handlerlist;
   c.lisp_eval_depth = lisp_eval_depth;
+  c.poll_suppress_count = poll_suppress_count;
   c.pdlcount = specpdl_ptr - specpdl;
   c.gcpro = gcprolist;
   if (_setjmp (c.jmp))
@@ -1015,6 +1010,7 @@ internal_condition_case (bfun, handlers, hfun)
   catchlist = &c;
   h.handler = handlers;
   h.var = Qnil;
+  h.poll_suppress_count = poll_suppress_count;
   h.next = handlerlist;
   h.tag = &c;
   handlerlist = &h;
@@ -1044,26 +1040,22 @@ See  condition-case.")
   Lisp_Object conditions;
   extern int gc_in_progress;
   extern int waiting_for_input;
-#ifdef CB
   extern int x_in_handler_hook;
   extern Lisp_Object Vx_event_handler;
   extern Lisp_Object Vx_event_handler_abort;
-#endif
   Lisp_Object debugger_value;
 
-  quit_error_check ();
   immediate_quit = 0;
   if (gc_in_progress || waiting_for_input)
     abort ();
 
-#ifdef CB
   /* if we are getting error conditions while in the coed_hook we need to
    * remove the handler so we don't lock up. Flag reset is handled by an
    * unwind-protect in the handler invocation function.
    */
   if (x_in_handler_hook && !EQ(Vx_event_handler_abort,Qnil))
-      Vx_event_handler = Qnil;
-#endif
+      Vx_event_handler = Qnil;  
+  
 
   conditions = Fget (sig, Qerror_conditions);
 
@@ -1078,9 +1070,14 @@ See  condition-case.")
       if (EQ (clause, Qlambda))
 	return debugger_value;
 
-      if (!NULL (clause))
+      if (!NIL (clause))
 	{
 	  struct handler *h = handlerlist;
+	  /* Restore the polling-suppression count.  */
+	  if (h->poll_suppress_count > poll_suppress_count)
+	    abort ();
+	  while (h->poll_suppress_count < poll_suppress_count)
+	    start_polling ();	  
 	  handlerlist = allhandlers;
 	  unbind_catch (h->tag);
 	  h->tag->val = Fcons (clause, Fcons (sig, data));
@@ -1130,7 +1127,7 @@ find_handler_clause (handlers, conditions, sig, data, debugger_value_ptr)
       if (!CONSP (tem1))
 	continue;
       tem = Fmemq (Fcar (tem1), conditions);
-      if (!NULL (tem))
+      if (!NIL (tem))
         return tem1;
     }
   return Qnil;
@@ -1173,7 +1170,7 @@ Also, a symbol is commandp if its function definition is commandp.")
     {
       if (++i > 10) return Qnil;
       tem = Ffboundp (fun);
-      if (NULL (tem)) return Qnil;
+      if (NIL (tem)) return Qnil;
       fun = Fsymbol_function (fun);
     }
   if (XTYPE (fun) == Lisp_Subr)
@@ -1203,7 +1200,7 @@ DEFUN ("autoload", Fautoload, Sautoload, 2, 5, 0,
   "Define FUNCTION to autoload from FILE.\n\
 FUNCTION is a symbol; FILE is a file name string to pass to  load.\n\
 Third arg DOCSTRING is documentation for the function.\n\
-Fourth arg INTERACTIVE if non-nil says function can be called interactively.\n\
+Fourth arg FROM_KBD if non-nil says function can be called interactively.\n\
 Fifth arg MACRO if non-nil says the function is really a macro.\n\
 Third through fifth args give info about the real definition.\n\
 They default to nil.\n\
@@ -1306,7 +1303,7 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
       if (EQ (Vmocklisp_arguments, Qt))
         return Fsymbol_value (form);
       val = Fsymbol_value (form);
-      if (NULL (val))
+      if (NIL (val))
 	XFASTINT (val) = 0;
       else if (EQ (val, Qt))
 	XFASTINT (val) = 1;
@@ -1361,7 +1358,7 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
   if (XTYPE (fun) == Lisp_Subr)
     {
       Lisp_Object numargs;
-      Lisp_Object argvals[MAX_MAXARGS];
+      Lisp_Object argvals[5];
       Lisp_Object args_left;
       register int i, maxargs;
 
@@ -1370,7 +1367,11 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
 
       if (XINT (numargs) < XSUBR (fun)->min_args ||
 	  (XSUBR (fun)->max_args >= 0 && XSUBR (fun)->max_args < XINT (numargs)))
-	return Fsignal (Qwrong_number_of_arguments, Fcons (fun, Fcons (numargs, Qnil)));
+	{
+	  val = Fsignal(Qwrong_number_of_arguments,
+			Fcons (fun, Fcons (numargs, Qnil)));
+	  goto done;
+	}
 
       if (XSUBR (fun)->max_args == UNEVALLED)
 	{
@@ -1391,18 +1392,18 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
 	  gcpro3.var = vals;
 	  gcpro3.nvars = 0;
 
-	  while (!NULL (args_left))
+	  while (!NIL (args_left))
 	    {
 	      vals[argnum++] = Feval (Fcar (args_left));
 	      args_left = Fcdr (args_left);
 	      gcpro3.nvars = argnum;
 	    }
-	  UNGCPRO;
 
 	  backtrace.args = vals;
 	  backtrace.nargs = XINT (numargs);
 
 	  val = (*XSUBR (fun)->function) (XINT (numargs), vals);
+	  UNGCPRO;
 	  goto done;
 	}
 
@@ -1445,36 +1446,34 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
 	  val = (*XSUBR (fun)->function) (argvals[0], argvals[1], argvals[2],
 					  argvals[3], argvals[4]);
 	  goto done;
-	case 6:
-	  val = (*XSUBR (fun)->function) (argvals[0], argvals[1], argvals[2],
-					  argvals[3], argvals[4], argvals[5]);
-  	  goto done;
-
 	}
     }
   if (!CONSP (fun))
-    return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+    {
+      val = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+      goto done;
+    }
   funcar = Fcar (fun);
   if (XTYPE (funcar) != Lisp_Symbol)
-    return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
-  if (EQ (funcar, Qautoload))
+    val = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+  else if (EQ (funcar, Qautoload))
     {
       do_autoload (fun, original_fun);
       goto retry;
     }
-  if (EQ (funcar, Qmacro))
+  else if (EQ (funcar, Qmacro))
     val = Feval (apply1 (Fcdr (fun), original_args));
   else if (EQ (funcar, Qlambda))
     val = apply_lambda (fun, original_args, 1);
   else if (EQ (funcar, Qmocklisp))
     val = ml_apply (fun, original_args);
   else
-    return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+    val = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
 
  done:
   if (!EQ (Vmocklisp_arguments, Qt))
     {
-      if (NULL (val))
+      if (NIL (val))
 	XFASTINT (val) = 0;
       else if (EQ (val, Qt))
 	XFASTINT (val) = 1;
@@ -1497,7 +1496,8 @@ Thus, (apply '+ 1 2 '(3 4)) returns 10.")
   register int i, numargs;
   register Lisp_Object spread_arg;
   register Lisp_Object *funcall_args ;
-  Lisp_Object fun;
+  Lisp_Object fun,val;
+  struct gcpro gcpro1;
 
   fun = args [0];
   funcall_args = 0;
@@ -1540,23 +1540,33 @@ Thus, (apply '+ 1 2 '(3 4)) returns 10.")
 					       * sizeof (Lisp_Object));
 	for (i = numargs; i < XSUBR (fun)->max_args;)
 	  funcall_args[++i] = Qnil;
+	GCPRO1 (*funcall_args);
+	gcpro1.nvars = 1 + XSUBR(fun)->max_args;
       }
  funcall:
   /* We add 1 to numargs because funcall_args includes the
      function itself as well as its arguments.  */
   if (!funcall_args)
-    funcall_args = (Lisp_Object *) alloca ((1 + numargs)
-					   * sizeof (Lisp_Object));
+    {
+      funcall_args = (Lisp_Object *) alloca ((1 + numargs)
+					     * sizeof (Lisp_Object));
+
+      GCPRO1 (*funcall_args);
+      gcpro1.nvars = 1 + numargs;
+    }
+  
   bcopy (args, funcall_args, nargs * sizeof (Lisp_Object));
   /* Spread the last arg we got.  Its first element goes in
      the slot that it used to occupy, hence this value of I.  */
   i = nargs - 1;
-  while (!NULL (spread_arg))
+  while (!NIL (spread_arg))
     {
       funcall_args [i++] = XCONS (spread_arg)->car;
       spread_arg = XCONS (spread_arg)->cdr;
     }
-  return Ffuncall (numargs + 1, funcall_args);
+  val = Ffuncall (numargs + 1, funcall_args);
+  UNGCPRO;
+  return val;
 }
 
 /* Apply fn to arg */
@@ -1564,18 +1574,34 @@ Lisp_Object
 apply1 (fn, arg)
      Lisp_Object fn, arg;
 {
-  if (NULL (arg))
+  register Lisp_Object val;
+  struct gcpro gcpro1;
+  
+  if (NIL (arg))
+    /* No need to protect if all we have is the function. */
     return Ffuncall (1, &fn);
+  /* We must protect the vector givent to Fapply.
+     If ARG is a list of 1 element, that same vector is passed
+     on to Ffuncall. */
+  
 #ifdef NO_ARG_ARRAY
   {
     Lisp_Object args[2];
     args[0] = fn;
     args[1] = arg;
-    return Fapply (2, args);
+    GCPRO1 (fn);
+    gcpro1.var = args;
+    gcpro1.nvars = 2;
+    val = Fapply (2, args);
+    UNGCPRO;
   }
 #else /* not NO_ARG_ARRAY */
-  return Fapply (2, &fn);
+  GCPRO1 (fn);
+  gcpro1.nvars = 2;
+  val = Fapply (2, &fn);
+  UNGCPRO;
 #endif /* not NO_ARG_ARRAY */
+  return val;
 }
 
 /* Call function fn on no arguments */
@@ -1592,14 +1618,23 @@ Lisp_Object
 call1 (fn, arg)
      Lisp_Object fn, arg;
 {
+  Lisp_Object val;
+  struct gcpro gcpro1;
 #ifdef NO_ARG_ARRAY
   Lisp_Object args[2];
+#endif
+  GCPRO1 (fn);
+  gcpro1.nvars = 2;
+#ifdef NO_ARG_ARRAY  
   args[0] = fn;
   args[1] = arg;
-  return Ffuncall (2, args);
+  gcpro1.var = args;
+  val = Ffuncall (2, args);
 #else /* not NO_ARG_ARRAY */
-  return Ffuncall (2, &fn);
+  val = Ffuncall (2, &fn);
 #endif /* not NO_ARG_ARRAY */
+  UNGCPRO;
+  return val;
 }
 
 /* Call function fn with arguments arg, arg1 */
@@ -1608,15 +1643,24 @@ Lisp_Object
 call2 (fn, arg, arg1)
      Lisp_Object fn, arg, arg1;
 {
+  Lisp_Object val;
+  struct gcpro gcpro1;
 #ifdef NO_ARG_ARRAY
   Lisp_Object args[3];
+#endif
+  GCPRO1 (fn);
+  gcpro1.nvars = 3;
+#ifdef NO_ARG_ARRAY  
   args[0] = fn;
   args[1] = arg;
   args[2] = arg1;
-  return Ffuncall (3, args);
+  gcpro1.var = args;
+  val =  Ffuncall (3, args);
 #else /* not NO_ARG_ARRAY */
-  return Ffuncall (3, &fn);
+  val = Ffuncall (3, &fn);
 #endif /* not NO_ARG_ARRAY */
+  UNGCPRO;
+  return val;
 }
 
 /* Call function fn with arguments arg, arg1, arg2 */
@@ -1625,16 +1669,25 @@ Lisp_Object
 call3 (fn, arg, arg1, arg2)
      Lisp_Object fn, arg, arg1, arg2;
 {
+  Lisp_Object val;
+  struct gcpro gcpro1;
 #ifdef NO_ARG_ARRAY
   Lisp_Object args[4];
+#endif
+  GCPRO1 (fn);
+  gcpro1.nvars = 4;
+#ifdef NO_ARG_ARRAY  
   args[0] = fn;
   args[1] = arg;
   args[2] = arg1;
   args[3] = arg2;
-  return Ffuncall (4, args);
+  gcpro1.var = args;
+  val = Ffuncall (4, args);
 #else /* not NO_ARG_ARRAY */
-  return Ffuncall (4, &fn);
+  val = Ffuncall (4, &fn);
 #endif /* not NO_ARG_ARRAY */
+  UNGCPRO;
+  return val;
 }
 
 DEFUN ("funcall", Ffuncall, Sfuncall, 1, MANY, 0,
@@ -1650,18 +1703,12 @@ Thus,  (funcall 'cons 'x 'y)  returns  (x . y).")
   Lisp_Object lisp_numargs;
   Lisp_Object val;
   struct backtrace backtrace;
-  struct gcpro gcpro1;
   register Lisp_Object *internal_args;
   register int i;
 
   QUIT;
   if (consing_since_gc > gc_cons_threshold)
-    {
-      GCPRO1 (*args);
-      gcpro1.nvars = nargs;
-      Fgarbage_collect ();
-      UNGCPRO;
-    }
+    Fgarbage_collect ();    
 
   if (++lisp_eval_depth > max_lisp_eval_depth)
     {
@@ -1700,11 +1747,16 @@ Thus,  (funcall 'cons 'x 'y)  returns  (x . y).")
 	  (XSUBR (fun)->max_args >= 0 && XSUBR (fun)->max_args < numargs))
 	{
 	  XFASTINT (lisp_numargs) = numargs;
-	  return Fsignal (Qwrong_number_of_arguments, Fcons (fun, Fcons (lisp_numargs, Qnil)));
+	  val = Fsignal (Qwrong_number_of_arguments,
+			 Fcons (fun, Fcons (lisp_numargs, Qnil)));
+	  goto done;
 	}
 
       if (XSUBR (fun)->max_args == UNEVALLED)
-	return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+	{
+	  val = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+	  goto done;
+	}
 
       if (XSUBR (fun)->max_args == MANY)
 	{
@@ -1730,8 +1782,7 @@ Thus,  (funcall 'cons 'x 'y)  returns  (x . y).")
 	  val = (*XSUBR (fun)->function) (internal_args[0]);
 	  goto done;
 	case 2:
-	  val = (*XSUBR (fun)->function) (internal_args[0],
-					  internal_args[1]);
+	  val = (*XSUBR (fun)->function) (internal_args[0], internal_args[1]);
 	  goto done;
 	case 3:
 	  val = (*XSUBR (fun)->function) (internal_args[0], internal_args[1],
@@ -1747,19 +1798,17 @@ Thus,  (funcall 'cons 'x 'y)  returns  (x . y).")
 					  internal_args[2], internal_args[3],
 					  internal_args[4]);
 	  goto done;
-	case 6:
-	  val = (*XSUBR (fun)->function) (internal_args[0], internal_args[1],
-					  internal_args[2], internal_args[3],
-					  internal_args[4], internal_args[5]);
-  	  goto done;
 	}
     }
   if (!CONSP (fun))
-    return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+    {
+      val = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+      goto done;
+    }
   funcar = Fcar (fun);
   if (XTYPE (funcar) != Lisp_Symbol)
-    return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
-  if (EQ (funcar, Qlambda))
+    val = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+  else if (EQ (funcar, Qlambda))
     val = funcall_lambda (fun, numargs, args + 1);
   else if (EQ (funcar, Qmocklisp))
     val = ml_apply (fun, Flist (numargs, args + 1));
@@ -1769,7 +1818,7 @@ Thus,  (funcall 'cons 'x 'y)  returns  (x . y).")
       goto retry;
     }
   else
-    return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
+    val = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
 
  done:
   lisp_eval_depth--;
@@ -1843,17 +1892,19 @@ funcall_lambda (fun, nargs, arg_vector)
   XFASTINT (numargs) = nargs;
 
   i = 0;
-  for (syms_left = Fcar (Fcdr (fun)); !NULL (syms_left); syms_left = Fcdr (syms_left))
+  for (syms_left = Fcar (Fcdr (fun)); !NIL (syms_left); syms_left = Fcdr (syms_left))
     {
       QUIT;
       next = Fcar (syms_left);
+      while (XTYPE (next) != Lisp_Symbol)
+	next = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
       if (EQ (next, Qand_rest))
 	rest = 1;
       else if (EQ (next, Qand_optional))
 	optional = 1;
       else if (rest)
 	{
-	  specbind (Fcar (syms_left), Flist (nargs - i, &arg_vector[i]));
+	  specbind (next, Flist (nargs - i, &arg_vector[i]));
 	  i = nargs;
 	}
       else if (i < nargs)
@@ -1904,6 +1955,8 @@ specbind (symbol, value)
   extern void store_symval_forwarding (); /* in eval.c */
   Lisp_Object ovalue;
 
+  CHECK_SYMBOL (symbol, 0);
+  
   if (specpdl_ptr == specpdl + specpdl_size)
     grow_specpdl ();
   specpdl_ptr->symbol = symbol;
@@ -1934,7 +1987,7 @@ void
 unbind_to (count)
      int count;
 {
-  int quitf = !NULL (Vquit_flag);
+  int quitf = !NIL (Vquit_flag);
 
   Vquit_flag = Qnil;
 
@@ -1945,12 +1998,12 @@ unbind_to (count)
 	(*specpdl_ptr->func) (specpdl_ptr->old_value);
       /* Note that a "binding" of nil is really an unwind protect,
 	so in that case the "old value" is a list of forms to evaluate.  */
-      else if (NULL (specpdl_ptr->symbol))
+      else if (NIL (specpdl_ptr->symbol))
 	Fprogn (specpdl_ptr->old_value);
       else
         Fset (specpdl_ptr->symbol, specpdl_ptr->old_value);
     }
-  if (NULL (Vquit_flag) && quitf) Vquit_flag = Qt;
+  if (NIL (Vquit_flag) && quitf) Vquit_flag = Qt;
 }
 
 #if 0
@@ -2010,7 +2063,7 @@ The debugger is entered when that frame exits, if the flag is non-nil.")
     }
 
   if (backlist)
-    backlist->debug_on_exit = !NULL (flag);
+    backlist->debug_on_exit = !NIL (flag);
 
   return flag;
 }
@@ -2022,8 +2075,12 @@ Output stream used is value of standard-output.")
 {
   register struct backtrace *backlist = backtrace_list;
   register int i;
-  register Lisp_Object tail;
+  Lisp_Object tail;
   Lisp_Object tem;
+  struct gcpro gcpro1;
+
+  tail = Qnil;
+  GCPRO1 (tail);
 
   while (backlist)
     {
@@ -2042,7 +2099,7 @@ Output stream used is value of standard-output.")
       else if (backlist->nargs == MANY)
 	{
 	  write_string ("(", -1);
-	  for (tail = *backlist->args, i = 0; !NULL (tail); tail = Fcdr (tail), i++)
+	  for (tail = *backlist->args, i = 0; !NIL (tail); tail = Fcdr (tail), i++)
 	    {
 	      if (i) write_string (" ", -1);
 	      Fprin1 (Fcar (tail), Qnil);
@@ -2060,6 +2117,8 @@ Output stream used is value of standard-output.")
       write_string (")\n", -1);
       backlist = backlist->next;
     }
+
+  UNGCPRO;
   return Qnil;
 }
 
@@ -2076,6 +2135,8 @@ syms_of_eval ()
 Typing C-G sets  quit-flag  non-nil, regardless of  inhibit-quit.");
   Vquit_flag = Qnil;
 
+  Qinhibit_quit = intern ("inhibit-quit");
+  staticpro (&Qinhibit_quit);
   DEFVAR_LISP ("inhibit-quit", &Vinhibit_quit,
     "Non-nil inhibits C-g quitting from happening immediately.\n\
 Note that  quit-flag  will still be set by typing C-g,\n\
@@ -2090,6 +2151,9 @@ before making  inhibit-quit  nil.");
   Qmacro = intern ("macro");
   staticpro (&Qmacro);
 
+  /* Note that the process handling also uses Qexit, but we don't want
+     to staticpro it twice, so we just do it here.
+     Bug fix from gnu.emacs.bug */
   Qexit = intern ("exit");
   staticpro (&Qexit);
 
