@@ -3,21 +3,29 @@
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.6 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/fns.c,v $
+ * $Date: 91/10/18 12:26:51 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 91/10/18 12:26:51 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/fns.c,v $ $Revision: 1.6 $";
+#endif
 
 #include "config.h"
 
@@ -52,15 +60,15 @@ and this notice must be preserved on all copies.  */
 #endif /* not VMS */
 #endif /* LOAD_AVE_TYPE */
 
+#ifdef DGUX
+#include <sys/dg_sys_info.h>  /* for load average info - DJB */
+#endif
+
 /* Note on some machines this defines `vector' as a typedef,
    so make sure we don't use that name in this file.  */
 #undef vector
 #define vector *****
 
-#ifdef DGUX
-#include <sys/dg_sys_info.h>	/* for getting load averages */
-#endif
-
 #ifdef NULL
 #undef NULL
 #endif
@@ -256,7 +264,7 @@ concat (nargs, args, target_type, last_special)
 {
   Lisp_Object val;
   Lisp_Object len;
-  register Lisp_Object tail = Qnil;
+  register Lisp_Object tail;
   register Lisp_Object this;
   int toindex;
   register int leni;
@@ -316,7 +324,7 @@ concat (nargs, args, target_type, last_special)
   for (argnum = 0; argnum < nargs; argnum++)
     {
       Lisp_Object thislen;
-      int thisleni = 0;
+      int thisleni;
       register int thisindex = 0;
 
       this = args[argnum];
@@ -406,8 +414,6 @@ If FROM or TO is negative, it counts from the end.")
      Lisp_Object string;
      register Lisp_Object from, to;
 {
-  register Lisp_Object val, len;
-
   CHECK_STRING (string, 0);
   CHECK_NUMBER (from, 1);
   if (NULL (to))
@@ -423,12 +429,8 @@ If FROM or TO is negative, it counts from the end.")
         && XINT (to) <= XSTRING (string)->size))
     args_out_of_range_3 (string, from, to);
 
-  XFASTINT (len) = XINT (to) - XINT (from);
-  val = Fmake_string (len, len);
-
-  bcopy (XSTRING (string)->data + XINT (from), XSTRING (val)->data, XINT (len));
-
-  return val;
+  return make_string (XSTRING (string)->data + XINT (from),
+		      XINT (to) - XINT (from));
 }
 
 DEFUN ("nthcdr", Fnthcdr, Snthcdr, 2, 2, 0,
@@ -440,7 +442,7 @@ DEFUN ("nthcdr", Fnthcdr, Snthcdr, 2, 2, 0,
   register int i, num;
   CHECK_NUMBER (n, 0);
   num = XINT (n);
-  for (i = 0; i < num; i++)
+  for (i = 0; i < num && !NULL (list); i++)
     {
       QUIT;
       list = Fcdr (list);
@@ -834,10 +836,8 @@ do_cdr:
 	return Qnil;
       return Qt;
     }
-#ifdef CB
   if (XTYPE(o1) == Lisp_Xresource && XTYPE(o2) == Lisp_Xresource)
       return EqualXresource(o1,o2);
-#endif
   return Qnil;
 }
 
@@ -895,7 +895,7 @@ Only the last argument is not altered, and need not be a list.")
      Lisp_Object *args;
 {
   register int argnum;
-  register Lisp_Object tail = Qnil, tem, val;
+  register Lisp_Object tail, tem, val;
 
   val = Qnil;
 
@@ -996,6 +996,7 @@ Thus, \" \" as SEP results in spaces between the values return by FN.")
   register Lisp_Object *args;
   register int i;
   int j;
+  struct gcpro gcpro1;
 
   len = Flength (seq);
   leni = XINT (len);
@@ -1004,7 +1005,9 @@ Thus, \" \" as SEP results in spaces between the values return by FN.")
 
   args = (Lisp_Object *) alloca (nargs * sizeof (Lisp_Object));
 
+  GCPRO1 (sep);
   mapcar1 (leni, args, fn, seq);
+  UNGCPRO;
 
   /* Broken Xenix/386 compiler can't use a register variable here */
   for (j = leni - 1; j > 0; j--)
@@ -1058,7 +1061,7 @@ Also accepts Space to mean yes, or Delete to mean no.")
     {
       message ("%s(y or n) ", XSTRING (xprompt)->data);
       cursor_in_echo_area = 1;
-      ans = get_char (0);
+      ans = read_command_char (0);
       cursor_in_echo_area = -1;
       message ("%s(y or n) %c", XSTRING (xprompt)->data, ans);
       cursor_in_echo_area = ocech;
@@ -1101,16 +1104,21 @@ The user must confirm the answer with a newline, and can rub it out if not confi
   args[1] = build_string ("(yes or no) ");
   prompt = Fconcat (2, args);
 
+  GCPRO1 (prompt);
   while (1)
     {
-      GCPRO1 (prompt);
       ans = Fdowncase (read_minibuf (Vminibuffer_local_map,
 				     Qnil, prompt, 0));
-      UNGCPRO;
       if (XSTRING (ans)->size == 3 && !strcmp (XSTRING (ans)->data, "yes"))
-	return Qt;
+	{
+	  UNGCPRO;
+	  return Qt;
+	}
       if (XSTRING (ans)->size == 2 && !strcmp (XSTRING (ans)->data, "no"))
-	return Qnil;
+	{
+	  UNGCPRO;
+	  return Qnil;
+	}
 
       Fding (Qnil);
       Fdiscard_input ();
@@ -1120,7 +1128,9 @@ The user must confirm the answer with a newline, and can rub it out if not confi
 }
 
 /* Avoid static vars inside a function since in HPUX they dump as pure.  */
-#ifndef DGUX
+#ifdef DGUX
+static struct dg_sys_info_load_info load_info;  /* what-a-mouthful! */
+#else
 static int ldav_initialized;
 static int ldav_channel;
 #ifdef LOAD_AVE_TYPE
@@ -1132,10 +1142,7 @@ static struct nlist ldav_nl[2];
 #define channel ldav_channel
 #define initialized ldav_initialized
 #define nl ldav_nl
-
-#else /* DGUX */
-static struct dg_sys_info_load_info load_info;
-#endif /* DGUX */
+#endif /* not DGUX */
 
 DEFUN ("load-average", Fload_average, Sload_average, 0, 0, 0,
   "Return the current 1 minute, 5 minute and 15 minute load averages\n\
@@ -1143,7 +1150,7 @@ in a list (all floating point load average values are multiplied by 100\n\
 and then turned into integers).")
   ()
 {
-#ifndef DGUX
+#ifndef DGUX  
 #ifndef LOAD_AVE_TYPE
   error ("load-average not implemented for this operating system");
 
@@ -1213,19 +1220,30 @@ and then turned into integers).")
       strcpy (nl[0].n_name, LDAV_SYMBOL);
       nl[1].n_zeroes = 0;
 #else /* NLIST_STRUCT */
-#if defined(convex) || defined (NeXT)
+#if defined(convex) || defined(NeXT)
       nl[0].n_un.n_name = LDAV_SYMBOL;
       nl[1].n_un.n_name = 0;
-#else /* not convex or NeXT */
+#else /* not convex */
       nl[0].n_name = LDAV_SYMBOL;
       nl[1].n_name = 0;
-#endif /* not convex */
+#endif /* not convex or next */
 #endif /* NLIST_STRUCT */
+
+#ifdef IRIS_4D
+	{
+#include <sys/types.h>
+#include <sys/sysmp.h>
+	    nl[0].n_value = sysmp(MP_KERNADDR, MPKA_AVENRUN);
+	    nl[0].n_value &= 0x7fffffff;
+	}
+#else
 #ifdef NeXT
       nlist (KERN_FILE, nl);
-#else
+#else      
       nlist (KERNEL_FILE, nl);
-#endif
+#endif /* NeXT */      
+#endif /* IRIS */
+
 #ifdef FIXUP_KERNEL_SYMBOL_ADDR
       if ((nl[0].n_type & N_TYPE) != N_ABS)
 	nl[0].n_value = (nlp->n_value >> 2) | 0xc0000000;
@@ -1269,21 +1287,19 @@ and then turned into integers).")
 			      Qnil)));
 #endif /* LOAD_AVE_TYPE */
 #else /* DGUX */
-  /*
-   * Return the list of load average values.
-   */
+  /* perhaps there should be a "sys_load_avg" call in sysdep.c?! - DJB */
   load_info.one_minute     = 0.0;	/* just in case there is an error */
   load_info.five_minute    = 0.0;
   load_info.fifteen_minute = 0.0;
-  (void) dg_sys_info( &load_info,
+  (void) dg_sys_info (&load_info,
 		      DG_SYS_INFO_LOAD_INFO_TYPE,
-		      DG_SYS_INFO_LOAD_VERSION_0 );
+		      DG_SYS_INFO_LOAD_VERSION_0);
 
-  return Fcons (make_number ((int)( load_info.one_minute * 100.0 )),
-	     Fcons (make_number ((int)( load_info.five_minute * 100.0 )),
-		Fcons (make_number ((int)( load_info.fifteen_minute * 100.0 )),
-		   Qnil)));
-#endif /* DG/UX */
+  return Fcons (make_number ((int)(load_info.one_minute * 100.0)),
+	Fcons (make_number ((int)(load_info.five_minute * 100.0)),
+	Fcons (make_number ((int)(load_info.fifteen_minute * 100.0)),
+ 		   Qnil)));
+#endif /* DGUX */
 }
 
 #undef channel
@@ -1333,12 +1349,23 @@ load FILENAME.  FILENAME is optional and defaults to FEATURE.")
   tem = Fmemq (feature, Vfeatures);
   if (NULL (tem))
     {
+      int count = specpdl_ptr - specpdl;
+
+      /* Value saved here is to be restored into Vautoload_queue */
+      record_unwind_protect (un_autoload, Vautoload_queue);
+      Vautoload_queue = Qt;
+
       Fload (NULL (file_name) ? Fsymbol_name (feature) : file_name,
 	     Qnil, Qt, Qnil);
+
       tem = Fmemq (feature, Vfeatures);
       if (NULL (tem))
 	error ("Required feature %s was not provided",
 	       XSYMBOL (feature)->name->data );
+
+      /* Once loading finishes, don't undo it.  */
+      Vautoload_queue = Qt;
+      unbind_to (count);
     }
   return feature;
 }
