@@ -3,21 +3,26 @@
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.6 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/lisp.h,v $
+ * $Date: 92/02/23 11:03:40 $
+ * $Author: love $
+ */
 
 /* Define the fundamental Lisp data structures */
 
@@ -45,6 +50,17 @@ enum Lisp_Type
     /* Cons.  object.v.cons points to a struct Lisp_Cons. */
     Lisp_Cons,
 
+    /* >>> No longer used */
+    Lisp_Object_Unused_1,
+#if 0
+    was...
+    /* Treated like vector in GC, except do not set its mark bit.
+       Used for internal data blocks that will be explicitly freed
+       but which, while active, are reached by GC mark exactly once
+       and should be marked through like a vector.  */
+    Lisp_Temp_Vector,
+#endif /* 0 */
+
     /* Editor buffer.  obj.v.buffer points to a struct buffer.
        No buffer is ever truly freed; they can be "killed", but this
        just marks them as dead. */
@@ -84,6 +100,9 @@ enum Lisp_Type
        obj.v.objfwd points to the Lisp_Object variable. */
     Lisp_Objfwd,
 
+      /* was Lisp_Internal */
+    Lisp_Object_Unused_2,
+
     /* Used when a FILE * value needs to be passed
        in an argument of type Lisp_Object.
        You must do (FILE *) obj.v.integer to get the value.
@@ -139,17 +158,19 @@ enum Lisp_Type
     /* Used by save,set,restore-window-configuration */
     Lisp_Window_Configuration,
 
-#ifdef HAVE_X_WINDOWS 
-    /* x-window control block structures [amc]*/
-    /* Root Block for storing an X-window and associated data */
-    Lisp_Root_Block
-    /* raw data block, which should not be handled by the GC */
-    , Lisp_Raw_Data
-    /* wrapper for X-window ID's */
-    , Lisp_Xresource
-    , Lisp_Button
-    , Lisp_Style
-#endif
+    /* Root Block for storing screen information. */
+    Lisp_Root_Block,
+
+    /* Raw data not handled by regular garbage collector */
+    Lisp_Raw_Data,
+
+    /* Wrapper for X ID's */
+    Lisp_Xresource,
+
+    /* Distinct types for button & styles */
+    Lisp_Button,
+    Lisp_Style
+       
   };
 
 #ifndef NO_UNION_TYPE
@@ -270,7 +291,7 @@ Lisp_Object;
 /* Extract the value of a Lisp_Object as a signed integer.  */
 
 #ifndef XINT   /* Some machines need to do this differently.  */
-#define XINT(a) (((a) << INTBITS-VALBITS) >> INTBITS-VALBITS)
+#define XINT(a) (((a) << (INTBITS-VALBITS)) >> (INTBITS-VALBITS))
 #endif
 
 /* Extract the value as an unsigned integer.  This is a basis
@@ -280,6 +301,11 @@ Lisp_Object;
 #define XUINT(a) ((a) & VALMASK)
 #endif
 
+#ifdef HAVE_SHM
+/* In this representation, data is found in two widely separated segments.  */
+#define XPNTR(a) \
+  (XUINT (a) | (XUINT (a) > PURESIZE ? DATA_SEG_BITS : PURE_SEG_BITS))
+#else /* not HAVE_SHM */
 #ifdef DATA_SEG_BITS
 /* This case is used for the rt-pc.
    In the diffs I was given, it checked for ptr = 0
@@ -287,9 +313,10 @@ Lisp_Object;
    But I don't think that zero should ever be found
    in a Lisp object whose data type says it points to something.  */
 #define XPNTR(a) (XUINT (a) | DATA_SEG_BITS)
-#else
+#else /* not DATA_SEG_BITS */
 #define XPNTR(a) XUINT (a)
 #endif
+#endif /* not HAVE_SHM */
 
 #ifndef XSETINT
 #define XSETINT(a, b)  ((a) = ((a) & ~VALMASK) |  ((b) & VALMASK))
@@ -394,6 +421,7 @@ Lisp_Object;
 #define XINTPTR(a) ((int *) XPNTR(a))
 #define XWINDOW(a) ((struct window *) XPNTR(a))
 #define XPROCESS(a) ((struct Lisp_Process *) XPNTR(a))
+
 #define XSETCONS(a, b) XSETPNTR(a, (int) (b))
 #define XSETBUFFER(a, b) XSETPNTR(a, (int) (b))
 #define XSETVECTOR(a, b) XSETPNTR(a, (int) (b))
@@ -466,7 +494,6 @@ struct Lisp_Marker
     struct buffer *buffer;
     Lisp_Object chain;
     int bufpos;
-    int modified;
   };
 
 /* Data type checking */
@@ -474,18 +501,19 @@ struct Lisp_Marker
 #ifdef NULL
 #undef NULL
 #endif
-
 #define NULL(x)  (XFASTINT (x) == XFASTINT (Qnil))
+/* NULL may get stomped on by other #includes (X11R5) */
+#define NIL(x)  (XFASTINT (x) == XFASTINT (Qnil))
 /* #define LISTP(x) (XTYPE ((x)) == Lisp_Cons)*/
 #define CONSP(x) (XTYPE ((x)) == Lisp_Cons)
+#define EQ(x, y) (XFASTINT (x) == XFASTINT (y))
 #define VECTORP(x) (XTYPE ((x)) == Lisp_Vector)
 #define STRINGP(x) (XTYPE ((x)) == Lisp_String)
 #define INTEGERP(x) (XTYPE ((x)) == Lisp_Int)
 #define SYMBOLP(x) (XTYPE ((x)) == Lisp_Symbol)
-#define EQ(x, y) (XFASTINT (x) == XFASTINT (y))
 
 #define CHECK_LIST(x, i) \
-  { if ((XTYPE ((x)) != Lisp_Cons) && !NULL (x)) x = wrong_type_argument (Qlistp, (x)); }
+  { if ((XTYPE ((x)) != Lisp_Cons) && !NIL (x)) x = wrong_type_argument (Qlistp, (x)); }
 
 #define CHECK_STRING(x, i) \
   { if (XTYPE ((x)) != Lisp_String) x = wrong_type_argument (Qstringp, (x)); }
@@ -600,34 +628,12 @@ struct Lisp_Marker
   struct Lisp_Subr sname = {fnname, minargs, maxargs, lname, prompt, 0}; \
   Lisp_Object fnname
 
-#ifdef DGUX
-/* Special hack for Fthrow which is declared volatile when GCC -W
-   warnings are enabled.  */
-#define DEFUN2(lname, fnname, sname, minargs, maxargs, prompt, doc) \
-  Lisp_Object NO_RETURN fnname (); \
-  struct Lisp_Subr sname = {(Lisp_Object (*)())fnname, minargs, maxargs, lname, prompt, 0}; \
-  Lisp_Object fnname
-
-#endif /* DGUX */
-  
 /* defsubr (Sname);
  is how we define the symbol for function `name' at start-up time. */
 extern void defsubr ();
 
 #define MANY -2
 #define UNEVALLED -1
-#define MAX_MAXARGS 6
-
-#define DEFSIMPLE(lname, fnname, sname, doc, valtype, setcomp, exp) \
-  DEFUN (lname, fnname, sname, 0, 0, 0, 0) () \
-  { \
-    Lisp_Object val; \
-    XSET (val, valtype, exp); \
-    return val; }
-
-#define DEFPRED(lname, fnname, sname, doc, boolexp) \
-  DEFUN (lname, fnname, sname, 0, 0, 0, 0) () \
-  { if (boolexp) return Qt; return Qnil; }
 
 /* Macros we use to define forwarded Lisp variables.
    These are used in the syms_of_FILENAME functions.  */
@@ -636,14 +642,14 @@ extern void defsubr ();
 #define DEFVARBOOL(lname, vname, doc) defvar_bool (lname, vname)
 #define DEFVARINT(lname, vname, doc) defvar_int (lname, vname)
 #define DEFVARPERBUFFER(lname, vname, doc)  \
- defvar_per_buffer (lname, vname)
+ defvar_per_buffer (lname, vname, 0)
 
 #define DEFVAR_LISP(lname, vname, doc) defvar_lisp (lname, vname)
 #define DEFVAR_LISP_NOPRO(lname, vname, doc) defvar_lisp_nopro (lname, vname)
 #define DEFVAR_BOOL(lname, vname, doc) defvar_bool (lname, vname)
 #define DEFVAR_INT(lname, vname, doc) defvar_int (lname, vname)
 #define DEFVAR_PER_BUFFER(lname, vname, doc)  \
- defvar_per_buffer (lname, vname)
+ defvar_per_buffer (lname, vname, 0)
 
 /* Structure for recording Lisp call stack for backtrace purposes */
 
@@ -662,6 +668,8 @@ struct handler
   {
     Lisp_Object handler;
     Lisp_Object var;
+    int poll_suppress_count;	/* No error should exit a piece of code
+				   in which polling is suppressed.  */
     struct catchtag *tag;
     struct handler *next;
   };
@@ -671,12 +679,12 @@ extern struct handler *handlerlist;
 /* Check quit-flag and quit if it is non-nil. */
 
 #define QUIT \
-  if (!NULL (Vquit_flag) && NULL (Vinhibit_quit)) \
+  if (!NIL (Vquit_flag) && NIL (Vinhibit_quit)) \
     { Vquit_flag = Qnil; Fsignal (Qquit, Qnil); }
 
 /* Nonzero if ought to quit now.  */
 
-#define QUITP (!NULL (Vquit_flag) && NULL (Vinhibit_quit))
+#define QUITP (!NIL (Vquit_flag) && NIL (Vinhibit_quit))
 
 /* 1 if CH is upper case.  */
 
@@ -711,6 +719,10 @@ extern int consing_since_gc;
 
 extern int gc_cons_threshold;
 
+/* value of consing_since_gc when undos were last truncated.  */
+
+extern int consing_at_last_truncate;
+
 /* Structure for recording stack slots that need marking */
 
 /* This is a chain of structures, each of which points at a Lisp_Object variable
@@ -759,18 +771,6 @@ struct gcpro
 void staticpro();
   
 #define UNGCPRO (gcprolist = gcpro1.next)
-
-#if defined(DGUX)
-
-#if defined(__GNUC__) && defined(__STDC__)
-#define NO_RETURN volatile
-#endif
-
-#ifndef NO_RETURN
-#define NO_RETURN
-#endif
-
-#endif
 
 /* Defined in data.c */
 extern Lisp_Object Qnil, Qt, Qquote, Qlambda, Qsubr, Qunbound;
@@ -821,7 +821,7 @@ extern Lisp_Object Vfeatures;
 extern Lisp_Object Fidentity (), Frandom ();
 extern Lisp_Object Flength ();
 extern Lisp_Object Fappend (), Fconcat (), Fvconcat (), Fcopy_sequence ();
-extern Lisp_Object Fsubstring (), Fcopy_alist();
+extern Lisp_Object Fsubstring ();
 extern Lisp_Object Fnthcdr (), Fmemq (), Fassq (), Fassoc ();
 extern Lisp_Object Frassq (), Fdelq (), Fsort ();
 extern Lisp_Object Freverse (), Fnreverse (), Fget (), Fput (), Fequal ();
@@ -858,19 +858,14 @@ extern Lisp_Object intern (), oblookup ();
 
 /* Defined in eval.c */
 extern Lisp_Object Qautoload, Qexit, Qinteractive, Qcommandp, Qdefun, Qmacro;
-extern Lisp_Object Vinhibit_quit, Vquit_flag;
+extern Lisp_Object Vinhibit_quit, Vquit_flag, Qinhibit_quit;
 extern Lisp_Object Vmocklisp_arguments, Qmocklisp, Qmocklisp_arguments;
 extern Lisp_Object Vautoload_queue;
 extern Lisp_Object Fand (), For (), Fif (), Fprogn (), Fprog1 (), Fprog2 ();
-extern Lisp_Object Fsetq (), Fquote (), Fcond ();
+extern Lisp_Object Fsetq (), Fquote ();
 extern Lisp_Object Fuser_variable_p (), Finteractive_p ();
 extern Lisp_Object Fdefun (), Flet (), FletX (), Fwhile ();
-#ifdef DGUX
-extern Lisp_Object Fcatch (), Funwind_protect ();
-extern Lisp_Object NO_RETURN Fthrow();
-#else
 extern Lisp_Object Fcatch (), Fthrow (), Funwind_protect ();
-#endif
 extern Lisp_Object Fcondition_case (), Fsignal ();
 extern Lisp_Object Ffunction_type (), Fautoload (), Fcommandp ();
 extern Lisp_Object Feval (), Fapply (), Ffuncall ();
@@ -880,11 +875,8 @@ extern Lisp_Object apply_lambda ();
 extern Lisp_Object internal_catch ();
 extern Lisp_Object internal_condition_case ();
 extern void unbind_to ();
-#ifdef DGUX
-extern void NO_RETURN error ();
-#else
 extern void error ();
-#endif
+extern Lisp_Object un_autoload ();
 
 /* Defined in editfns.c */
 extern Lisp_Object Vprefix_arg, Qminus, Vcurrent_prefix_arg;
@@ -912,7 +904,7 @@ extern struct buffer *all_buffers;
 /* defined in marker.c */
 
 extern Lisp_Object Fmarker_position (), Fmarker_buffer ();
-extern Lisp_Object Fcopy_marker (), Fset_marker ();
+extern Lisp_Object Fcopy_marker ();
 
 /* Defined in fileio.c */
 
@@ -921,11 +913,6 @@ extern Lisp_Object Ffile_name_as_directory ();
 extern Lisp_Object Fexpand_file_name (), Ffile_name_nondirectory ();
 extern Lisp_Object Fsubstitute_in_file_name ();
 extern Lisp_Object Ffile_symlink_p ();
-extern Lisp_Object Ffile_exists_p ();
-extern Lisp_Object Fverify_visited_file_modtime ();
-extern Lisp_Object Fdirectory_file_name ();
-extern Lisp_Object Ffile_name_directory ();
-extern Lisp_Object Fdo_auto_save ();
 
 /* Defined in abbrev.c */
 
@@ -938,7 +925,7 @@ extern Lisp_Object Fscan_buffer ();
 
 /* defined in minibuf.c */
 
-extern Lisp_Object last_minibuf_string;
+extern Lisp_Object last_minibuf_string, Vminibuffer_list;
 extern Lisp_Object read_minibuf (), Fcompleting_read ();
 extern Lisp_Object Fread_from_minibuffer ();
 extern Lisp_Object Fread_variable ();
@@ -962,7 +949,7 @@ extern Lisp_Object Fdowncase (), Fupcase (), Fcapitalize ();
 extern Lisp_Object Vhelp_form, Vtop_level;
 extern Lisp_Object Fdiscard_input (), Frecursive_edit ();
 extern Lisp_Object Fcommand_execute (), Finput_pending_p ();
-extern Lisp_Object Fset_input_mode();
+extern int poll_suppress_count;
 
 /* defined in keymap.c */
 
@@ -970,7 +957,7 @@ extern Lisp_Object Qkeymap;
 extern Lisp_Object Fkey_description (), Fsingle_key_description ();
 extern Lisp_Object Fwhere_is_internal ();
 extern Lisp_Object access_keymap (), store_in_keymap ();
-extern Lisp_Object get_keyelt (), get_keymap(), get_keymap_1 ();
+extern Lisp_Object get_keyelt (), get_keymap();
 
 /* defined in indent.c */
 extern Lisp_Object Fvertical_motion (), Findent_to (), Fcurrent_column ();
@@ -980,11 +967,9 @@ extern Lisp_Object Qwindowp;
 extern Lisp_Object Fget_buffer_window ();
 extern Lisp_Object Fsave_window_excursion ();
 extern Lisp_Object Fset_window_configuration (), Fcurrent_window_configuration ();
-extern Lisp_Object Ffind_window();
 
 /* defined in emacs.c */
 extern Lisp_Object decode_env_path ();
-extern Lisp_Object Fkill_emacs ();
 /* Nonzero means don't do interactive redisplay and don't change tty modes */
 extern int noninteractive;
 /* Nonzero means don't do use window-system-specific display code */
@@ -1024,33 +1009,11 @@ extern int immediate_quit;	    /* Nonzero means ^G can quit instantly */
 
 extern void debugger ();
 
-#ifdef __STDC__
-extern void *malloc (), *realloc ();
-extern void free();
-#else
 extern char *malloc (), *realloc (), *getenv (), *ctime (), *getwd ();
 extern long *xmalloc (), *xrealloc ();
-#endif
 
 #ifdef MAINTAIN_ENVIRONMENT
 extern unsigned char *egetenv ();
 #else
 #define egetenv getenv
-extern char * getenv();
-#endif
-
-extern void init_obarray ();
-extern void defsubr ();
-extern void defvar_int ();
-extern void defvar_bool ();
-extern void defvar_lisp ();
-extern void defvar_lisp_nopro ();
-extern void defkey ();
-extern void ndefkey ();
-#ifndef standalone
-extern void defvar_per_buffer ();
-#endif
-#ifdef NOTDEF
-extern void defalias ();
-extern void synkey ();
 #endif
