@@ -3,21 +3,29 @@
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.6 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/lread.c,v $
+ * $Date: 92/02/23 11:00:57 $ 
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/02/23 11:00:57 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/lread.c,v $ $Revision: 1.6 $";
+#endif
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -58,8 +66,6 @@ static int read_pure;
 /* For use within read-from-string (this reader is non-reentrant!!) */
 static int read_from_string_index;
 static int read_from_string_limit;
-
-static int read_escape();
 
 /* Handle unreading and rereading of characters.
  Write READCHAR to read a character, UNREAD(c) to unread c to be read again. */
@@ -70,7 +76,7 @@ static int readchar (readcharfun)
      Lisp_Object readcharfun;
 {
   Lisp_Object tem;
-  register struct buffer_text *inbuffer;
+  register struct buffer *inbuffer;
   register int c, mpos;
 
   if (unrch >= 0)
@@ -81,31 +87,23 @@ static int readchar (readcharfun)
     }
   if (XTYPE (readcharfun) == Lisp_Buffer)
     {
-      if (XBUFFER (readcharfun) == bf_cur)
-	inbuffer = &bf_text;
-      else
-	inbuffer = &XBUFFER (readcharfun)->text;
+      inbuffer = XBUFFER (readcharfun);
 
-      if (inbuffer->pointloc >
-	  inbuffer->size1 + inbuffer->size2 - inbuffer->tail_clip)
+      if (BUF_PT (inbuffer) >= BUF_ZV (inbuffer))
 	return -1;
-      c = *(unsigned char *) &(inbuffer->pointloc > inbuffer->size1 ? inbuffer->p2 : inbuffer->p1)[inbuffer->pointloc];
-      inbuffer->pointloc++;
+      c = *(unsigned char *) BUF_CHAR_ADDRESS (inbuffer, BUF_PT (inbuffer));
+      SET_BUF_PT (inbuffer, BUF_PT (inbuffer) + 1);
       return c;
     }
   if (XTYPE (readcharfun) == Lisp_Marker)
     {
-      if (XMARKER (readcharfun)->buffer == bf_cur)
-	inbuffer = &bf_text;
-      else
-	inbuffer = &XMARKER (readcharfun)->buffer->text;
+      inbuffer = XMARKER (readcharfun)->buffer;
       mpos = marker_position (readcharfun);
 
-      if (mpos >
-	  inbuffer->size1 + inbuffer->size2 - inbuffer->tail_clip)
+      if (mpos > BUF_ZV (inbuffer) - 1)
 	return -1;
-      c = *(unsigned char *) &(mpos > inbuffer->size1 ? inbuffer->p2 : inbuffer->p1)[mpos];
-      if (mpos != inbuffer->size1 + 1)
+      c = *(unsigned char *) BUF_CHAR_ADDRESS (inbuffer, mpos);
+      if (mpos != BUF_GPT (inbuffer))
 	XMARKER (readcharfun)->bufpos++;
       else
 	Fset_marker (readcharfun, make_number (mpos + 1),
@@ -138,6 +136,7 @@ static int readchar (readcharfun)
 #define UNREAD(c) (unrch = c)
 
 static Lisp_Object read0 (), read1 (), read_list (), read_vector ();
+static int read_escape ();
 
 /* get a character from the tty */
 
@@ -149,7 +148,7 @@ It is returned as a number.")
   register Lisp_Object val;
 
 #ifndef standalone
-  XSET (val, Lisp_Int, get_char (0));
+  XSET (val, Lisp_Int, read_command_char (0));
 #else
   XSET (val, Lisp_Int, getchar ());
 #endif
@@ -173,15 +172,15 @@ DEFUN ("load", Fload, Sload, 1, 4, 0,
   "Execute a file of Lisp code named FILE.\n\
 First tries FILE with .elc appended, then tries with .el,\n\
  then tries FILE unmodified.  Searches directories in  load-path.\n\
-If optional second arg MISSING-OK is non-nil,\n\
+If optional second arg NOERROR is non-nil,\n\
  report no error if FILE doesn't exist.\n\
 Print messages at start and end of loading unless\n\
  optional third arg NOMESSAGE is non-nil.\n\
 If optional fourth arg NOSUFFIX is non-nil, don't try adding\n\
  suffixes .elc or .el to the specified name FILE.\n\
 Return t if file exists.")
-  (str, missing_ok, nomessage, nosuffix)
-     Lisp_Object str, missing_ok, nomessage, nosuffix;
+  (str, noerror, nomessage, nosuffix)
+     Lisp_Object str, noerror, nomessage, nosuffix;
 {
   register FILE *stream;
   register int fd = -1;
@@ -201,7 +200,7 @@ Return t if file exists.")
     }
 
   if (fd < 0)
-    if (NULL (missing_ok))
+    if (NULL (noerror))
       while (1)
 	Fsignal (Qfile_error, Fcons (build_string ("Cannot open load file"),
 				     Fcons (str, Qnil)));
@@ -237,6 +236,7 @@ load_unwind (stream)  /* used as unwind-protect function in load */
      Lisp_Object stream;
 {
   fclose (*(FILE **) XSTRING (stream));
+  free (XPNTR (stream));
   if (--load_in_progress < 0) load_in_progress = 0;
   return Qnil;
 }
@@ -299,7 +299,7 @@ openp (path, str, suffix, storeptr, exec_only)
 	/* Of course, this could conceivably lose if luser sets
 	   default-directory to be something non-absolute... */
 	{
-	  filename = Fexpand_file_name (filename, bf_cur->directory);
+	  filename = Fexpand_file_name (filename, current_buffer->directory);
 	  if (!absolute_filename_p (filename))
 	    /* Give up on this path element! */
 	    continue;
@@ -331,7 +331,7 @@ openp (path, str, suffix, storeptr, exec_only)
 	    {
 	      /* Check that we can access or open it.  */
 	      if (exec_only)
-		fd = !access (fn, X_OK);
+		fd = !access (fn, X_OK) ? 1 : -1;
 	      else
 		fd = open (fn, 0, 0);
 
@@ -437,7 +437,7 @@ nil means discard it; anything else is stream for print.")
     tem = printflag;
   specbind (Qstandard_output, tem);
   record_unwind_protect (save_excursion_restore, save_excursion_save ());
-  SetPoint (FirstCharacter);
+  SET_PT (BEGV);
   readevalloop (Fcurrent_buffer (), 0, Feval, !NULL (printflag));
   unbind_to (count);
   return Qnil;
@@ -455,6 +455,7 @@ nil means discard it; anything else is stream for print.")
 {
   int count = specpdl_ptr - specpdl;
   Lisp_Object tem;
+
   if (NULL (printflag))
     tem = Qsymbolp;
   else
@@ -463,14 +464,15 @@ nil means discard it; anything else is stream for print.")
   if (NULL (printflag))
     record_unwind_protect (save_excursion_restore, save_excursion_save ());
   record_unwind_protect (save_restriction_restore, save_restriction_save ());
-  SetPoint (XINT (b));
-  Fnarrow_to_region (make_number (FirstCharacter), e);
+  /* This both uses b and checks its type.  */
+  Fgoto_char (b);
+  Fnarrow_to_region (make_number (BEGV), e);
   readevalloop (Fcurrent_buffer (), 0, Feval, !NULL (printflag));
   unbind_to (count);
   return Qnil;
 }
 
-#endif /* standalone */
+#endif standalone
 
 DEFUN ("read", Fread, Sread, 0, 1, 0,
   "Read one Lisp expression as text from STREAM, return as Lisp object.\n\
@@ -660,8 +662,10 @@ read1 (readcharfun)
       }
 
     default:
-      if (c <= 040) goto retry;
+      if (c <= 040)
+	goto retry;
       {
+	int quoted = 0;
 	register char *p = read_buffer;
 
 	{
@@ -681,7 +685,10 @@ read1 (readcharfun)
 		  end = read_buffer + read_buffer_size;
 		}
 	      if (c == '\\')
-		c = READCHAR;
+		{
+		  quoted = 1;
+		  c = READCHAR;
+		}
 	      *p++ = c;
 	      c = READCHAR;
 	    }
@@ -703,7 +710,7 @@ read1 (readcharfun)
 	  register Lisp_Object val;
 	  p1 = read_buffer;
 	  if (*p1 == '+' || *p1 == '-') p1++;
-	  if (p1 != p)
+	  if (p1 != p && !quoted)
 	    {
 	      while (p1 != p && (c = *p1) >= '0' && c <= '9') p1++;
 	      if (p1 == p)
@@ -821,7 +828,7 @@ read_escape (readcharfun)
   switch (c)
     {
     case 'a':
-      return '\a';
+      return 007;
     case 'b':
       return '\b';
     case 'e':
@@ -1047,7 +1054,7 @@ map_obarray (obarray, fn, arg)
   for (i = XVECTOR (obarray)->size - 1; i >= 0; i--)
     {
       tail = XVECTOR (obarray)->contents[i];
-      if (XFASTINT (tail) != 0)
+      if (XFASTINT (tail) != 0 && XTYPE (tail) == Lisp_Symbol)
 	while (1)
 	  {
 	    (*fn) (tail, arg);
@@ -1079,9 +1086,7 @@ OBARRAY defaults to the value of  obarray.")
   return Qnil;
 }
 
-#define OBARRAY_SIZE 2053
-/* [amc] Previous value was 511, which is not even prime! (7*73) */
-/* 2053 is prime */
+#define OBARRAY_SIZE 511
 
 void
 init_obarray ()
@@ -1095,7 +1100,6 @@ init_obarray ()
   Qnil = Fmake_symbol (make_pure_string ("nil", 3));
   Vobarray = Fmake_vector (oblength, make_number (0));
   initial_obarray = Vobarray;
-  staticpro (&Vobarray);
   staticpro (&initial_obarray);
   /* Intern nil in the obarray */
   /* These locals are to kludge around a pyramid compiler bug. */
@@ -1143,7 +1147,7 @@ defalias (sname, string)
   sym = intern (string);
   XSET (XSYMBOL (sym)->function, Lisp_Subr, sname);
 }
-#endif /* NOTDEF */
+#endif NOTDEF
 
 /* New replacement for DefIntVar; it ignores the doc string argument
    on the assumption that make-docfile will handle that.  */
@@ -1221,7 +1225,7 @@ defvar_per_buffer (namestring, address, doc)
   extern struct buffer buffer_local_symbols;
 
   sym = intern (namestring);
-  offset = (char *)address - (char *)bf_cur;
+  offset = (char *)address - (char *)current_buffer;
 
   XSET (XSYMBOL (sym)->value, Lisp_Buffer_Objfwd,
 	(Lisp_Object *) offset);
@@ -1232,15 +1236,30 @@ defvar_per_buffer (namestring, address, doc)
     abort ();
 }
 
-#endif /* standalone */
+#endif standalone
 
 init_read ()
 {
   char *normal = PATH_LOADSEARCH;
   Lisp_Object normal_path;
 
+  /* Compute the default load-path.  */
+#ifndef CANNOT_DUMP
+  /* If running a dumped Emacs in which load-path was set before dumping
+     to a nonstandard value, use that value.  */
+  if (initialized
+      && !(XTYPE (Vload_path) == Lisp_Cons
+         && XTYPE (XCONS (Vload_path)->car) == Lisp_String
+         && !strcmp (XSTRING (XCONS (Vload_path)->car)->data,
+		     "../epoch-lisp")))
+    normal_path = Vload_path;
+  else
+#endif
+    normal_path = decode_env_path (0, normal);
+
+  Vload_path = normal_path;
+
   /* Warn if dirs in the *standard* path don't exist.  */
-  normal_path = decode_env_path ("", normal);
   for (; !NULL (normal_path); normal_path = XCONS (normal_path)->cdr)
     {
       Lisp_Object dirfile;
@@ -1254,15 +1273,18 @@ init_read ()
 	}
     }
 
-  Vvalues = Qnil;
-
-  Vload_path = decode_env_path ("EMACSLOADPATH", normal);
+  if (egetenv ("EMACSLOADPATH"))
+    Vload_path = decode_env_path ("EMACSLOADPATH", normal);
 #ifndef CANNOT_DUMP
   if (!NULL (Vpurify_flag))
-    Vload_path = Fcons (build_string ("../epoch-lisp"),
-			Fcons(build_string ("../lisp"),
-			      Vload_path));
+    {
+      Vload_path = Fcons (build_string ("../lisp"), Vload_path);
+      Vload_path = Fcons (build_string ("../epoch-lisp"), Vload_path);
+    }
 #endif /* not CANNOT_DUMP */
+
+  Vvalues = Qnil;
+  
   load_in_progress = 0;
 }
 
