@@ -1,23 +1,33 @@
 /* Minibuffer input and completion.
-   Copyright (C) 1985, 1986 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1990 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.6 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/minibuf.c,v $
+ * $Date: 92/03/22 15:45:26 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/03/22 15:45:26 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/minibuf.c,v $ $Revision: 1.6 $";
+#endif
+
+#include <signal.h>
 
 #include "config.h"
 #include "lisp.h"
@@ -25,8 +35,12 @@ and this notice must be preserved on all copies.  */
 #include "buffer.h"
 #include "window.h"
 #include "syntax.h"
-#include "dispextern.h"
-#include "screenW.h"
+#include "x11term.h"		/* Epoch */
+#include "dispepoch.h"		/* Epoch */
+#include "screen.h"		/* Epoch */
+#include "screenW.h"		/* Epoch */
+
+/* X11 includes used; use NIL rather than NULL from lisp.h */
 #define min(a, b) ((a) < (b) ? (a) : (b))
 
 /* List of buffers for use as minibuffers.
@@ -46,27 +60,28 @@ struct minibuf_save_data
 int minibuf_save_vector_size;
 struct minibuf_save_data *minibuf_save_vector;
 
-int auto_help;		/* Nonzero means display completion help for invalid input */
+/* Depth in minibuffer invocations.  */
+int minibuf_level;
 
-/* Fread_minibuffer leaves the input, as a string, here */
+/* Nonzero means display completion help for invalid input.  */
+int completion_auto_help;
+
+/* Fread_minibuffer leaves the input, as a string, here.  */
 Lisp_Object last_minibuf_string;
 
 /* Nonzero means let functions called when within a minibuffer 
-   invoke recursive minibuffers (to read arguments, or whatever) */
+   invoke recursive minibuffers (to read arguments, or whatever).  */
 int enable_recursive_minibuffers;
 
 /* help-form is bound to this while in the minibuffer.  */
-
 Lisp_Object Vminibuffer_help_form;
 
 /* Nonzero means completion ignores case.  */
-
 int completion_ignore_case;
 
 Lisp_Object Quser_variable_p;
 
 /* Width in columns of current minibuffer prompt.  */
-
 extern int minibuf_prompt_width;
 
 /* Actual minibuffer invocation. */
@@ -76,15 +91,15 @@ Lisp_Object get_minibuffer ();
 Lisp_Object read_minibuf ();
 
 Lisp_Object
-read_minibuf (map, prefix, prompt, expflag)
+read_minibuf (map, initial, prompt, expflag)
      Lisp_Object map;
-     Lisp_Object prefix;
+     Lisp_Object initial;
      Lisp_Object prompt;
      int expflag;
 {
   register Lisp_Object val;
   int count = specpdl_ptr - specpdl;
-  struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
+  struct gcpro gcpro1, gcpro2;
 
   if (XTYPE (prompt) != Lisp_String)
     prompt = build_string ("");
@@ -97,62 +112,59 @@ read_minibuf (map, prefix, prompt, expflag)
       (EQ (selected_window, minibuf_window)))
     error ("Command attempted to use minibuffer while in minibuffer");
 
-  if (MinibufDepth == minibuf_save_vector_size)
+  if (minibuf_level == minibuf_save_vector_size)
     minibuf_save_vector =
      (struct minibuf_save_data *) xrealloc (minibuf_save_vector,
 		      (minibuf_save_vector_size *= 2) * sizeof (struct minibuf_save_data)); 
-  minibuf_save_vector[MinibufDepth].prompt = minibuf_prompt;
-  minibuf_save_vector[MinibufDepth].prompt_width = minibuf_prompt_width;
+  minibuf_save_vector[minibuf_level].prompt = minibuf_prompt;
+  minibuf_save_vector[minibuf_level].prompt_width = minibuf_prompt_width;
   minibuf_prompt_width = 0;
   /* >> Why is this done this way rather than binding these variables? */
-  minibuf_save_vector[MinibufDepth].help_form = Vhelp_form;
-  minibuf_save_vector[MinibufDepth].current_prefix_arg = Vcurrent_prefix_arg;
-  GCPRO4 (minibuf_save_vector[MinibufDepth].help_form,
-	  minibuf_save_vector[MinibufDepth].current_prefix_arg,
-	  prompt, prefix);		/* set-window-buffer can cause GC */
+  minibuf_save_vector[minibuf_level].help_form = Vhelp_form;
+  minibuf_save_vector[minibuf_level].current_prefix_arg = Vcurrent_prefix_arg;
+  GCPRO2 (minibuf_save_vector[minibuf_level].help_form,
+	  minibuf_save_vector[minibuf_level].current_prefix_arg);
 
 
   record_unwind_protect (Fset_window_configuration,
 			 Fcurrent_window_configuration ());
 
-  val = bf_cur->directory;
-  Fset_buffer (get_minibuffer (MinibufDepth));
-  bf_cur->directory = val;
+  val = current_buffer->directory;
+  Fset_buffer (get_minibuffer (minibuf_level));
+  current_buffer->directory = val;
 
   Fset_window_buffer (minibuf_window, Fcurrent_buffer ());
   Fselect_window (minibuf_window);
   XFASTINT (XWINDOW (minibuf_window)->hscroll) = 0;
 
   Ferase_buffer ();
-  MinibufDepth++;
+  minibuf_level++;
   record_unwind_protect (read_minibuf_unwind, Qnil);
   Vminibuf_scroll_window = Qnil;
 
-  if (!NULL (prefix))
-    Finsert (1, &prefix);
+  if (!NIL (initial))
+    Finsert (1, &initial);
 
   minibuf_prompt = (char *) alloca (XSTRING (prompt)->size + 1);
   bcopy (XSTRING (prompt)->data, minibuf_prompt, XSTRING (prompt)->size + 1);
-  minibuf_message = 0;
+  echo_area_contents = 0;
 
   Vhelp_form = Vminibuffer_help_form;
-  bf_cur->keymap = map;
-  Frecursive_edit ();
+  current_buffer->keymap = map;
+  recursive_edit_1 ();
 
   /* If cursor is on the minibuffer line,
      show the user we have exited by putting it in column 0.  */
-  if (WS->cursor_y >= XFASTINT (XWINDOW (minibuf_window)->top)
+  if (WS->cursor_y >= XFASTINT (XWINDOW (minibuf_window)->pixtop)
       && !noninteractive)
     {
       WS->cursor_x = 0;
-      update_screen(1,1);
+      update_window(XWINDOW(minibuf_window));
     }
 
   /* Make minibuffer contents into a string */
-  val = make_string (bf_p1 + 1, bf_s1 + bf_s2);
-  bcopy (bf_p2 + bf_s1 + 1,
-	 XSTRING (val)->data + bf_s1,
-	 bf_s2);
+  val = make_string (BEG_ADDR, Z - BEG);
+  bcopy (GAP_END_ADDR, XSTRING (val)->data + GPT - BEG, Z - GPT);
   unbind_to (count);
   UNGCPRO;
 
@@ -181,13 +193,13 @@ get_minibuffer (depth)
 
   XFASTINT (num) = depth;
   tail = Fnthcdr (num, Vminibuffer_list);
-  if (NULL (tail))
+  if (NIL (tail))
     {
       tail = Fcons (Qnil, Qnil);
       Vminibuffer_list = nconc2 (Vminibuffer_list, tail);
     }
   buf = Fcar (tail);
-  if (NULL (buf) || NULL (XBUFFER (buf)->name))
+  if (NIL (buf) || NIL (XBUFFER (buf)->name))
     {
       sprintf (name, " *Minibuf-%d*", depth);
       buf = Fget_buffer_create (build_string (name));
@@ -207,19 +219,21 @@ read_minibuf_unwind ()
   /* Erase the minibuffer we were using at this level.  */
   Fset_buffer (XWINDOW (minibuf_window)->buffer);
   Ferase_buffer ();
+  /* Prevent error if user has done something strange.  */
+  current_buffer->read_only = Qnil;
 
   /* If this was a recursive minibuffer,
      tie the minibuffer window back to the outer level minibuffer buffer */
-  MinibufDepth--;
+  minibuf_level--;
   /* Make sure minibuffer window is erased, not ignored */
   windows_or_buffers_changed++;
   XFASTINT (XWINDOW (minibuf_window)->last_modified) = 0;
 
   /* Restore prompt from outer minibuffer */
-  minibuf_prompt = minibuf_save_vector[MinibufDepth].prompt;
-  minibuf_prompt_width = minibuf_save_vector[MinibufDepth].prompt_width;
-  Vhelp_form = minibuf_save_vector[MinibufDepth].help_form;
-  Vcurrent_prefix_arg = minibuf_save_vector[MinibufDepth].current_prefix_arg;
+  minibuf_prompt = minibuf_save_vector[minibuf_level].prompt;
+  minibuf_prompt_width = minibuf_save_vector[minibuf_level].prompt_width;
+  Vhelp_form = minibuf_save_vector[minibuf_level].help_form;
+  Vcurrent_prefix_arg = minibuf_save_vector[minibuf_level].current_prefix_arg;
 }
 
 DEFUN ("read-from-minibuffer", Fread_from_minibuffer, Sread_from_minibuffer, 1, 4, 0,
@@ -234,13 +248,13 @@ If fourth arg READ is non-nil, then interpret the result as a lisp object\n\
      Lisp_Object prompt, initial_input, keymap, read;
 {
   CHECK_STRING (prompt, 0);
-  if (!NULL (initial_input))
+  if (!NIL (initial_input))
     CHECK_STRING (initial_input, 1);
-  if (NULL (keymap))
+  if (NIL (keymap))
     keymap = Vminibuffer_local_map;
   else
     keymap = get_keymap (keymap,2);
-  return read_minibuf (keymap, initial_input, prompt, !NULL(read));
+  return read_minibuf (keymap, initial_input, prompt, !NIL(read));
 }
 
 DEFUN ("read-minibuffer", Fread_minibuffer, Sread_minibuffer, 1, 2, 0,
@@ -251,7 +265,7 @@ is a string to insert in the minibuffer before reading.")
      Lisp_Object prompt, initial_contents;
 {
   CHECK_STRING (prompt, 0);
-  if (!NULL (initial_contents))
+  if (!NIL (initial_contents))
     CHECK_STRING (initial_contents, 1)
   return read_minibuf (Vminibuffer_local_map, initial_contents, prompt, 1);
 }
@@ -337,7 +351,7 @@ If optional third arg REQUIRE-MATCH is non-nil, only existing buffer names are a
 
   if (XTYPE (def) == Lisp_Buffer)
     def = XBUFFER (def)->name;
-  if (!NULL (def))
+  if (!NIL (def))
     {
       args[0] = build_string ("%s(default %s) ");
       args[1] = prompt;
@@ -370,13 +384,13 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
   (string, alist, pred)
      Lisp_Object string, alist, pred;
 {
-  Lisp_Object bestmatch, tail = Qnil, elt, eltstring;
-  int bestmatchsize = 0;
+  Lisp_Object bestmatch, tail, elt, eltstring;
+  int bestmatchsize;
   int compare, matchsize;
-  int list = CONSP (alist);
-  int index = 0, obsize = 0;
+  int list = CONSP (alist) || NIL (alist);
+  int index, obsize;
   int matchcount = 0;
-  Lisp_Object bucket = Qnil, zero, end, tem;
+  Lisp_Object bucket, zero, end, tem;
   struct gcpro gcpro1, gcpro2, gcpro3;
 
   CHECK_STRING (string, 0);
@@ -403,7 +417,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 
       if (list)
 	{
-	  if (NULL (tail))
+	  if (NIL (tail))
 	    break;
 	  elt = Fcar (tail);
 	  eltstring = Fcar (elt);
@@ -440,7 +454,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	  /* Ignore this element if there is a predicate
 	     and the predicate doesn't like it. */
 
-	  if (!NULL (pred))
+	  if (!NIL (pred))
 	    {
 	      if (EQ (pred, Qcommandp))
 		tem = Fcommandp (elt);
@@ -450,13 +464,13 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 		  tem = call1 (pred, elt);
 		  UNGCPRO;
 		}
-	      if (NULL (tem)) continue;
+	      if (NIL (tem)) continue;
 	    }
 
 	  /* Update computation of how much all possible completions match */
 
 	  matchcount++;
-	  if (NULL (bestmatch))
+	  if (NIL (bestmatch))
 	    bestmatch = eltstring, bestmatchsize = XSTRING (eltstring)->size;
 	  else
 	    {
@@ -464,14 +478,53 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	      matchsize = scmp (XSTRING (bestmatch)->data,
 				XSTRING (eltstring)->data,
 				compare);
-	      bestmatchsize = (matchsize >= 0) ? matchsize : compare;
+	      if (matchsize < 0)
+                matchsize = compare;
+	      if (completion_ignore_case)
+		{
+		  /* If this is an exact match except for case,
+		     use it as the best match rather than one that is not an
+		     exact match.  This way, we get the case pattern
+		     of the actual match.  */
+		  if ((matchsize == XSTRING (eltstring)->size
+		       && matchsize < XSTRING (bestmatch)->size)
+		      ||
+		      /* If there is no exact match ignoring case,
+			 prefer a match that does not change the case
+			 of the input.  */
+		      (((matchsize == XSTRING (eltstring)->size)
+			==
+			(matchsize == XSTRING (bestmatch)->size))
+		       /* If there is more than one exact match ignoring case,
+			  and one of them is exact including case,
+			  prefer that one.  */
+		       && !bcmp (XSTRING (eltstring)->data,
+				 XSTRING (string)->data,
+				 XSTRING (string)->size)
+		       && bcmp (XSTRING (bestmatch)->data,
+				XSTRING (string)->data,
+				XSTRING (string)->size)))
+		    bestmatch = eltstring;
+		}
+	      bestmatchsize = matchsize;
 	    }
 	}
     }
 
-  if (NULL (bestmatch))
+  if (NIL (bestmatch))
     return Qnil;		/* No completions found */
-  if (matchcount == 1 && bestmatchsize == XSTRING (string)->size)
+  /* If we are ignoring case, and there is no exact match,
+     and no additional text was supplied,
+     don't change the case of what the user typed.  */
+  if (completion_ignore_case && bestmatchsize == XSTRING (string)->size
+      && XSTRING (bestmatch)->size > bestmatchsize)
+    return string;
+
+  /* Return t if the supplied string is an exact match (counting case);
+     it does not require any change to be made.  */
+  if (matchcount == 1 && bestmatchsize == XSTRING (string)->size
+      && !bcmp (XSTRING (bestmatch)->data, XSTRING (string)->data,
+              bestmatchsize))
     return Qt;
 
   XFASTINT (zero) = 0;		/* Else extract the part in which */
@@ -521,10 +574,10 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 {
   Lisp_Object tail, elt, eltstring;
   Lisp_Object allmatches;
-  int list = CONSP (alist);
-  int index = 0, obsize = 0;
-  Lisp_Object bucket = Qnil, tem;
-  struct gcpro gcpro1, gcpro2, gcpro3;
+  int list = CONSP (alist) || NIL (alist);
+  int index, obsize;
+  Lisp_Object bucket, tem;
+  struct gcpro gcpro1, gcpro2, gcpro3, gcpro4;
 
   CHECK_STRING (string, 0);
   if (!list && XTYPE (alist) != Lisp_Vector)
@@ -533,9 +586,9 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
     }
   allmatches = Qnil;
 
-  if (list)
-    tail = alist;
-  else
+  /* If ALIST is not a list, set TAIL just for gc pro.  */
+  tail = alist;
+  if (! list)
     {
       index = 0;
       obsize = XVECTOR (alist)->size;
@@ -551,7 +604,7 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 
       if (list)
 	{
-	  if (NULL (tail))
+	  if (NIL (tail))
 	    break;
 	  elt = Fcar (tail);
 	  eltstring = Fcar (elt);
@@ -589,17 +642,17 @@ The argument given to PREDICATE is the alist element or the symbol from the obar
 	  /* Ignore this element if there is a predicate
 	     and the predicate doesn't like it. */
 
-	  if (!NULL (pred))
+	  if (!NIL (pred))
 	    {
 	      if (EQ (pred, Qcommandp))
 		tem = Fcommandp (elt);
 	      else
 		{
-		  GCPRO3 (tail, eltstring, allmatches);
+		  GCPRO4 (tail, eltstring, allmatches, string);
 		  tem = call1 (pred, elt);
 		  UNGCPRO;
 		}
-	      if (NULL (tem)) continue;
+	      if (NIL (tem)) continue;
 	    }
 	  /* Ok => put it on the list. */
 	  allmatches = Fcons (eltstring, allmatches);
@@ -633,7 +686,7 @@ Case is ignored if ambient value of  completion-ignore-case  is non-nil.")
   specbind (Qminibuffer_completion_predicate, pred);
   specbind (Qminibuffer_completion_confirm,
 	    EQ (require_match, Qt) ? Qnil : Qt);
-  val = read_minibuf (NULL (require_match)
+  val = read_minibuf (NIL (require_match)
 		      ? Vminibuffer_local_completion_map
 		      : Vminibuffer_local_must_match_map,
 		      init, prompt, 0);
@@ -641,28 +694,29 @@ Case is ignored if ambient value of  completion-ignore-case  is non-nil.")
   return val;
 }
 
-temp_minibuf_message (m)
+temp_echo_area_contents (m)
      char *m;
 {
-  int osize = NumCharacters + 1;
+  int osize = ZV;
   Lisp_Object oinhibit;
   oinhibit = Vinhibit_quit;
 
-  SetPoint (osize);
+  SET_PT (osize);
   InsStr (m);
-  SetPoint (osize);
+  SET_PT (osize);
   Vinhibit_quit = Qt;
   Fsit_for (make_number (2), Qnil);
-  del_range (point, NumCharacters + 1);
-  if (!NULL (Vquit_flag))
+  del_range (point, ZV);
+  if (!NIL (Vquit_flag))
     {
       Vquit_flag = Qnil;
-      unread_command_char = Ctl ('g');
+      unread_command_char = quit_char;
     }
   Vinhibit_quit = oinhibit;
 }
 
 Lisp_Object Fminibuffer_completion_help ();
+Lisp_Object assoc_for_completion ();
 
 /* returns:
  * 0 no possible completion
@@ -680,10 +734,12 @@ do_completion ()
 
   completion = Ftry_completion (Fbuffer_string (), Vminibuffer_completion_table,
 				Vminibuffer_completion_predicate);
-  if (NULL (completion))
+  if (NIL (completion))
     {
-      Ding ();
-      temp_minibuf_message (" [No match]");
+      bell ();
+      /* Clearing this prevents sit-for from leaving the message up.  */
+      prev_echo_area_contents = 0;
+      temp_echo_area_contents (" [No match]");
       return 0;
     }
 
@@ -692,15 +748,16 @@ do_completion ()
 
   /* compiler bug */
   tem = Fstring_equal (completion, Fbuffer_string());
-  if (completedp = NULL (tem))
+  if (completedp = NIL (tem))
     {
       Ferase_buffer ();		/* Some completion happened */
       Finsert (1, &completion);
     }
 
   /* It did find a match.  Do we match some possibility exactly now? */
-  if (CONSP (Vminibuffer_completion_table))
-    tem = Fassoc (Fbuffer_string (), Vminibuffer_completion_table);
+  if (CONSP (Vminibuffer_completion_table)
+      || NIL (Vminibuffer_completion_table))
+    tem = assoc_for_completion (Fbuffer_string (), Vminibuffer_completion_table);
   else if (XTYPE (Vminibuffer_completion_table) == Lisp_Vector)
     {
       /* the primitive used by Fintern_soft */
@@ -712,7 +769,7 @@ do_completion ()
 		      XSTRING (tem)->data, XSTRING (tem)->size);
       if (XTYPE (tem) != Lisp_Symbol)
 	tem = Qnil;
-      else if (!NULL (Vminibuffer_completion_predicate))
+      else if (!NIL (Vminibuffer_completion_predicate))
 	tem = call1 (Vminibuffer_completion_predicate, tem);
       else
 	tem = Qt;
@@ -723,20 +780,47 @@ do_completion ()
 		 Vminibuffer_completion_predicate,
 		 Qlambda);
 
-  if (NULL (tem))
+  if (NIL (tem))
     { /* not an exact match */
       if (completedp)
 	return 5;
-      else if (auto_help)
+      else if (completion_auto_help)
 	Fminibuffer_completion_help ();
       else
-	temp_minibuf_message (" [Next char not unique]");
+	temp_echo_area_contents (" [Next char not unique]");
       return 6;
     }
   else
     return (completedp ? 4 : 3);
 }
-  
+/* Like assoc but assumes KEY is a string, and ignores case if appropriate.  */
+
+Lisp_Object
+assoc_for_completion (key, list)
+     register Lisp_Object key;
+     Lisp_Object list;
+{
+  register Lisp_Object tail;
+
+  if (completion_ignore_case)
+    key = Fupcase (key);
+
+  for (tail = list; !NIL (tail); tail = Fcdr (tail))
+    {
+      register Lisp_Object elt, tem, thiscar;
+      elt = Fcar (tail);
+      if (!CONSP (elt)) continue;
+      thiscar = Fcar (elt);
+      if (XTYPE (thiscar) != Lisp_String)
+      continue;
+      if (completion_ignore_case)
+      thiscar = Fupcase (thiscar);
+      tem = Fequal (thiscar, key);
+      if (!NIL (tem)) return elt;
+      QUIT;
+    }
+  return Qnil;
+}
 
 DEFUN ("minibuffer-complete", Fminibuffer_complete, Sminibuffer_complete, 0, 0, "",
   "Complete the minibuffer contents as far as possible.")
@@ -749,11 +833,11 @@ DEFUN ("minibuffer-complete", Fminibuffer_complete, Sminibuffer_complete, 0, 0,
       return Qnil;
 
     case 1:
-      temp_minibuf_message(" [Sole completion]");
+      temp_echo_area_contents(" [Sole completion]");
       break;
 
     case 3:
-      temp_minibuf_message(" [Complete, but not unique]");
+      temp_echo_area_contents(" [Complete, but not unique]");
       break;
     }
   return Qt;
@@ -770,7 +854,7 @@ a repetition of this command will exit.")
   register int i;
 
   /* Allow user to specify null string */
-  if (NumCharacters == 0)
+  if (BEGV == ZV)
     goto exit;
 
   i = do_completion ();
@@ -781,9 +865,9 @@ a repetition of this command will exit.")
       goto exit;
 
     case 4:
-      if (!NULL (Vminibuffer_completion_confirm))
+      if (!NIL (Vminibuffer_completion_confirm))
 	{
-	  temp_minibuf_message(" [Confirm]");
+	  temp_echo_area_contents(" [Confirm]");
 	  return Qnil;
 	}
       else
@@ -812,10 +896,10 @@ DEFUN ("minibuffer-complete-word", Fminibuffer_complete_word, Sminibuffer_comple
   completion = Ftry_completion (Fbuffer_string (),
 				Vminibuffer_completion_table,
 				Vminibuffer_completion_predicate);
-  if (NULL (completion))
+  if (NIL (completion))
     {
-      Ding ();
-      temp_minibuf_message (" [No match]");
+      bell ();
+      temp_echo_area_contents (" [No match]");
       return Qnil;
     }
   if (EQ (completion, Qt))
@@ -824,17 +908,17 @@ DEFUN ("minibuffer-complete-word", Fminibuffer_complete_word, Sminibuffer_comple
 #if 0 /* How the below code used to look, for reference */
   tem = Fbuffer_string ();
   b = XSTRING (tem)->data;
-  i = NumCharacters - XSTRING (completion)->size;
+  i = ZV - 1 - XSTRING (completion)->size;
   p = XSTRING (completion)->data;
   if (i > 0 ||
-      0 <= scmp (b, p, NumCharacters))
+      0 <= scmp (b, p, ZV - 1))
     {
       i = 1;
       /* Set buffer to longest match of buffer tail and completion head. */
-      while (0 <= scmp (b + i, p, NumCharacters - i))
+      while (0 <= scmp (b + i, p, ZV - 1 - i))
 	i++;
       del_range (1, i + 1);
-      SetPoint (NumCharacters + 1);
+      SET_PT (ZV);
     }
 #else /* Rewritten code */
   {
@@ -844,7 +928,7 @@ DEFUN ("minibuffer-complete-word", Fminibuffer_complete_word, Sminibuffer_comple
     tem = Fbuffer_string ();
     buffer_string = XSTRING (tem)->data;
     completion_string = XSTRING (completion)->data;
-    buffer_length = XSTRING (tem)->size; /* ie NumCharacters */
+    buffer_length = XSTRING (tem)->size; /* ie ZV - BEGV */
     completion_length = XSTRING (completion)->size;
     i = buffer_length - completion_length;
     /* Mly: I don't understand what this is supposed to do AT ALL */
@@ -858,11 +942,11 @@ DEFUN ("minibuffer-complete-word", Fminibuffer_complete_word, Sminibuffer_comple
 	while (0 <= scmp (buffer_string++, completion_string, buffer_length--))
 	  i++;
 	del_range (1, i + 1);
-	SetPoint (NumCharacters + 1);
+	SET_PT (ZV);
       }
   }
 #endif /* Rewritten code */
-  i = NumCharacters;
+  i = ZV - BEGV;
 
   /* If completion finds next char not unique,
      consider adding a space or a hyphen */
@@ -894,9 +978,9 @@ DEFUN ("minibuffer-complete-word", Fminibuffer_complete_word, Sminibuffer_comple
 
   /* If got no characters, print help for user.  */
 
-  if (i == NumCharacters)
+  if (i == ZV - BEGV)
     {
-      if (auto_help)
+      if (completion_auto_help)
 	Fminibuffer_completion_help ();
       return Qnil;
     }
@@ -904,7 +988,7 @@ DEFUN ("minibuffer-complete-word", Fminibuffer_complete_word, Sminibuffer_comple
   /* Otherwise insert in minibuffer the chars we got */
 
   Ferase_buffer ();
-  InsCStr (completion_string, i);
+  insert (completion_string, i);
   return Qt;
 }
 
@@ -918,15 +1002,18 @@ or may be a list of two strings to be printed as if concatenated.")
 {
   register Lisp_Object tail, elt;
   register int i;
-  struct buffer *old = bf_cur;
-  SetBfp (XBUFFER (Vstandard_output));
+  struct buffer *old = current_buffer;
+  /* No GCPRO needed, since (when it matters) every variable
+     points to a non-string that is pointed to by COMPLETIONS.  */
 
-  if (NULL (completions))
+  set_buffer_internal (XBUFFER (Vstandard_output));
+
+  if (NIL (completions))
     InsStr ("There are no possible completions of what you have typed.");
   else
     {
       InsStr ("Possible completions are:");
-      for (tail = completions, i = 0; !NULL (tail); tail = Fcdr (tail), i++)
+      for (tail = completions, i = 0; !NIL (tail); tail = Fcdr (tail), i++)
 	{
 	  /* this needs fixing for the case of long completions
 	     and/or narrow windows */
@@ -946,7 +1033,7 @@ or may be a list of two strings to be printed as if concatenated.")
 	    Fprinc (elt, Qnil);
 	}
     }
-  SetBfp (old);
+  set_buffer_internal (old);
   return Qnil;
 }
 
@@ -959,10 +1046,12 @@ DEFUN ("minibuffer-completion-help", Fminibuffer_completion_help, Sminibuffer_co
   message ("Making completion list...");
   completions = Fall_completions (Fbuffer_string (), Vminibuffer_completion_table,
 				  Vminibuffer_completion_predicate);
-  minibuf_message = 0;
-  if (NULL (completions))
-    { Ding ();
-      temp_minibuf_message (" [No completions]"); }
+  echo_area_contents = 0;
+  if (NIL (completions))
+    {
+      bell ();
+      temp_echo_area_contents (" [No completions]");
+    }
   else
     internal_with_output_to_temp_buffer (" *Completions*",
 					 Fdisplay_completion_list,
@@ -974,7 +1063,7 @@ DEFUN ("self-insert-and-exit", Fself_insert_and_exit, Sself_insert_and_exit, 0,
   "Terminate minibuffer input.")
   ()
 {
-  SelfInsert (last_command_char, 0);
+  self_insert_internal (last_command_char, 0);
   Fthrow (Qexit, Qnil);
 }
 
@@ -989,7 +1078,7 @@ DEFUN ("minibuffer-depth", Fminibuffer_depth, Sminibuffer_depth, 0, 0, 0,
   "Return current depth of activations of minibuffer, a nonnegative integer.")
   ()
 {
-  return make_number (MinibufDepth);
+  return make_number (minibuf_level);
 }
 
 
@@ -1001,7 +1090,7 @@ init_minibuf_once ()
 
 syms_of_minibuf ()
 {
-  MinibufDepth = 0;
+  minibuf_level = 0;
   minibuf_prompt = 0;
   minibuf_save_vector_size = 5;
   minibuf_save_vector = (struct minibuf_save_data *) malloc (5 * sizeof (struct minibuf_save_data));
@@ -1023,9 +1112,9 @@ syms_of_minibuf ()
 
 
 
-  DEFVAR_BOOL ("completion-auto-help", &auto_help,
+  DEFVAR_BOOL ("completion-auto-help", &completion_auto_help,
     "*Non-nil means automatically provide help for invalid completion input.");
-  auto_help = 1;
+  completion_auto_help = 1;
 
   DEFVAR_BOOL ("completion-ignore-case", &completion_ignore_case,
     "Non-nil means don't consider case significant in completion.");
