@@ -1,4 +1,16 @@
-/* X-window property handling */
+/*
+ * Epoch 4.0 : X-window property handling
+ */
+/*
+ * $Revision: 1.2 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/property.c,v $
+ * $Date: 91/04/12 11:34:45 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 91/04/12 11:34:45 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/property.c,v $ $Revision: 1.2 $";
+#endif
+
 
 #include <signal.h>
 #include <sys/ioctl.h>
