@@ -1,102 +1,26 @@
+
+/*
+ * $Header: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/regex.c,v 1.1 91/02/27 15:44:12 love Exp $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 91/02/27 15:44:12 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/regex.c,v $ $Revision: 1.1 $";
+#endif
 /* Extended regular expression matching and search.
    Copyright (C) 1985 Free Software Foundation, Inc.
 
-		       NO WARRANTY
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 1, or (at your option)
+    any later version.
 
-  BECAUSE THIS PROGRAM IS LICENSED FREE OF CHARGE, WE PROVIDE ABSOLUTELY
-NO WARRANTY, TO THE EXTENT PERMITTED BY APPLICABLE STATE LAW.  EXCEPT
-WHEN OTHERWISE STATED IN WRITING, FREE SOFTWARE FOUNDATION, INC,
-RICHARD M. STALLMAN AND/OR OTHER PARTIES PROVIDE THIS PROGRAM "AS IS"
-WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
-BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY
-AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE PROGRAM PROVE
-DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
-CORRECTION.
-
- IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL RICHARD M.
-STALLMAN, THE FREE SOFTWARE FOUNDATION, INC., AND/OR ANY OTHER PARTY
-WHO MAY MODIFY AND REDISTRIBUTE THIS PROGRAM AS PERMITTED BELOW, BE
-LIABLE TO YOU FOR DAMAGES, INCLUDING ANY LOST PROFITS, LOST MONIES, OR
-OTHER SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-USE OR INABILITY TO USE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
-DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY THIRD PARTIES OR
-A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS) THIS
-PROGRAM, EVEN IF YOU HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
-DAMAGES, OR FOR ANY CLAIM BY ANY OTHER PARTY.
-
-		GENERAL PUBLIC LICENSE TO COPY
-
-  1. You may copy and distribute verbatim copies of this source file
-as you receive it, in any medium, provided that you conspicuously and
-appropriately publish on each copy a valid copyright notice "Copyright
-(C) 1985 Free Software Foundation, Inc."; and include following the
-copyright notice a verbatim copy of the above disclaimer of warranty
-and of this License.  You may charge a distribution fee for the
-physical act of transferring a copy.
-
-  2. You may modify your copy or copies of this source file or
-any portion of it, and copy and distribute such modifications under
-the terms of Paragraph 1 above, provided that you also do the following:
-
-    a) cause the modified files to carry prominent notices stating
-    that you changed the files and the date of any change; and
-
-    b) cause the whole of any work that you distribute or publish,
-    that in whole or in part contains or is a derivative of this
-    program or any part thereof, to be licensed at no charge to all
-    third parties on terms identical to those contained in this
-    License Agreement (except that you may choose to grant more extensive
-    warranty protection to some or all third parties, at your option).
-
-    c) You may charge a distribution fee for the physical act of
-    transferring a copy, and you may at your option offer warranty
-    protection in exchange for a fee.
-
-Mere aggregation of another unrelated program with this program (or its
-derivative) on a volume of a storage or distribution medium does not bring
-the other program under the scope of these terms.
-
-  3. You may copy and distribute this program (or a portion or derivative
-of it, under Paragraph 2) in object code or executable form under the terms
-of Paragraphs 1 and 2 above provided that you also do one of the following:
-
-    a) accompany it with the complete corresponding machine-readable
-    source code, which must be distributed under the terms of
-    Paragraphs 1 and 2 above; or,
-
-    b) accompany it with a written offer, valid for at least three
-    years, to give any third party free (except for a nominal
-    shipping charge) a complete machine-readable copy of the
-    corresponding source code, to be distributed under the terms of
-    Paragraphs 1 and 2 above; or,
-
-    c) accompany it with the information you received as to where the
-    corresponding source code may be obtained.  (This alternative is
-    allowed only for noncommercial distribution and only if you
-    received the program in object code or executable form alone.)
-
-For an executable file, complete source code means all the source code for
-all modules it contains; but, as a special exception, it need not include
-source code for modules which are standard libraries that accompany the
-operating system on which the executable file runs.
-
-  4. You may not copy, sublicense, distribute or transfer this program
-except as expressly provided under this License Agreement.  Any attempt
-otherwise to copy, sublicense, distribute or transfer this program is void and
-your rights to use the program under this License agreement shall be
-automatically terminated.  However, parties who have received computer
-software programs from you with this License Agreement will not have
-their licenses terminated so long as such parties remain in full compliance.
-
-  5. If you wish to incorporate parts of this program into other free
-programs whose distribution conditions are different, write to the Free
-Software Foundation at 675 Mass Ave, Cambridge, MA 02139.  We have not yet
-worked out a simple rule that can be stated here, but we will often permit
-this.  We will be guided by the two goals of preserving the free status of
-all derivatives of our free software and of promoting the sharing and reuse of
-software.
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
 
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 In other words, you are welcome to use, share and improve this program.
 You are forbidden to forbid anyone else to use, share and improve
@@ -173,7 +97,7 @@ init_syntax_once ()
 
 #ifndef NFAILURES
 #define NFAILURES 80
-#endif /* NFAILURES */
+#endif NFAILURES
 
 /* width of a byte in bits */
 
@@ -609,7 +533,7 @@ re_compile_pattern (pattern, size, bufp)
 	      PATFETCH (c);
 	      PATPUSH (syntax_spec_code[c]);
 	      break;
-#endif /* emacs */
+#endif emacs
 
 	    case 'w':
 	      laststart = b;
@@ -906,7 +830,7 @@ re_compile_fastmap (bufp)
 	    if (SYNTAX (j) != (enum syntaxcode) k)
 	      fastmap[j] = 1;
 	  break;
-#endif /* emacs */
+#endif emacs
 
 	case charset:
 	  for (j = *p++ * BYTEWIDTH - 1; j >= 0; j--)
@@ -1052,13 +976,14 @@ re_search_2 (pbufp, string1, size1, string2, size2, startpos, range, regs, mstop
 	  && fastmap && pbufp->can_be_null == 0)
 	return -1;
 
-      val = re_match_2 (pbufp, string1, size1, string2, size2, startpos, regs, mstop);
+      val = re_match_2 (pbufp, string1, size1, string2, size2, startpos, regs,
+			mstop);
+      /* Propagate error indication if worse than mere failure.  */
+      if (val == -2)
+	return -2;
+      /* Return position on success.  */
       if (0 <= val)
-	{
-	  if (val == -2)
-	    return -2;
-	  return startpos;
-	}
+	return startpos;
 
 #ifdef C_ALLOCA
       alloca (0);
@@ -1272,6 +1197,12 @@ re_match_2 (pbufp, string1, size1, string2, size2, pos, regs, mstop)
 	    int regno = *p++;   /* Get which register to match against */
 	    register unsigned char *d2, *dend2;
 
+	    /* Don't allow matching a register that hasn't been used.
+	       This isn't fully reliable in the current version,
+	       but it is better than crashing.  */
+	    if ((int) regend[regno] <= -1)
+	      goto fail;
+
 	    d2 = regstart[regno];
  	    dend2 = ((regstart_seg1[regno] == regend_seg1[regno])
 		     ? regend[regno] : end_match_1);
@@ -1496,23 +1427,17 @@ re_match_2 (pbufp, string1, size1, string2, size2, pos, regs, mstop)
 
 #ifdef emacs
 	case before_dot:
-	  if (((d - string2 <= (unsigned) size2)
-	       ? d - bf_p2 : d - bf_p1)
-	      <= point)
+	  if (PTR_CHAR_POS (d) + 1 >= point)
 	    goto fail;
 	  break;
 
 	case at_dot:
-	  if (((d - string2 <= (unsigned) size2)
-	       ? d - bf_p2 : d - bf_p1)
-	      == point)
+	  if (PTR_CHAR_POS (d) + 1 != point)
 	    goto fail;
 	  break;
 
 	case after_dot:
-	  if (((d - string2 <= (unsigned) size2)
-	       ? d - bf_p2 : d - bf_p1)
-	      >= point)
+	  if (PTR_CHAR_POS (d) + 1 <= point)
 	    goto fail;
 	  break;
 
@@ -1547,7 +1472,7 @@ re_match_2 (pbufp, string1, size1, string2, size2, pos, regs, mstop)
 	  PREFETCH;
 	  if (SYNTAX (*d++) != 0) goto fail;
 	  break;
-#endif /* not emacs */
+#endif not emacs
 
 	case begbuf:
 	  if (d == string1)	/* Note, d cannot equal string2 */
@@ -1794,4 +1719,4 @@ error (string)
   exit (1);
 }
 
-#endif /* test */
+#endif test
