@@ -3,21 +3,29 @@
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.5 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/term.c,v $
+ * $Date: 91/09/16 15:55:33 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 91/09/16 15:55:33 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/term.c,v $ $Revision: 1.5 $";
+#endif
 
 #include <stdio.h>
 #include <ctype.h>
@@ -27,24 +35,25 @@ and this notice must be preserved on all copies.  */
 #include "termopts.h"
 #include "cm.h"
 
-#include "screenW.h"		/* [amc] */
+#include "x11term.h"		/* Epoch */
+#include "screen.h"		/* Epoch */
+#include "screenW.h"		/* Epoch */
+#include "dispepoch.h"		/* Epoch */
 
 #define max(a, b) ((a) > (b) ? (a) : (b))
 #define min(a, b) ((a) < (b) ? (a) : (b))
 
-#define OUTPUT(a) tputs (a, WS->height - curY, cmputc)
+#define OUTPUT(a) tputs (a, cur_Wscreen->height - curY, cmputc)
 #define OUTPUT1(a) tputs (a, 1, cmputc)
 #define OUTPUTL(a, lines) tputs (a, lines, cmputc)
-#define OUTPUT_IF(a) { if (a) tputs (a, WS->height - curY, cmputc); }
+#define OUTPUT_IF(a) { if (a) tputs (a, cur_Wscreen->height - curY, cmputc); }
 #define OUTPUT1_IF(a) { if (a) tputs (a, 1, cmputc); }
 
+extern struct Root_Block *cur_root; /* Current screen */
+
 /* Terminal charateristics that higher levels want to look at.
    These are all extern'd in termchar.h */
 
-#ifdef NO_COED
-int screen_width;		/* Number of usable columns */
-int screen_height;		/* Number of lines */
-#endif
 int must_write_spaces;		/* Nonzero means spaces in the text
 				   must actually be output; can't just skip
 				   over some columns to leave them blank.  */
@@ -67,27 +76,29 @@ int no_redraw_on_reenter;
 /* DCICcost[n] is cost of inserting N characters.
    DCICcost[-n] is cost of deleting N characters. */
 
-#define DCICcost (&DC_ICcost[MScreenWidth])
-int DC_ICcost[1 + 2 * MScreenWidth];
-
+#define DCICcost (&DC_ICcost[ws->width])
+int *DC_ICcost;
 
 /* Hook functions that you can set to snap out the functions in this file.
    These are all extern'd in termhooks.h  */
 
-int (*topos_hook) ();
-int (*raw_topos_hook) ();
+int (*move_cursor_hook) ();
+int (*raw_move_cursor_hook) ();
 
+int (*clear_window_end_hook) ();
 int (*clear_to_end_hook) ();
 int (*clear_screen_hook) ();
 int (*clear_end_of_line_hook) ();
 
 int (*ins_del_lines_hook) ();
+int (*shift_region_hook) ();
 
 int (*change_line_highlight_hook) ();
 int (*reassert_line_highlight_hook) ();
 
+int (*update_line_hook) ();
 int (*insert_chars_hook) ();
-int (*write_chars_hook) ();
+int (*output_chars_hook) ();
 int (*delete_chars_hook) ();
 
 int (*ring_bell_hook) ();
@@ -100,6 +111,9 @@ int (*set_terminal_window_hook) ();
 
 int (*read_socket_hook) ();
 int (*fix_screen_hook) ();
+int (*calculate_costs_hook) ();
+
+int (*text_width_hook) ();
 
 /* Strings, numbers and flags taken from the termcap entry.  */
 
@@ -165,7 +179,8 @@ static int se_is_so;	/* 1 if same string both enters and leaves standout mode */
 /* Number of chars of space used for standout marker at beginning of line,
    or'd with 0100.  Zero if no standout marker at all.  */
 /* used iff TN_standout_width >= 0. */
-char chars_wasted[MScreenLength];
+char *chars_wasted;
+static char *copybuf;
 
 /* nonzero means supposed to write text in standout mode.  */
 int standout_requested;
@@ -184,6 +199,21 @@ int specified_window;
 
 char *tparam ();
 
+short
+text_width(font_struct,s,l)
+     struct XFontStruct *font_struct;
+     char *s;
+     int l;
+{
+  int ret_val;
+  if (text_width_hook)
+    {
+      ret_val = (*text_width_hook) (font_struct,s,l);
+      return (short) ret_val;
+    }
+  return l;
+}
+
 ring_bell ()
 {
   if (ring_bell_hook)
@@ -265,11 +295,12 @@ set_scroll_region (start, stop)
   else if (TS_set_scroll_region_1)
     {
       buf = tparam (TS_set_scroll_region_1, 0, 0,
-		    WS->height, start, WS->height - stop, WS->height);
+		    cur_Wscreen->height, start, cur_Wscreen->height - stop,
+		    cur_Wscreen->height);
     }
   else
     {
-      buf = tparam (TS_set_window, 0, 0, start, 0, stop, WS->width);
+      buf = tparam (TS_set_window, 0, 0, start, 0, stop, cur_Wscreen->width);
     }
   OUTPUT (buf);
   free (buf);
@@ -402,8 +433,17 @@ change_line_highlight (new_highlight, vpos, first_unused_hpos)
       (*change_line_highlight_hook) (new_highlight, vpos, first_unused_hpos);
       return;
     }
-
-  topos (vpos, 0);
+  WS->cursor_x = 0;
+  WS->cursor_y = 0;
+  WS->new_cur_w = WS->cur_w;
+  WS->new_cur_char = WS->cur_line->body;
+  WS->new_cur_line = WS->cur_line;
+  
+  move_cursor (WS->new_cur_line,
+	       WS->new_cur_char,
+	       vpos,
+	       0,
+	       WS->new_cur_w,cur_root);
 
   if (TN_standout_width < 0)
     background_highlight ();
@@ -414,7 +454,7 @@ change_line_highlight (new_highlight, vpos, first_unused_hpos)
       /* On Teleray, make sure to erase the SO marker.  */
       if (TF_teleray)
 	{
-	  cmgoto (curY - 1, WS->width - 4);
+	  cmgoto (curY - 1, cur_Wscreen->width - 4);
 	  OUTPUT ("\033S");
 	  curY++;		/* ESC S moves to next line where the TS_standout_mode was */
 	  curX = 0;
@@ -428,14 +468,20 @@ change_line_highlight (new_highlight, vpos, first_unused_hpos)
 
 /* Move to absolute position, specified origin 0 */
 
-topos (row, col)
+move_cursor (l,cb,row, col,win,rb)
+     struct line_header *l;
+     struct char_block *cb;
+     int row,col;
+     struct window *win;
+     struct Root_Block *rb;
 {
-  col += chars_wasted[row] & 077;
-  if (topos_hook)
+
+  if (move_cursor_hook)
     {
-      (*topos_hook) (row, col);
+      (*move_cursor_hook) (l,cb,row,col,win,rb);
       return;
     }
+  col += chars_wasted[row] & 077;  
   if (curY == row && curX == col)
     return;
   if (!TF_standout_motion)
@@ -447,11 +493,11 @@ topos (row, col)
 
 /* Similar but don't take any account of the wasted characters.  */
 
-raw_topos (row, col)
+raw_move_cursor (row, col)
 {
-  if (raw_topos_hook)
+  if (raw_move_cursor_hook)
     {
-      (*raw_topos_hook) (row, col);
+      (*raw_move_cursor_hook) (row, col);
       return;
     }
   if (curY == row && curX == col)
@@ -479,14 +525,17 @@ clear_to_end ()
     {
       background_highlight ();
       OUTPUT (TS_clr_to_bottom);
-      bzero (chars_wasted + curY, WS->height - curY);
+      bzero (chars_wasted + curY, cur_Wscreen->height - curY);
     }
   else
     {
-      for (i = curY; i < WS->height; i++)
+      for (i = curY; i < cur_Wscreen->height; i++)
 	{
-	  topos (i, 0);
-	  clear_end_of_line_raw (WS->width);
+	  move_cursor (cur_Wscreen->new_cur_line,
+		       cur_Wscreen->new_cur_char,
+		       0,0,
+		       cur_Wscreen->new_cur_w,cur_root); /* WRONG */
+	  clear_end_of_line_raw (cur_Wscreen->width);
 	}
     }
 }
@@ -504,16 +553,36 @@ clear_screen ()
     {
       background_highlight ();
       OUTPUT (TS_clr_screen);
-      bzero (chars_wasted, WS->height);
+      bzero (chars_wasted, cur_Wscreen->height);
       cmat (0, 0);
     }
   else
     {
-      topos (0, 0);
+      move_cursor (cur_Wscreen->new_cur_line,
+		   cur_Wscreen->new_cur_char,
+		   cur_Wscreen->cursor_x,
+		   cur_Wscreen->cursor_y,
+		   cur_Wscreen->new_cur_w,cur_root);	/* WRONG */
       clear_to_end ();
     }
 }
 
+/* Clear from last visible line on window to window end (presumably the line
+ * above window's modeline.
+ */
+clear_window_end (w,ystart,yend)
+     struct window *w;
+     int ystart,yend;
+{
+  if (clear_window_end_hook)
+    {
+      (*clear_window_end_hook) (w,ystart,yend);
+      return;
+    }
+
+  /* Stuff goes here */
+}
+
 /* Clear to end of line, but do not clear any standout marker.
    Assumes that the cursor is positioned at a character of real text,
    which implies it cannot be before a standout marker
@@ -525,7 +594,7 @@ clear_end_of_line (first_unused_hpos)
      int first_unused_hpos;
 {
   if (TN_standout_width == 0 && curX == 0 && chars_wasted[curY] != 0)
-    write_chars (" ", 1);
+    output_chars (" ", 1);
   clear_end_of_line_raw (first_unused_hpos);
 }
 
@@ -568,8 +637,23 @@ clear_end_of_line_raw (first_unused_hpos)
     }
 }
 
-write_chars (start, len)
-     register char *start;
+update_line(w,l,cb,end)
+     struct window *w;
+     struct line_header *l;
+     struct char_block *cb;
+     struct char_block *end;
+{
+  if (update_line_hook)
+    {
+      (*update_line_hook) (w,l,cb,end);
+      return;
+    }
+
+  /* Stuff goes here */
+}
+
+output_chars (string, len)
+     register char *string;
      int len;
 {
   register char *p;
@@ -578,9 +662,9 @@ write_chars (start, len)
   register int c;
   char *first_check;
 
-  if (write_chars_hook)
+  if (output_chars_hook)
     {
-      (*write_chars_hook) (start, len);
+      (*output_chars_hook) (string, len);
       return;
     }
   highlight_if_desired ();
@@ -589,41 +673,44 @@ write_chars (start, len)
   /* Don't dare write in last column of bottom line, if AutoWrap,
      since that would scroll the whole screen on some terminals.  */
 
-  if (AutoWrap && curY + 1 == WS->height
-      && curX + len - (chars_wasted[curY] & 077) == WS->width)
+  if (AutoWrap && curY + 1 == cur_Wscreen->height
+      && curX + len - (chars_wasted[curY] & 077) == cur_Wscreen->width)
     len --;
 
   cmplus (len);
 
-  first_check = start;
+  first_check = string;
 
   if (RPov > len && !TF_underscore && !TF_hazeltine)
     {
-      fwrite (start, 1, len, stdout);
+      fwrite (string, 1, len, stdout);
       if (ferror (stdout))
 	clearerr (stdout);
       if (termscript)
-	fwrite (start, 1, len, termscript);
+	fwrite (string, 1, len, termscript);
     }
   else
     while (--len >= 0)
       {
-	if (RPov + 1 < len && start >= first_check && *start == start[1])
+	c = *string;
+	if (RPov + 1 < len && string >= first_check)
 	  {
-	    p = start + 2;
+	    int repeat_count;
+
+	    p = string + 1;
 
 	    /* Now, len is number of chars left starting at p */
-	    while (*p++ == *start);
-	    --p;
-	    /* n is number of identical chars in this run */
-	    n = p - start;
-	    if (n > RPov)
+	    while (*p++ == c);
+	    p--;
+
+	    repeat_count = p - string;
+	    if (repeat_count > RPov)
 	      {
-		buf = tparam (TS_repeat, 0, 0, *start, n);
-		tputs (buf, n, cmputc);
+		buf = tparam (TS_repeat, 0, 0, *string, repeat_count);
+		tputs (buf, repeat_count, cmputc);
 		free (buf);
-		start = p;
-		len -= n - 1;
+		string = p;
+		len -= repeat_count - 1;
 		continue;
 	      }
 	    else
@@ -631,7 +718,6 @@ write_chars (start, len)
 		 don't even consider the last N-1, the last N-2,...  */
 	      first_check = p;
 	  }
-	c = *start++;
 	if (c == '_' && TF_underscore)
 	  {
 	    if (termscript)
@@ -644,6 +730,7 @@ write_chars (start, len)
 	if (termscript)
 	  fputc (c, termscript);
 	putchar (c);
+	string++;
       }
 }
 
@@ -669,7 +756,7 @@ insert_chars (start, len)
       OUTPUT1 (buf);
       free (buf);
       if (start)
-	write_chars (start, len);
+	output_chars (start, len);
       return;
     }
 
@@ -736,6 +823,21 @@ delete_chars (n)
   if (!delete_in_insert_mode)
     OUTPUT_IF (TS_end_delete_mode);
 }
+
+/* Shift a region between START and END to its new position */
+shift_region(w,start,end)
+     struct window *w;
+     struct line_header *start,*end;
+{
+  if (shift_region_hook)
+    {
+      (*shift_region_hook) (w,start,end);
+      return;
+    }
+
+  /* STUFF goes here */
+}
+   
 
 /* Insert N lines at vpos VPOS.  If N is negative, delete -N lines.  */
 
@@ -748,7 +850,6 @@ ins_del_lines (vpos, n)
 
   register int i = n > 0 ? n : -n;
   register char *buf;
-  char copybuf[MScreenWidth];
 
   if (ins_del_lines_hook)
     {
@@ -765,12 +866,12 @@ ins_del_lines (vpos, n)
      as there will be a matching inslines later that will flush them. */
   if (scroll_region_ok && vpos + i >= specified_window)
     return;
-  if (!memory_below_screen && vpos + i >= WS->height)
+  if (!memory_below_screen && vpos + i >= cur_Wscreen->height)
     return;
 
   if (multi)
     {
-      raw_topos (vpos, 0);
+      raw_move_cursor (vpos, 0);
       background_highlight ();
       buf = tparam (multi, 0, 0, i);
       OUTPUT (buf);
@@ -778,7 +879,7 @@ ins_del_lines (vpos, n)
     }
   else if (single)
     {
-      raw_topos (vpos, 0);
+      raw_move_cursor (vpos, 0);
       background_highlight ();
       while (--i >= 0)
 	OUTPUT (single);
@@ -789,9 +890,9 @@ ins_del_lines (vpos, n)
     {
       set_scroll_region (vpos, specified_window);
       if (n < 0)
-	raw_topos (specified_window - 1, 0);
+	raw_move_cursor (specified_window - 1, 0);
       else
-	raw_topos (vpos, 0);
+	raw_move_cursor (vpos, 0);
       background_highlight ();
       while (--i >= 0)
 	OUTPUTL (scroll, specified_window - vpos);
@@ -800,21 +901,29 @@ ins_del_lines (vpos, n)
 
   if (TN_standout_width >= 0)
     {
+      register lower_limit
+	= scroll_region_ok ? specified_window : cur_Wscreen->height;
       if (n < 0)
 	{
-	  bcopy (&chars_wasted[curY - n], &chars_wasted[curY], WS->height - curY + n);
-	  bzero (&chars_wasted[WS->height + n], - n);
+	  bcopy (&chars_wasted[vpos - n], &chars_wasted[vpos],
+		 lower_limit - vpos + n);
+	  bzero (&chars_wasted[lower_limit + n], - n);
 	}
       else
 	{
-	  bcopy (&chars_wasted[curY], &copybuf[curY], WS->height - curY - n);
-	  bcopy (&copybuf[curY], &chars_wasted[curY + n], WS->height - curY - n);
-	  bzero (&chars_wasted[curY], n);
+	  bcopy (&chars_wasted[vpos], &copybuf[vpos], lower_limit - vpos - n);
+	  bcopy (&copybuf[vpos], &chars_wasted[vpos + n],
+		 lower_limit - vpos - n);
+	  bzero (&chars_wasted[vpos], n);
 	}
     }
   if (!scroll_region_ok && memory_below_screen && n < 0)
     {
-      topos (WS->height + n, 0);
+      move_cursor (cur_Wscreen->new_cur_line,
+		   cur_Wscreen->new_cur_char,
+		   cur_Wscreen->cursor_x,
+		   cur_Wscreen->cursor_y,
+		   cur_Wscreen->new_cur_w,cur_root); /* WRONG */
       clear_to_end ();
     }
 }
@@ -865,6 +974,7 @@ calculate_ins_del_char_costs ()
   int ins_cost_per_char, del_cost_per_char;
   register int i;
   register int *p;
+  struct W_Screen *ws = cur_Wscreen;
 
   if (TS_ins_multi_chars)
     {
@@ -905,7 +1015,7 @@ calculate_ins_del_char_costs ()
 
   /* Delete costs are at negative offsets */
   p = &DCICcost[0];
-  for (i = WS->width; --i >= 0;)
+  for (i = cur_Wscreen->width; --i >= 0;)
     *--p = (del_startup_cost += del_cost_per_char);
 
   /* Doing nothing is free */
@@ -913,21 +1023,37 @@ calculate_ins_del_char_costs ()
   *p++ = 0;
 
   /* Insert costs are at positive offsets */
-  for (i = WS->width; --i >= 0;)
+  for (i = cur_Wscreen->width; --i >= 0;)
     *p++ = (ins_startup_cost += ins_cost_per_char);
 }
 
 calculate_costs ()
 {
-#ifdef CB
-  return;
-#else
   register char *s
     = TS_set_scroll_region ? TS_set_scroll_region : TS_set_scroll_region_1;
 
-  if (dont_calculate_costs)
+  if (!inhibit_window_system)	/* Epoch */
     return;
 
+  if (chars_wasted != 0)
+    chars_wasted = (char *) xrealloc (chars_wasted, cur_Wscreen->height);
+  else
+    chars_wasted = (char *) xmalloc (cur_Wscreen->height);
+  bzero (chars_wasted, cur_Wscreen->height);
+
+  if (copybuf != 0)
+    copybuf = (char *) xrealloc (copybuf, cur_Wscreen->height);
+  else
+    copybuf = (char *) xmalloc (cur_Wscreen->height);
+
+  if (DC_ICcost != 0)
+    DC_ICcost = (int *) xrealloc (DC_ICcost,
+				  (2 * cur_Wscreen->width + 1) * sizeof (int));
+  else
+    DC_ICcost = (int *) xmalloc ((2 * cur_Wscreen->width + 1) * sizeof (int));
+#if 0
+  /* Always call CalcIDCosts because it allocates some vectors.
+     That function handles dont_calculate_costs.  */
   if (s && (!TS_ins_line && !TS_del_line))
     CalcIDCosts (TS_rev_scroll, TS_ins_multi_lines,
 		 TS_fwd_scroll, TS_del_multi_lines,
@@ -936,6 +1062,12 @@ calculate_costs ()
     CalcIDCosts (TS_ins_line, TS_ins_multi_lines,
 		 TS_del_line, TS_del_multi_lines,
 		 0, 0);
+#endif
+  if (dont_calculate_costs)
+    {
+      bzero (DC_ICcost, 2 * cur_Wscreen->width * sizeof (int));
+      return;
+    }
 
   calculate_ins_del_char_costs ();
 
@@ -943,19 +1075,17 @@ calculate_costs ()
   if (TS_repeat && per_line_cost (TS_repeat) * baud_rate < 9000)
     RPov = string_cost (TS_repeat);
   else
-    RPov = MScreenWidth;
+    RPov = cur_Wscreen->width * 2;
 
   cmcostinit ();		/* set up cursor motion costs */
-#endif
 }
 
 term_init (terminal_type)
      char *terminal_type;
 {
-#ifdef NO_COED
-  char *combuf;
-  char *fill;
-  char tbuf[2044];
+  char *area;
+  char **address = &area;
+  char buffer[2044];
   register char *p;
   int status;
 
@@ -964,52 +1094,51 @@ term_init (terminal_type)
   Wcm_clear ();
   dont_calculate_costs = 0;
 
-  status = tgetent (tbuf, terminal_type);
+  status = tgetent (buffer, terminal_type);
   if (status < 0)
     fatal ("Cannot open termcap database file.\n");
   if (status == 0)
     fatal ("Terminal type %s is not defined.\n", terminal_type);
 
 #ifdef TERMINFO
-  combuf = (char *) malloc (2044);
+  area = (char *) malloc (2044);
 #else
-  combuf = (char *) malloc (strlen (tbuf));
+  area = (char *) malloc (strlen (buffer));
 #endif /* not TERMINFO */
-  if (combuf == 0)
+  if (area == 0)
     abort ();
-  fill = combuf;
 
-  TS_ins_line = tgetstr ("al", &fill);
-  TS_ins_multi_lines = tgetstr ("AL", &fill);
-  TS_bell = tgetstr ("bl", &fill);
-  TS_clr_to_bottom = tgetstr ("cd", &fill);
-  TS_clr_line = tgetstr ("ce", &fill);
-  TS_clr_screen = tgetstr ("cl", &fill);
-  ColPosition = tgetstr ("ch", &fill);
-  AbsPosition = tgetstr ("cm", &fill);
-  CR = tgetstr ("cr", &fill);
-  TS_set_scroll_region = tgetstr ("cs", &fill);
-  TS_set_scroll_region_1 = tgetstr ("cS", &fill);
-  RowPosition = tgetstr ("cv", &fill);
-  TS_del_char = tgetstr ("dc", &fill);
-  TS_del_multi_chars = tgetstr ("DC", &fill);
-  TS_del_line = tgetstr ("dl", &fill);
-  TS_del_multi_lines = tgetstr ("DL", &fill);
-  TS_delete_mode = tgetstr ("dm", &fill);
-  TS_end_delete_mode = tgetstr ("ed", &fill);
-  TS_end_insert_mode = tgetstr ("ei", &fill);
-  Home = tgetstr ("ho", &fill);
-  TS_ins_char = tgetstr ("ic", &fill);
-  TS_ins_multi_chars = tgetstr ("IC", &fill);
-  TS_insert_mode = tgetstr ("im", &fill);
-  TS_pad_inserted_char = tgetstr ("ip", &fill);
-  TS_end_keypad_mode = tgetstr ("ke", &fill);
-  TS_keypad_mode = tgetstr ("ks", &fill);
-  LastLine = tgetstr ("ll", &fill);
-  Right = tgetstr ("nd", &fill);
-  Down = tgetstr ("do", &fill);
+  TS_ins_line = tgetstr ("al", address);
+  TS_ins_multi_lines = tgetstr ("AL", address);
+  TS_bell = tgetstr ("bl", address);
+  TS_clr_to_bottom = tgetstr ("cd", address);
+  TS_clr_line = tgetstr ("ce", address);
+  TS_clr_screen = tgetstr ("cl", address);
+  ColPosition = tgetstr ("ch", address);
+  AbsPosition = tgetstr ("cm", address);
+  CR = tgetstr ("cr", address);
+  TS_set_scroll_region = tgetstr ("cs", address);
+  TS_set_scroll_region_1 = tgetstr ("cS", address);
+  RowPosition = tgetstr ("cv", address);
+  TS_del_char = tgetstr ("dc", address);
+  TS_del_multi_chars = tgetstr ("DC", address);
+  TS_del_line = tgetstr ("dl", address);
+  TS_del_multi_lines = tgetstr ("DL", address);
+  TS_delete_mode = tgetstr ("dm", address);
+  TS_end_delete_mode = tgetstr ("ed", address);
+  TS_end_insert_mode = tgetstr ("ei", address);
+  Home = tgetstr ("ho", address);
+  TS_ins_char = tgetstr ("ic", address);
+  TS_ins_multi_chars = tgetstr ("IC", address);
+  TS_insert_mode = tgetstr ("im", address);
+  TS_pad_inserted_char = tgetstr ("ip", address);
+  TS_end_keypad_mode = tgetstr ("ke", address);
+  TS_keypad_mode = tgetstr ("ks", address);
+  LastLine = tgetstr ("ll", address);
+  Right = tgetstr ("nd", address);
+  Down = tgetstr ("do", address);
   if (!Down)
-    Down = tgetstr ("nl", &fill); /* Obsolete name for "do" */
+    Down = tgetstr ("nl", address); /* Obsolete name for "do" */
 #ifdef VMS
   /* VMS puts a carriage return before each linefeed,
      so it is not safe to use linefeeds.  */
@@ -1019,29 +1148,29 @@ term_init (terminal_type)
   if (tgetflag ("bs"))
     Left = "\b";		  /* can't possibly be longer! */
   else				  /* (Actually, "bs" is obsolete...) */
-    Left = tgetstr ("le", &fill);
+    Left = tgetstr ("le", address);
   if (!Left)
-    Left = tgetstr ("bc", &fill); /* Obsolete name for "le" */
-  TS_pad_char = tgetstr ("pc", &fill);
-  TS_repeat = tgetstr ("rp", &fill);
-  TS_end_standout_mode = tgetstr ("se", &fill);
-  TS_fwd_scroll = tgetstr ("sf", &fill);
-  TS_standout_mode = tgetstr ("so", &fill);
-  TS_rev_scroll = tgetstr ("sr", &fill);
-  Wcm.cm_tab = tgetstr ("ta", &fill);
-  TS_end_termcap_modes = tgetstr ("te", &fill);
-  TS_termcap_modes = tgetstr ("ti", &fill);
-  Up = tgetstr ("up", &fill);
-  TS_visible_bell = tgetstr ("vb", &fill);
-  TS_end_visual_mode = tgetstr ("ve", &fill);
-  TS_visual_mode = tgetstr ("vs", &fill);
-  TS_set_window = tgetstr ("wi", &fill);
+    Left = tgetstr ("bc", address); /* Obsolete name for "le" */
+  TS_pad_char = tgetstr ("pc", address);
+  TS_repeat = tgetstr ("rp", address);
+  TS_end_standout_mode = tgetstr ("se", address);
+  TS_fwd_scroll = tgetstr ("sf", address);
+  TS_standout_mode = tgetstr ("so", address);
+  TS_rev_scroll = tgetstr ("sr", address);
+  Wcm.cm_tab = tgetstr ("ta", address);
+  TS_end_termcap_modes = tgetstr ("te", address);
+  TS_termcap_modes = tgetstr ("ti", address);
+  Up = tgetstr ("up", address);
+  TS_visible_bell = tgetstr ("vb", address);
+  TS_end_visual_mode = tgetstr ("ve", address);
+  TS_visual_mode = tgetstr ("vs", address);
+  TS_set_window = tgetstr ("wi", address);
 
   AutoWrap = tgetflag ("am");
   memory_below_screen = tgetflag ("db");
   TF_hazeltine = tgetflag ("hz");
   must_write_spaces = tgetflag ("in");
-  MetaFlag = tgetflag ("km") || tgetflag ("MT");
+  meta_key = tgetflag ("km") || tgetflag ("MT");
   TF_insmode_motion = tgetflag ("mi");
   TF_standout_motion = tgetflag ("ms");
   TF_underscore = tgetflag ("ul");
@@ -1050,11 +1179,11 @@ term_init (terminal_type)
   TF_teleray = tgetflag ("xt");
 
   /* Get screen size from system, or else from termcap.  */
-  get_screen_size (&WS->width, &WS->height);
-  if (WS->width <= 0)
-    WS->width = tgetnum ("co");
-  if (WS->height <= 0)
-    WS->height = tgetnum ("li");
+  get_screen_size (&cur_Wscreen->width, &cur_Wscreen->height);
+  if (cur_Wscreen->width <= 0)
+    cur_Wscreen->width = tgetnum ("co");
+  if (cur_Wscreen->height <= 0)
+    cur_Wscreen->height = tgetnum ("li");
 
   min_padding_speed = tgetnum ("pb");
   TN_standout_width = tgetnum ("sg");
@@ -1089,8 +1218,8 @@ term_init (terminal_type)
   if (TS_standout_mode == 0)
     {
       TN_standout_width = tgetnum ("ug");
-      TS_end_standout_mode = tgetstr ("ue", &fill);
-      TS_standout_mode = tgetstr ("us", &fill);
+      TS_end_standout_mode = tgetstr ("ue", address);
+      TS_standout_mode = tgetstr ("us", address);
     }
 
   if (TF_teleray)
@@ -1141,14 +1270,14 @@ term_init (terminal_type)
       must_write_spaces = 1;
       /* :ti string typically fails to have \E^G! in it */
       /* This limits scope of insert-char to one line.  */
-      strcpy (fill, TS_termcap_modes);
-      strcat (fill, "\033\007!");
-      TS_termcap_modes = fill;
-      fill += strlen (fill) + 1;
-      p = combuf;
+      strcpy (area, TS_termcap_modes);
+      strcat (area, "\033\007!");
+      TS_termcap_modes = area;
+      area += strlen (area) + 1;
+      p = AbsPosition;
       /* Change all %+ parameters to %C, to handle
 	 values above 96 correctly for the C100.  */
-      while (p != fill)
+      while (*p)
 	{
 	  if (p[0] == '%' && p[1] == '+')
 	    p[1] = 'C';
@@ -1156,11 +1285,8 @@ term_init (terminal_type)
 	}
     }
 
-  WS->height = min (WS->height, MScreenLength);
-  WS->width  = min (WS->width, MScreenWidth);
-
-  ScreenRows = WS->height;
-  ScreenCols = WS->width;
+  ScreenRows = cur_Wscreen->height;
+  ScreenCols = cur_Wscreen->width;
   specified_window = WS->height;
 
   if (Wcm_init ())	/* can't do cursor motion */
@@ -1189,7 +1315,7 @@ It may be necessary to do `unsetenv TERMCAP' as well.\n",
 
   /* Remove width of standout marker from usable width of line */
   if (TN_standout_width > 0)
-    WS->width -= TN_standout_width;
+    cur_Wscreen->width -= TN_standout_width;
 
   UseTabs = tabs_safe_p () && TabWidth == 8;
 
@@ -1202,7 +1328,8 @@ It may be necessary to do `unsetenv TERMCAP' as well.\n",
 			 && TS_fwd_scroll
 			 && TS_rev_scroll));
 
-  char_ins_del_ok = ((TS_ins_char || TS_ins_multi_chars)
+  char_ins_del_ok = ((TS_ins_char || TS_insert_mode ||
+		      TS_pad_inserted_char || TS_ins_multi_chars)
 		     && (TS_del_char || TS_del_multi_chars));
 
   fast_clear_end_of_line = TS_clr_line != 0;
@@ -1211,14 +1338,11 @@ It may be necessary to do `unsetenv TERMCAP' as well.\n",
   if (read_socket_hook)		/* Baudrate is somewhat */
 				/* meaningless in this case */
     baud_rate = 9600;
-
-  calculate_costs ();
-
-#endif /* NO_COED */
 }
 
 /* VARARGS 1 */
 fatal (str, arg1, arg2)
+     char *str;
 {
   fprintf (stderr, "emacs: ");
   fprintf (stderr, str, arg1, arg2);
