@@ -1,106 +1,33 @@
 /* Copyright (C) 1985, 1986, 1987, 1988 Free Software Foundation, Inc.
 
-		       NO WARRANTY
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 1, or (at your option)
+    any later version.
 
-  BECAUSE THIS PROGRAM IS LICENSED FREE OF CHARGE, WE PROVIDE ABSOLUTELY
-NO WARRANTY, TO THE EXTENT PERMITTED BY APPLICABLE STATE LAW.  EXCEPT
-WHEN OTHERWISE STATED IN WRITING, FREE SOFTWARE FOUNDATION, INC,
-RICHARD M. STALLMAN AND/OR OTHER PARTIES PROVIDE THIS PROGRAM "AS IS"
-WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
-BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
-FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY
-AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE PROGRAM PROVE
-DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
-CORRECTION.
-
- IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW WILL RICHARD M.
-STALLMAN, THE FREE SOFTWARE FOUNDATION, INC., AND/OR ANY OTHER PARTY
-WHO MAY MODIFY AND REDISTRIBUTE THIS PROGRAM AS PERMITTED BELOW, BE
-LIABLE TO YOU FOR DAMAGES, INCLUDING ANY LOST PROFITS, LOST MONIES, OR
-OTHER SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-USE OR INABILITY TO USE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR
-DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY THIRD PARTIES OR
-A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS) THIS
-PROGRAM, EVEN IF YOU HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
-DAMAGES, OR FOR ANY CLAIM BY ANY OTHER PARTY.
-
-		GENERAL PUBLIC LICENSE TO COPY
-
-  1. You may copy and distribute verbatim copies of this source file
-as you receive it, in any medium, provided that you conspicuously and
-appropriately publish on each copy a valid copyright notice "Copyright
-(C) 1987 Free Software Foundation, Inc."; and include following the
-copyright notice a verbatim copy of the above disclaimer of warranty
-and of this License.  You may charge a distribution fee for the
-physical act of transferring a copy.
-
-  2. You may modify your copy or copies of this source file or
-any portion of it, and copy and distribute such modifications under
-the terms of Paragraph 1 above, provided that you also do the following:
-
-    a) cause the modified files to carry prominent notices stating
-    that you changed the files and the date of any change; and
-
-    b) cause the whole of any work that you distribute or publish,
-    that in whole or in part contains or is a derivative of this
-    program or any part thereof, to be licensed at no charge to all
-    third parties on terms identical to those contained in this
-    License Agreement (except that you may choose to grant more extensive
-    warranty protection to some or all third parties, at your option).
-
-    c) You may charge a distribution fee for the physical act of
-    transferring a copy, and you may at your option offer warranty
-    protection in exchange for a fee.
-
-Mere aggregation of another unrelated program with this program (or its
-derivative) on a volume of a storage or distribution medium does not bring
-the other program under the scope of these terms.
-
-  3. You may copy and distribute this program (or a portion or derivative
-of it, under Paragraph 2) in object code or executable form under the terms
-of Paragraphs 1 and 2 above provided that you also do one of the following:
-
-    a) accompany it with the complete corresponding machine-readable
-    source code, which must be distributed under the terms of
-    Paragraphs 1 and 2 above; or,
-
-    b) accompany it with a written offer, valid for at least three
-    years, to give any third party free (except for a nominal
-    shipping charge) a complete machine-readable copy of the
-    corresponding source code, to be distributed under the terms of
-    Paragraphs 1 and 2 above; or,
-
-    c) accompany it with the information you received as to where the
-    corresponding source code may be obtained.  (This alternative is
-    allowed only for noncommercial distribution and only if you
-    received the program in object code or executable form alone.)
-
-For an executable file, complete source code means all the source code for
-all modules it contains; but, as a special exception, it need not include
-source code for modules which are standard libraries that accompany the
-operating system on which the executable file runs.
-
-  4. You may not copy, sublicense, distribute or transfer this program
-except as expressly provided under this License Agreement.  Any attempt
-otherwise to copy, sublicense, distribute or transfer this program is void and
-your rights to use the program under this License agreement shall be
-automatically terminated.  However, parties who have received computer
-software programs from you with this License Agreement will not have
-their licenses terminated so long as such parties remain in full compliance.
-
-  5. If you wish to incorporate parts of this program into other free
-programs whose distribution conditions are different, write to the Free
-Software Foundation at 675 Mass Ave, Cambridge, MA 02139.  We have not yet
-worked out a simple rule that can be stated here, but we will often permit
-this.  We will be guided by the two goals of preserving the free status of
-all derivatives of our free software and of promoting the sharing and reuse of
-software.
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
 
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 In other words, you are welcome to use, share and improve this program.
 You are forbidden to forbid anyone else to use, share and improve
 what you give them.   Help stamp out software-hoarding!  */
 
+/*
+ * $Revision: 1.3 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/unexec.c,v $
+ * $Author: love $
+ * $Date: 92/03/10 16:44:23 $
+ */
+
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/03/10 16:44:23 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p1/src/RCS/unexec.c,v $ $Revision: 1.3 $";
+#endif
 
 /*
  * unexec.c - Convert a running program into an a.out file.
@@ -150,6 +77,10 @@ what you give them.   Help stamp out software-hoarding!  */
  *
  */
 
+/* Modified to support SysVr3 shared libraries by James Van Artsdalen
+ * of Dell Computer Corporation.  james@bigtex.cactus.org.
+ */
+
 /* There are several compilation parameters affecting unexec:
 
 * COFF
@@ -244,11 +175,8 @@ pointer looks like an int) but not on all machines.
 #ifndef CANNOT_DUMP  /* all rest of file!  */
 
 #ifndef CANNOT_UNEXEC /* most of rest of file */
-#ifdef NeXT
-#include "a.out.h"
-#else
+
 #include <a.out.h>
-#endif
 /* Define getpagesize () if the system does not.
    Note that this may depend on symbols defined in a.out.h
  */
@@ -264,19 +192,20 @@ pointer looks like an int) but not on all machines.
 extern char *start_of_text ();		/* Start of text */
 extern char *start_of_data ();		/* Start of initialized data */
 
+static int make_hdr (), copy_text_and_data (), copy_sym ();
+static int mark_x ();
+
 #ifdef COFF
 #ifndef USG
 #ifndef STRIDE
 #ifndef UMAX
 #ifndef sun386
-#ifndef m88k
 /* I have a suspicion that these are turned off on all systems
    and can be deleted.  Try it in version 19.  */
 #include <filehdr.h>
 #include <aouthdr.h>
 #include <scnhdr.h>
 #include <syms.h>
-#endif /* not m88k */
 #endif /* not sun386 */
 #endif /* not UMAX */
 #endif /* Not STRIDE */
@@ -343,11 +272,6 @@ static int unexec_data_start;
 #endif /* not COFF */
 
 static int pagemask;
-static int make_hdr ();
-static int copy_text_and_data ();
-static int copy_sym ();
-static void mark_x ();
-  
 
 /* Correct an int which is the bit pattern of a pointer to a byte
    into an int which is the number of a byte.
@@ -398,7 +322,7 @@ unexec (new_name, a_name, data_start, bss_start, entry_address)
      char *new_name, *a_name;
      unsigned data_start, bss_start, entry_address;
 {
-  int new = -1, a_out = -1;
+  int new, a_out = -1;
 
   if (a_name && (a_out = open (a_name, 0)) < 0)
     {
@@ -410,7 +334,7 @@ unexec (new_name, a_name, data_start, bss_start, entry_address)
     }
 
   if (make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name) < 0
-      || copy_text_and_data (new) < 0
+      || copy_text_and_data (new, a_out) < 0
       || copy_sym (new, a_out, a_name, new_name) < 0
 #ifdef COFF
       || adjust_lnnoptrs (new, a_out, new_name) < 0
@@ -450,7 +374,11 @@ make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name)
   auto struct scnhdr scntemp;		/* Temporary section header */
   register int scns;
 #endif /* COFF */
+#ifdef USG_SHARED_LIBRARIES
+  extern unsigned int bss_end;
+#else
   unsigned int bss_end;
+#endif
 
   pagemask = getpagesize () - 1;
 
@@ -543,7 +471,21 @@ make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name)
   /* Now we alter the contents of all the f_*hdr variables
      to correspond to what we want to dump.  */
 
+#ifdef USG_SHARED_LIBRARIES
+
+  /* The amount of data we're adding to the file is distance from the
+   * end of the original .data space to the current end of the .data
+   * space.
+   */
+
+  bias = bss_end - (f_ohdr.data_start + f_dhdr.s_size);
+
+#endif
+
   f_hdr.f_flags |= (F_RELFLG | F_EXEC);
+#ifdef TPIX
+  f_hdr.f_nscns = 3;
+#endif  
 #ifdef EXEC_MAGIC
   f_ohdr.magic = EXEC_MAGIC;
 #endif
@@ -586,7 +528,9 @@ make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name)
   f_bhdr.s_vaddr = f_ohdr.data_start + f_ohdr.dsize;
   f_bhdr.s_size = f_ohdr.bsize;
   f_bhdr.s_scnptr = 0L;
+#ifndef USG_SHARED_LIBRARIES
   bias = f_dhdr.s_scnptr + f_dhdr.s_size - block_copy_start;
+#endif
 
   if (f_hdr.f_symptr > 0L)
     {
@@ -612,6 +556,8 @@ make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name)
       PERROR (new_name);
     }
 
+#ifndef USG_SHARED_LIBRARIES
+
   if (write (new, &f_thdr, sizeof (f_thdr)) != sizeof (f_thdr))
     {
       PERROR (new_name);
@@ -626,6 +572,54 @@ make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name)
     {
       PERROR (new_name);
     }
+
+#else /* USG_SHARED_LIBRARIES */
+
+  /* The purpose of this code is to write out the new file's section
+   * header table.
+   *
+   * Scan through the original file's sections.  If the encountered
+   * section is one we know (.text, .data or .bss), write out the
+   * correct header.  If it is a section we do not know (such as
+   * .lib), adjust the address of where the section data is in the
+   * file, and write out the header.
+   *
+   * If any section preceeds .text or .data in the file, this code
+   * will not adjust the file pointer for that section correctly.
+   */
+
+  lseek (a_out, sizeof (f_hdr) + sizeof (f_ohdr), 0);
+
+  for (scns = f_hdr.f_nscns; scns > 0; scns--)
+    {
+      if (read (a_out, &scntemp, sizeof (scntemp)) != sizeof (scntemp))
+	PERROR (a_name);
+
+      if (!strcmp (scntemp.s_name, f_thdr.s_name))	/* .text */
+	{
+	  if (write (new, &f_thdr, sizeof (f_thdr)) != sizeof (f_thdr))
+	    PERROR (new_name);
+	}
+      else if (!strcmp (scntemp.s_name, f_dhdr.s_name))	/* .data */
+	{
+	  if (write (new, &f_dhdr, sizeof (f_dhdr)) != sizeof (f_dhdr))
+	    PERROR (new_name);
+	}
+      else if (!strcmp (scntemp.s_name, f_bhdr.s_name))	/* .bss */
+	{
+	  if (write (new, &f_bhdr, sizeof (f_bhdr)) != sizeof (f_bhdr))
+	    PERROR (new_name);
+	}
+      else
+	{
+	  if (scntemp.s_scnptr)
+	    scntemp.s_scnptr += bias;
+	  if (write (new, &scntemp, sizeof (scntemp)) != sizeof (scntemp))
+	    PERROR (new_name);
+	}
+    }
+#endif /* USG_SHARED_LIBRARIES */
+
   return (0);
 
 #else /* if not COFF */
@@ -695,13 +689,71 @@ make_hdr (new, a_out, data_start, bss_start, entry_address, a_name, new_name)
  * Copy the text and data segments from memory to the new a.out
  */
 static int
-copy_text_and_data (new)
-     int new;
+copy_text_and_data (new, a_out)
+     int new, a_out;
 {
   register char *end;
   register char *ptr;
 
 #ifdef COFF
+
+#ifdef USG_SHARED_LIBRARIES
+
+  int scns;
+  struct scnhdr scntemp;		/* Temporary section header */
+
+  /* The purpose of this code is to write out the new file's section
+   * contents.
+   *
+   * Step through the section table.  If we know the section (.text,
+   * .data) do the appropriate thing.  Otherwise, if the section has
+   * no allocated space in the file (.bss), do nothing.  Otherwise,
+   * the section has space allocated in the file, and is not a section
+   * we know.  So just copy it.
+   */
+
+  lseek (a_out, sizeof (struct filehdr) + sizeof (struct aouthdr), 0);
+
+  for (scns = f_hdr.f_nscns; scns > 0; scns--)
+    {
+      if (read (a_out, &scntemp, sizeof (scntemp)) != sizeof (scntemp))
+	PERROR ("temacs");
+
+      if (!strcmp (scntemp.s_name, ".text"))
+	{
+	  lseek (new, (long) text_scnptr, 0);
+	  ptr = (char *) f_ohdr.text_start;
+	  end = ptr + f_ohdr.tsize;
+	  write_segment (new, ptr, end);
+	}
+      else if (!strcmp (scntemp.s_name, ".data"))
+	{
+	  lseek (new, (long) data_scnptr, 0);
+	  ptr = (char *) f_ohdr.data_start;
+	  end = ptr + f_ohdr.dsize;
+	  write_segment (new, ptr, end);
+	}
+      else if (!scntemp.s_scnptr)
+	; /* do nothing - no data for this section */
+      else
+	{
+	  char page[BUFSIZ];
+	  int size, n;
+	  long old_a_out_ptr = lseek (a_out, 0, 1);
+
+	  lseek (a_out, scntemp.s_scnptr, 0);
+	  for (size = scntemp.s_size; size > 0; size -= sizeof (page))
+	    {
+	      n = size > sizeof (page) ? sizeof (page) : size;
+	      if (read (a_out, page, n) != n || write (new, page, n) != n)
+		PERROR ("xemacs");
+	    }
+	  lseek (a_out, old_a_out_ptr, 0);
+	}
+    }
+
+#else /* COFF, but not USG_SHARED_LIBRARIES */
+
   lseek (new, (long) text_scnptr, 0);
   ptr = (char *) f_ohdr.text_start;
   end = ptr + f_ohdr.tsize;
@@ -712,6 +764,8 @@ copy_text_and_data (new)
   end = ptr + f_ohdr.dsize;
   write_segment (new, ptr, end);
 
+#endif /* USG_SHARED_LIBRARIES */
+
 #else /* if not COFF */
 
 /* Some machines count the header as part of the text segment.
@@ -836,8 +890,7 @@ copy_sym (new, a_out, a_name, new_name)
  *
  * After succesfully building the new a.out, mark it executable
  */
-static
-void
+static int
 mark_x (name)
      char *name;
 {
@@ -919,7 +972,7 @@ adjust_lnnoptrs (writedesc, readdesc, new_name)
 	  }
 	}
     }
-  return close (new);
+  close (new);
 }
 
 #endif /* COFF */
