@@ -1,23 +1,28 @@
-/* Window definitions for GNU Emacs.
+/* Window definitions for GNU Emacs. 
    Copyright (C) 1985, 1986 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
+GNU Emacs is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 1, or (at your option)
+any later version.
+
 GNU Emacs is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY.  No author or distributor
-accepts responsibility to anyone for the consequences of using it
-or for whether it serves any particular purpose or works at all,
-unless he says so in writing.  Refer to the GNU Emacs General Public
-License for full details.
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
 
-Everyone is granted permission to copy, modify and redistribute
-GNU Emacs, but only under the conditions described in the
-GNU Emacs General Public License.   A copy of this license is
-supposed to have been given to you along with GNU Emacs so you
-can know your rights and responsibilities.  It should be in a
-file named COPYING.  Among other things, the copyright notice
-and this notice must be preserved on all copies.  */
+You should have received a copy of the GNU General Public License
+along with GNU Emacs; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+/*
+ * $Revision: 1.7 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/window.h,v $
+ * $Date: 92/02/23 11:03:38 $
+ * $Author: love $
+ */
 
 /* Windows are allocated as if they were vectors, but then the
 Lisp data type is changed to Lisp_Window.  They are garbage
@@ -86,12 +91,18 @@ struct window
     /* The window this one is a child of. */
     Lisp_Object parent;
     /* The upper left corner coordinates of this window,
-       as integers relative to upper left corner of screen = 0, 0 */
-    Lisp_Object left;
-    Lisp_Object top;
-    /* The size of the window */
-    Lisp_Object height;
-    Lisp_Object width;
+       as integers (pixels) relative to upper left corner of screen */
+    Lisp_Object pixleft;
+    Lisp_Object pixtop;
+    /* The size of this window (pixels) */
+    Lisp_Object pixwidth;
+    Lisp_Object pixheight;
+    /* Display structures for this window */
+    Lisp_Object lines;
+    /* Display structure for modeline */
+    Lisp_Object modeline;
+    /* Has window changed in size since last redisplay? */
+    Lisp_Object size_change;
     /* The buffer displayed in this window */
     /* Of the fields vchild, hchild and buffer, only one is non-nil.  */
     Lisp_Object buffer;
@@ -105,8 +116,7 @@ struct window
     /* Non-nil means next redisplay must use the value of start
        set up for it in advance.  Set by scrolling commands.  */
     Lisp_Object force_start;
-    /* Number of columns display within the window is scrolled to the left.
-       Currently always zero since this is not implemented.  */
+    /* Number of columns display within the window is scrolled to the left.  */
     Lisp_Object hscroll;
     /* Number saying how recently window was selected */
     Lisp_Object use_time;
@@ -136,27 +146,29 @@ struct window
     Lisp_Object window_end_valid;
     /* Vertical position (relative to window top) of that buffer position
        of the first of those characters */
-    Lisp_Object window_end_vpos;
+    Lisp_Object window_end_vpos; /* lines */
+    Lisp_Object window_end_ppos; /* pixels */
     /* Non-nil means must regenerate mode line of this window */
-    Lisp_Object redo_mode_line;
+    Lisp_Object update_mode_line;
     /* Non-nil means current value of `start'
        was the beginning of a line when it was chosen.  */
     Lisp_Object start_at_line_beg;
-    /* pointer to the root block that describe the sheet this window
-     * is in
-     */
+    /* Pointer to root block containing this window. */
     Lisp_Object root;
+    /* # of lines in window; only meaningful if window is "full" */
+    Lisp_Object height;
+    Lisp_Object start_button;
   };
 
 /* This is the window which displays the minibuffer.
-It is always the same window.  */
+   It is always the same window.  */
 
 extern Lisp_Object minibuf_window;
 
 /* This is the window in which the terminal's cursor should
- be left when nothing is being done with it.  This must
- always be a leaf window, and its buffer is selected by
- the top level editing loop at the end of each command.  */
+   be left when nothing is being done with it.  This must
+   always be a leaf window, and its buffer is selected by
+   the top level editing loop at the end of each command.  */
 
 extern Lisp_Object selected_window;
 
@@ -169,80 +181,42 @@ extern Lisp_Object Fselect_window ();
 extern Lisp_Object Fdisplay_buffer ();
 extern Lisp_Object Fset_window_buffer ();
 
-extern char *minibuf_prompt;	/* Prompt to display in front of the minibuffer contents */
+/* Prompt to display in front of the minibuffer contents.  */
+extern char *minibuf_prompt;
 
-extern char *minibuf_message;	/* Message to display instead of minibuffer contents */
-				/* This is what the functions error and message make, */
-				/* and command echoing uses it as well. */
-				/* It overrides the minibuf_prompt as well as the buffer */
+/* Message to display instead of minibuffer contents.  */
+extern char *echo_area_contents;
 
-extern int RecurseDepth;	/* Depth in recursive edits */
+/* Last message that was displayed in the echo area.  */
+extern char *prev_echo_area_contents;
 
-extern int MinibufDepth;	/* Depth in minibuffer invocations */
+/* Depth in minibuffer invocations.  */
+extern int minibuf_level;
 
-extern int RedoModes;		/* true iff we should redraw the mode lines
-				   on the next redisplay */
+/* Nonzero means redisplay all mode lines.  */
+extern int update_mode_lines;
 
-/* Minimum value of bf_s1 since last redisplay that finished.
- Valid for current buffer unless Cant1WinOpt is nonzero. */
+/* Minimum value of GPT since last redisplay that finished.  */
 
 extern int beg_unchanged;
 
-/* Minimum value of bf_s2 since last redisplay that finished.
- Valid for current buffer unless Cant1WinOpt is nonzero. */
+/* Minimum value of Z-GPT since last redisplay that finished.  */
 
 extern int end_unchanged;
 
-/* bf_modified as of last redisplay that finished;
- if it matches bf_modified, beg_unchanged and end_unchanged
- contain no useful information */
+/* MODIFF as of last redisplay that finished;
+   if it matches MODIFF, beg_unchanged and end_unchanged
+   contain no useful information */
 extern int unchanged_modified;
 
-/* Nonzero if head_clip or tail_clip of current buffer has changed
- since last redisplay that finished */
+/* Nonzero if BEGV-BEG or Z-ZV of current buffer has changed
+   since last redisplay that finished */
 extern int clip_changed;
 
 /* Nonzero if window sizes or contents have changed
- since last redisplay that finished */
+   since last redisplay that finished */
 extern int windows_or_buffers_changed;
 
 /* Number of windows displaying the selected buffer.
    Normally this is 1, but it can be more.  */
 extern int buffer_shared;
-
-/* declarations & interface to cheap window switching */
-extern int window_switched;
-
-struct WINDOW_SWITCH_data {
-  int flag;
-  Lisp_Object selected_window;
-  Lisp_Object current_buffer;
-  int current_point;
-  int last_known_column;
-  int last_known_column_point;
-  int last_known_column_modified;
-  struct WINDOW_SWITCH_data *prev;
-  int count;
-};
-
-extern struct WINDOW_SWITCH_data * window_switch_data;
-
-#define WINDOW_DECLARE() \
-  struct WINDOW_SWITCH_data WINDOW_SWITCH_sav
-
-extern VOID window_switch();
-extern Lisp_Object window_switch_unwind();
-
-/* when doing a window switch, you should not do anything really drastic
- * like killing the buffer or the window that was saved. I should protect
- * against that, but I am not going to right now.
- */
-
-#define WINDOW_SWITCH(w) \
-  { if ((w) == selected_window && XBUFFER(XWINDOW(w)->buffer) == bf_cur) \
-      WINDOW_SWITCH_sav.flag = 0; \
-    else \
-      window_switch ((w),&WINDOW_SWITCH_sav); }
-
-#define WINDOW_RESTORE() \
-  { if (WINDOW_SWITCH_sav.flag) unbind_to (WINDOW_SWITCH_sav.count); }
