@@ -1,7 +1,16 @@
 /* Source code copyright 1989 by Alan M. Carroll, all rights reserved.
  * This code may be freely distributed as long as the copyright is preserved.
+ *
+ * Epoch 4.0 : Code for Elisp event handling
+ *
+ * $Revision: 1.4 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/xevent.c,v $
+ * $Author: love $
+ * $Date: 91/05/07 23:09:21 $
  */
-/* Code for Elisp event handling */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 91/05/07 23:09:21 $ $Source: /import/kaplan/stable/distrib/epoch-4.0/src/RCS/xevent.c,v $ $Revision: 1.4 $";
+#endif
 
 #include <stdio.h>
 #undef NULL
@@ -19,6 +28,9 @@
 
 #include "screen.h"
 #include "screenX.h"
+#include "screenW.h"
+#include "dispepoch.h"
+#include "button.h"
 
 #include "xresource.h"
 
@@ -30,6 +42,7 @@ extern Atom XA_current;                         /* current property atom */
 extern Atom XA_screen_id;                       /* root block sequence # */
 
 extern Lisp_Object ConvertXtoE();
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Q definitions */
 
@@ -47,6 +60,9 @@ Lisp_Object Vx_mouse_events;
 
 Lisp_Object Qx_property_change,Qx_focus,Qx_map,Qx_move,Qx_resize;
 Lisp_Object Qx_client_message,Qx_motion,Qx_button;
+Lisp_Object Qx_selection_clear;
+Lisp_Object Qx_selection_request;
+Lisp_Object Qx_selection_notify;
 #ifdef DENYS_ALARM
 Lisp_Object Qx_timer;
 #endif /* DENYS_ALARM */
@@ -127,73 +143,131 @@ ReadClientMessage(cm,xs)
     }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static Lisp_Object
+ReadSelectionClear(sc,xs)
+     XSelectionClearEvent *sc;
+     struct X_Screen *xs;
+{
+  Lisp_Object result;
+  char *atom_name;
+  BLOCK_INPUT_DECLARE ();
+
+  if (!NULL(Vx_lazy_events))
+    result = make_Xresource(xs->display,xs->plane,sc->selection,XA_ATOM);
+  else
+    {
+      BLOCK_INPUT();
+      atom_name = XGetAtomName(xs->display,sc->selection);
+      UNBLOCK_INPUT();
+      /* didn't get a name, blow this one off */
+      if (atom_name == (char *)0) return Qnil;
+
+      result = build_string(atom_name);
+      XFree(atom_name);
+    }
+  
+  return result;
+}
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+static Lisp_Object
+ReadSelectionNotify(sn,xs)
+     XSelectionEvent *sn;
+     struct X_Screen *xs;
+{
+  Lisp_Object result;
+  BLOCK_INPUT_DECLARE();
+  char *req,*sel,*tar,*pro;
+  
+  if (!NULL(Vx_lazy_events))
+    {
+      result = Fcons(make_Xresource(xs->display,xs->plane,
+				    sn->requestor,XA_WINDOW),
+		     Fcons(make_Xresource(xs->display,xs->plane,
+					  sn->selection,XA_ATOM),
+			   Fcons(make_Xresource(xs->display,xs->plane,
+						sn->target,XA_ATOM),
+				 Fcons(make_Xresource(xs->display,xs->plane,
+						      sn->property,XA_ATOM),
+				       Qnil))));
+    }
+  else
+    {
+      BLOCK_INPUT();
+      req = XGetAtomName(xs->display,sn->requestor);
+      sel = XGetAtomName(xs->display,sn->selection);
+      tar = XGetAtomName(xs->display,sn->target);
+      pro = XGetAtomName(xs->display,sn->property);
+      UNBLOCK_INPUT();
+
+      result = Fcons(build_string(req),
+		     Fcons(build_string(sel),
+			   Fcons(build_string(tar),
+				 Fcons(build_string(pro),
+				       Qnil))));
+    }
+  
+  return result;
+}
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+static Lisp_Object
 ReadPropertyEvent(pe,xs)
         XPropertyEvent *pe;
         struct X_Screen *xs;
+{
+  Lisp_Object result,value;
+  char * atom_name,lookup=0;
+  BLOCK_INPUT_DECLARE();
+
+  if (!NULL(Vx_lazy_events))
+    return make_Xresource(xs->display,xs->plane,pe->atom,XA_ATOM);
+    
+  /* convert the atom to a string: check for known atoms */
+  if (pe->atom == XA_current)
+    atom_name = XA_EPOCH_CURRENT;
+  else if (pe->atom == XA_screen_id)
+    atom_name = XA_EPOCH_SCREEN_ID;
+  else
     {
-    Lisp_Object result,value;
-    char * atom_name,lookup=0;
-    BLOCK_INPUT_DECLARE();
-
-    if (!NULL(Vx_lazy_events))
-      return make_Xresource(xs->display,xs->plane,pe->atom,XA_ATOM);
-    
-    /* convert the atom to a string: check for known atoms */
-    if (pe->atom == XA_current)
-        atom_name = XA_EPOCH_CURRENT;
-    else if (pe->atom == XA_screen_id)
-        atom_name = XA_EPOCH_SCREEN_ID;
-    else
-        {
-        BLOCK_INPUT();
-        atom_name = XGetAtomName(xs->display,pe->atom);
-        UNBLOCK_INPUT();
-        lookup = 1;
-        }
-    
-    /* didn't get a name, blow this one off */
-    if (atom_name == (char *)0) return Qnil;
-
-    value = raw_get_property(xs->display,xs->xid,pe->atom);
-    result = Fcons(build_string(atom_name),value);
-
-    if (lookup) XFree(atom_name);
-
-    return result;
+      BLOCK_INPUT();
+      atom_name = XGetAtomName(xs->display,pe->atom);
+      UNBLOCK_INPUT();
+      lookup = 1;
     }
+    
+  /* didn't get a name, blow this one off */
+  if (atom_name == (char *)0) return Qnil;
+
+  value = raw_get_property(xs->display,xs->xid,pe->atom);
+  result = Fcons(build_string(atom_name),value);
+
+  if (lookup) XFree(atom_name);
+
+  return result;
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static Lisp_Object
-ReadMotionEvent(me,xs)
-        XMotionEvent *me;
-        struct X_Screen *xs;
-    {
-    int cx,cy;                          /* character locations */
+ReadMotionEvent(me,xs,rb)
+     XMotionEvent *me;
+     struct X_Screen *xs;
+     struct Root_Block *rb;
+{
+  return Fcons(make_number(me->x),
+	       Fcons(make_number(me->y),
+		     Fcons(make_number(me->state),Qnil)));
 
-    cx = (me->x - xs->in_border) / xs->font_width;
-    cy = (me->y - xs->in_border) / xs->font_height;
-
-    return Fcons(make_number(cx),
-                 Fcons(make_number(cy),
-                       Fcons(make_number(me->state),Qnil)));
-
-    }
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static Lisp_Object
-ReadButtonEvent(be,xs)
-        XButtonEvent *be;
-        struct X_Screen *xs;
-    {
-    int cx,cy;                          /* character locations */
-
-    cx = (be->x - xs->in_border) / xs->font_width;
-    cy = (be->y - xs->in_border) / xs->font_height;
-
-    return Fcons(be->type == ButtonPress ? Qt : Qnil,
-                 Fcons(make_number(cx),
-                       Fcons(make_number(cy),
-                             Fcons(make_number(be->button),
-                                   Fcons(make_number(be->state),Qnil)))));
-    }
+ReadButtonEvent(be,xs,rb)
+     XButtonEvent *be;
+     struct X_Screen *xs;
+     struct Root_Block *rb;
+{
+  return Fcons(be->type == ButtonPress ? Qt : Qnil,
+	       Fcons(make_number(be->x),
+		     Fcons(make_number(be->y),
+			   Fcons(make_number(be->button),
+				 Fcons(make_number(be->state),Qnil)))));
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* Returned value is a vector of size 3. The elements are
  * 0 : A symbol indicating the type of event
@@ -203,15 +277,12 @@ ReadButtonEvent(be,xs)
 DEFUN ("epoch::get-event",Fget_event,Sget_event,
       0,0,0,
 "Get the next X-window event off the queue.\
-The returned value is nil or a 3-vector consisting of\n\
+The returned value is nil or a 4-vector consisting of\n\
 0 : A symbol indicating the type of event\n\
 1 : The value of the event (type dependent)\n\
-2 : The screen the event occured on") ()
+2 : The screen the event occured on\n\
+3 : The X time stamp of the event (or nil)") ()
 {
-  /*
-    Lisp_Object evarray = Fmake_vector(make_number(3),Qnil);
-    struct Lisp_Vector *evp = XVECTOR(evarray);
-    */
   Lisp_Object evarray;
   struct Lisp_Vector *evp;
   char * atom_name;
@@ -223,7 +294,7 @@ The returned value is nil or a 3-vector consisting of\n\
 
   /* make sure the epoch::event has a vector that's big enough */
   if (XTYPE(Vepoch_event) != Lisp_Vector || XVECTOR(Vepoch_event)->size < 3)
-    Vepoch_event = Fmake_vector(3,Qnil);
+    Vepoch_event = Fmake_vector(4,Qnil);
   evp = XVECTOR(Vepoch_event);
 
   BLOCK_INPUT ();                     /* let's not have race conditions */
@@ -244,12 +315,14 @@ The returned value is nil or a 3-vector consisting of\n\
 
   xs = XXSCREEN(rb->x11);
   XSET(evp->contents[2],Lisp_Root_Block,rb);
+  evp->contents[3] = Qnil;	/* default - no valid timestamp */
 
   switch (xp->xany.type)
     {
     case PropertyNotify :
       evp->contents[0] = Qx_property_change;
       evp->contents[1] = ReadPropertyEvent((XPropertyEvent *)xp,xs);
+      evp->contents[3] = make_number(xp->xproperty.time);
       break;
 
       /* Focus and Leave/Enter events aren't queued unles they're
@@ -281,13 +354,27 @@ The returned value is nil or a 3-vector consisting of\n\
 
     case MotionNotify :
       evp->contents[0] = Qx_motion;
-      evp->contents[1] = ReadMotionEvent((XMotionEvent *)xp,xs);
+      evp->contents[1] = ReadMotionEvent((XMotionEvent *)xp,xs,rb);
+      evp->contents[3] = make_number(xp->xmotion.time);
       break;
 
+    case SelectionClear :
+      evp->contents[0] = Qx_selection_clear;
+      evp->contents[1] = ReadSelectionClear((XSelectionClearEvent *)xp,xs);
+      evp->contents[3] = make_number(xp->xselectionclear.time);
+      break;
+
+    case SelectionNotify :
+      evp->contents[0] = Qx_selection_notify;
+      evp->contents[1] = ReadSelectionNotify((XSelectionEvent *)xp,xs);
+      evp->contents[3] = make_number(xp->xselection.time);
+      break;
+      
     case ButtonPress :
     case ButtonRelease :
       evp->contents[0] = Qx_button;
-      evp->contents[1] = ReadButtonEvent((XButtonEvent *)xp,xs);
+      evp->contents[1] = ReadButtonEvent((XButtonEvent *)xp,xs,rb);
+      evp->contents[3] = make_number(xp->xbutton.time);
       break;
     }
 
@@ -326,52 +413,58 @@ epoch_event_dispatch()
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 void syms_of_xevent()
-    {
+{
 
-    defsubr(&Sget_event);
+  defsubr(&Sget_event);
 
-    DEFVAR_LISP("epoch::event-handler",&Vx_event_handler,
-"If this variable is not nil, then it is assumed to have \
+  DEFVAR_LISP("epoch::event-handler",&Vx_event_handler,
+	      "If this variable is not nil, then it is assumed to have \
 a function in it. When a property change on a screen is received, \
 it is stored in an internal queue. Whenever the editor is waiting for \
 input, and there are events in the queue, the function is called. \
 Use *x-get-property-event* to retrieve the events." );
 
-    DEFVAR_LISP("epoch::event",&Vepoch_event,
-"Bound to the value of the current event when epoch::event-handler is called");
+  DEFVAR_LISP("epoch::event",&Vepoch_event,
+	      "Bound to the value of the current event when epoch::event-handler is called");
 
-    DEFVAR_LISP("epoch::event-handler-abort",&Vx_event_handler_abort,
-"If this variable is not nil, then any error in the X event handler\
+  DEFVAR_LISP("epoch::event-handler-abort",&Vx_event_handler_abort,
+	      "If this variable is not nil, then any error in the X event handler\
 will cause the variable epoch::event-handler to become nil, to prevent runaway\
 error lockup. If epoch::event-handler-abort is nil, only the current\
 call to the epoch::event-handler function will be aborted on error");
 
-    Vx_event_handler = Qnil;
-    x_in_handler_hook = 0;
-    Vx_event_handler_abort = Qt;
+  Vx_event_handler = Qnil;
+  x_in_handler_hook = 0;
+  Vx_event_handler_abort = Qt;
 
-    DEFVAR_LISP("epoch::lazy-events",&Vx_lazy_events,
-"If this variable is non-nil, then the internal event code will do as little\
+  DEFVAR_LISP("epoch::lazy-events",&Vx_lazy_events,
+	      "If this variable is non-nil, then the internal event code will do as little\
  work as possible. For example, property events will only report the atom\
  of the property changed, and leave it up to other code to acquire the value\
  if desired. NOT INSTALLED YET");
-    Vx_lazy_events = Qnil;
+  Vx_lazy_events = Qnil;
 
-    DEFVAR_LISP("epoch::mouse-events",&Vx_mouse_events,
-"If this variable is non-nil, then mouse events are placed into the Epoch\
+  DEFVAR_LISP("epoch::mouse-events",&Vx_mouse_events,
+	      "If this variable is non-nil, then mouse events are placed into the Epoch\
  event Q instead of generating fake key strokes");
-    Vx_mouse_events = Qnil;
+  Vx_mouse_events = Qnil;
 
-    Qx_property_change = intern("property-change");     staticpro(&Qx_property_change);
-    Qx_focus = intern("focus");                         staticpro(&Qx_focus);
-    Qx_map = intern("map");                             staticpro(&Qx_map);
-    Qx_move = intern("move");                           staticpro(&Qx_move);
-    Qx_resize = intern("resize");                       staticpro(&Qx_resize);
-    Qx_client_message = intern("client-message");       staticpro(&Qx_client_message);
-    Qx_motion = intern("motion");                       staticpro(&Qx_motion);
-    Qx_button = intern("button");                       staticpro(&Qx_button);
+  Qx_property_change = intern("property-change");     staticpro(&Qx_property_change);
+  Qx_focus = intern("focus");                         staticpro(&Qx_focus);
+  Qx_map = intern("map");                             staticpro(&Qx_map);
+  Qx_move = intern("move");                           staticpro(&Qx_move);
+  Qx_resize = intern("resize");                       staticpro(&Qx_resize);
+  Qx_client_message = intern("client-message");       staticpro(&Qx_client_message);
+  Qx_motion = intern("motion");                       staticpro(&Qx_motion);
+  Qx_button = intern("button");                       staticpro(&Qx_button);
 
+  Qx_selection_clear = intern("selection-clear");	staticpro(&Qx_selection_clear);
+  Qx_selection_request = intern("selection-request");	staticpro(&Qx_selection_request);
+  Qx_selection_notify = intern("selection-notify");	staticpro(&Qx_selection_notify);
+    
 #ifdef DENYS_ALARM
-    Qx_timer = intern("timer");                         staticpro(&Qx_timer);
-#endif /* DENYS_ALARM */
-    }
+  Qx_timer = intern("timer");                         staticpro(&Qx_timer);
+#endif				/* DENYS_ALARM */
+}
+
+
