@@ -19,6 +19,15 @@ can know your rights and responsibilities.  It should be in a
 file named COPYING.  Among other things, the copyright notice
 and this notice must be preserved on all copies.  */
 
+/*
+ * $Revision: 1.20 $
+ * $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/xutil.c,v $
+ * $Date: 92/03/22 15:45:42 $
+ * $Author: love $
+ */
+#ifndef LINT
+static char rcsid[] = "$Author: love $ $Date: 92/03/22 15:45:42 $ $Source: /import/kaplan/stable/distrib/epoch-4.0p0/src/RCS/xutil.c,v $ $Revision: 1.20 $";
+#endif
 
 #include <stdio.h>
 #undef NULL
@@ -33,22 +42,35 @@ and this notice must be preserved on all copies.  */
 #include "config.h"
 #include "lisp.h"
 #include "x11term.h"
-#include "dispextern.h"
+#include "buffer.h"
+#include "window.h"
 #include "screen.h"
 #include "screenW.h"
 #include "screenX.h"
+#include "dispepoch.h"
 #include "xdefault.h"
 #include "xresource.h"
+#include "button.h"
+
+/* X11 includes used; use NIL rather than NULL from lisp.h */
 
 extern int in_display;
+extern int font_change;
 extern int interrupt_input;
+extern char *epoch_get_default();
+extern char *strdup();
+extern struct Root_Block *x_find_screen();
+extern int distinct_minibuffer;
+extern int replot_attributes;
+
 void DEBUG();
 
 /* variables first */
 
 extern struct X_Screen *cur_Xscreen;    /* current active screen */
 extern struct W_Screen *cur_Wscreen;
-extern struct Root_Block *cur_root;
+extern struct Root_Block *cur_root,*root,*mini_root;
+extern Lisp_Object minibuf_rootblock,rootblock;
 
 /* Default values - set up Lisp Objects later to point into these */
 extern Display *XD_display;                    /* default display */
@@ -57,18 +79,11 @@ extern char * XD_display_name;                 /* its name */
 extern char * XD_resource_name;                /* process resource name */
 extern char * XD_resource_class;                /* resource class */
 extern int XD_plane;
+extern char XD_use_defaults;	/* Read the .Xdefaults file? */
 
 extern Atom XA_current;                 /* current property atom */
 extern Atom XA_rootblock;                       /* root block sequence # */
 
-/* mini buffer statics -
- * The mini buff serves as an anchor for the whole display structure.
- * Minibuff->root is the root structure for the minibuff, and it has
- * pointers into a ring of all the other root blocks
- */
-extern struct Root_Block *mini_root;
-extern Lisp_Object minibuf_rootblock;
-
 extern Lisp_Object Vx_screen_properties;
 
 /* external object */
@@ -91,12 +106,12 @@ extern Lisp_Object Qx_motion_hints;
 extern Lisp_Object Qx_parent;
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 struct Alist_Entry
-    {
-    Lisp_Object *symbol;
-    char *name;
-    VOID *slot;
-    XDefaultType type;
-    } ;
+{
+  Lisp_Object *symbol;
+  char *name;
+  VOID *slot;
+  XDefaultType type;
+} ;
 
 /* static variables for use in alist reader */
 
@@ -131,68 +146,68 @@ static struct Alist_Entry alistkey[] = {
  * put the values set into the default structure
  */
 void x_read_screen_alist(xd,alist)
+        struct Default_Set *xd;     
         Lisp_Object alist;
-        struct Default_Set *xd;
+{
+  register int ai;
+  register struct Alist_Entry *ap;
+  Lisp_Object cpair,cvalue,ckey;
+  char aflag[alist_size];
+
+  memset(aflag,0,sizeof(aflag)); /* clear flags */
+
+  /* because we need to store the address of the members, we have to
+   * copy the structure over, fill it in, and then copy it back
+   */
+  memcpy(&alist_xd,xd,sizeof(alist_xd));
+
+  for (; !EQ(Qnil,alist) && XTYPE(alist) == Lisp_Cons ; alist = Fcdr(alist))
     {
-    register int ai;
-    register struct Alist_Entry *ap;
-    Lisp_Object cpair,cvalue,ckey;
-    char aflag[alist_size];
 
-    memset(aflag,0,sizeof(aflag)); /* clear flags */
+      QUIT;			/* allow emergency exit */
 
-    /* because we need to store the address of the members, we have to
-     * copy the structure over, fill it in, and then copy it back
-     */
-    memcpy(&alist_xd,xd,sizeof(alist_xd));
+      cpair = XCONS(alist)->car;
+      if (XTYPE(cpair) != Lisp_Cons) continue;
+      ckey = XCONS(cpair)->car;
+      cvalue = XCONS(cpair)->cdr;
 
-    for (; !EQ(Qnil,alist) && XTYPE(alist) == Lisp_Cons ; alist = Fcdr(alist))
-        {
-
-        QUIT;                   /* allow emergency exit */
-
-        cpair = XCONS(alist)->car;
-        if (XTYPE(cpair) != Lisp_Cons) continue;
-        ckey = XCONS(cpair)->car;
-        cvalue = XCONS(cpair)->cdr;
-
-        for (ai = 0 , ap = alistkey; ai < alist_size ; ++ai, ++ap)
-            {
-            if (EQ(ckey,*(ap->symbol)) && !aflag[ai])
-                {
-                switch (ap->type)
-                    {
-                    case XDT_Int :
-                        if (XTYPE(cvalue) != Lisp_Int)
-                            error("%s must be an integer",ap->name);
-                        *( (int *) (ap->slot) ) = XFASTINT(cvalue);
-                        break;
-                    case XDT_String :
-                        if (XTYPE(cvalue) != Lisp_String)
-                            error("%s must be a string",ap->name);
-                        *( (unsigned char **) (ap->slot) )
-                            = XSTRING(cvalue)->data;
-                        break;
-                    case XDT_Flag :
-                        *((char *)(ap->slot)) = EQ(Qnil,cvalue) ? 0 : 1;
-                        break;
-                    case XDT_Window :
-                        if (XTYPE(cvalue) != Lisp_Xresource ||
-                            XXRESOURCE(cvalue)->type != XA_WINDOW)
-                            error("Parent must be X-resource window");
-                        *((Window *)(ap->slot)) = XXRESOURCE(cvalue)->id;
-                        break;
-                    }
-                aflag[ai] = 1;
-                break;          /* leave the for loop */
-                }
-            }
-        }
-
-    /* one bit of special handling */
-
-    memcpy(xd,&alist_xd,sizeof(alist_xd));
+      for (ai = 0 , ap = alistkey; ai < alist_size ; ++ai, ++ap)
+	{
+	  if (EQ(ckey,*(ap->symbol)) && !aflag[ai])
+	    {
+	      switch (ap->type)
+		{
+		case XDT_Int :
+		  if (XTYPE(cvalue) != Lisp_Int)
+		    error("%s must be an integer",ap->name);
+		  *( (int *) (ap->slot) ) = XFASTINT(cvalue);
+		  break;
+		case XDT_String :
+		  if (XTYPE(cvalue) != Lisp_String)
+		    error("%s must be a string",ap->name);
+		  *( (unsigned char **) (ap->slot) )
+		    = XSTRING(cvalue)->data;
+		  break;
+		case XDT_Flag :
+		  *((char *)(ap->slot)) = EQ(Qnil,cvalue) ? 0 : 1;
+		  break;
+		case XDT_Window :
+		  if (XTYPE(cvalue) != Lisp_Xresource ||
+		      XXRESOURCE(cvalue)->type != XA_WINDOW)
+		    error("Parent must be X-resource window");
+		  *((Window *)(ap->slot)) = XXRESOURCE(cvalue)->id;
+		  break;
+		}
+	      aflag[ai] = 1;
+	      break;		/* leave the for loop */
+	    }
+	}
     }
+
+  /* one bit of special handling */
+
+  memcpy(xd,&alist_xd,sizeof(alist_xd));
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 /* various utility functions to change the attributes of a screen */
 extern Lisp_Object Fepoch_get_color();
@@ -209,9 +224,11 @@ DEFUN ("epoch::color-components",
   if (!XRESOURCEP(color) || XXRESOURCE(color)->type != XA_CARDINAL)
     error("Color argument should be an X-cardinal resource");
 
-  c.pixel = cur_Xscreen->foreground;
+  c.pixel = XXRESOURCE(color)->id;
   BLOCK_INPUT();
-  XQueryColor(cur_Xscreen->display,cur_Xscreen->colormap,&c);
+  XQueryColor(XXRESOURCE(color)->dpy,
+	      DefaultColormap(XXRESOURCE(color)->dpy,
+			      XXRESOURCE(color)->plane), &c);
   UNBLOCK_INPUT();
   result = Fmake_vector(make_number(3),Qnil);
   XVECTOR(result)->contents[0] = make_number(c.red);
@@ -235,7 +252,7 @@ DEFUN ("epoch::foreground",Fx_foreground,Sx_foreground,0,2,0,
   xs = XXSCREEN(XROOT(block)->x11);
 
   if (EQ(Qnil,color))
-    return make_Xresource(xs->display,xs->plane,xs->foreground,XA_CARDINAL);
+    return XSTYLE(XROOT(block)->stylenorm)->foreground;
 
   color = Fepoch_get_color(color);
   if (EQ(Qnil,color)) return Qnil;
@@ -244,8 +261,8 @@ DEFUN ("epoch::foreground",Fx_foreground,Sx_foreground,0,2,0,
   XSetForeground(xs->display,xs->gc_norm,pixel);
   XSetBackground(xs->display,xs->gc_rev,pixel);
   result = Qt;
-  xs->foreground = pixel;
-
+  XSTYLE(XROOT(block)->stylenorm)->foreground =
+    make_Xresource(xs->display,xs->plane,pixel,XA_CARDINAL);
   return result;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -264,7 +281,7 @@ DEFUN ("epoch::background",Fx_background,Sx_background,0,2,0,
   xs = XXSCREEN(XROOT(block)->x11);
 
   if (EQ(Qnil,color))
-    return make_Xresource(xs->display,xs->plane,xs->background,XA_CARDINAL);
+    return XSTYLE(XROOT(block)->stylenorm)->background;
 
   color = Fepoch_get_color(color);
   if (EQ(Qnil,color)) return Qnil;
@@ -276,8 +293,8 @@ DEFUN ("epoch::background",Fx_background,Sx_background,0,2,0,
   attribs.background_pixel = pixel;
   XChangeWindowAttributes(xs->display,xs->xid,CWBackPixel,&attribs);
   result = Qt;
-  xs->background = pixel;
-
+  XSTYLE(XROOT(block)->stylenorm)->background =
+    make_Xresource(xs->display,xs->plane,pixel,XA_CARDINAL);
   return result;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -296,7 +313,7 @@ DEFUN ("epoch::cursor-color",
   xs = XXSCREEN(XROOT(block)->x11);
 
   if (EQ(Qnil,color))
-    return make_Xresource(xs->display,xs->plane,xs->foreground,XA_CARDINAL);
+    return XSTYLE(XROOT(block)->stylecursor)->cursor_background;
     
   color = Fepoch_get_color(color);
   if (EQ(Qnil,color)) return Qnil;
@@ -304,7 +321,8 @@ DEFUN ("epoch::cursor-color",
 
   XSetBackground(xs->display,xs->gc_curs,pixel);
   result = Qt;
-  xs->cursor_color = pixel;
+  XSTYLE(XROOT(block)->stylecursor)->cursor_background = make_Xresource
+    (xs->display,xs->plane,pixel,XA_CARDINAL);
 
   return result;
 }
@@ -406,72 +424,101 @@ DEFUN ("epoch::icon-name",Fx_icon_name,Sx_icon_name,0,2,0,
     return result;
     }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-/* This code provided by my lovely wife Susan. Blame her when it crashes */
+DEFUN ("epoch::ungrab-pointer", Fepoch_ungrab_pointer, Sepoch_ungrab_pointer,
+       0,0,0,
+       "Ungrab the pointer. Useful only after a mouse down event, so that another X client can act before the mouse up event.") ()
+{
+  XUngrabPointer(XD_display,CurrentTime);
+  return Qnil;
+}
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 char *
 x_make_command_line(argc, argv)
-int argc;
-char *argv[];
+     int argc;
+     char *argv[];
+{
+  char * cmd_line;
+  int  cmd_length = 0;
+  int  i;
+  int  offset  = 0;
+
+  /* Initialize these to zero, and see of cmd line args set them */
+  XD_resource_class = 0;
+  XD_resource_name = 0;
+  XD_use_defaults = 0;
+
+  for (i = 0; i < argc; i++)
+    cmd_length += strlen(argv[i]) + 1; /* space for word and space */
+
+  cmd_line = (char *) xmalloc(cmd_length + 1);
+
+  for (i = 0; i < argc; i++)
     {
-    char * cmd_line;
-    int  cmd_length = 0;
-    int  i;
-    int  offset  = 0;
+      sprintf(cmd_line+offset, "%s ", argv[i]);
+      offset += strlen(argv[i]) + 1;
 
-    for (i = 0; i < argc; i++)
-        cmd_length += strlen(argv[i]) + 1; /* space for word and space */
+      if ((!strcmp(argv[i],"-rn") || !strcmp(argv[i],"-resource") ||
+	   !strcmp(argv[i],"-name")) &&
+	  i < argc - 1)
+	{
+	  XD_resource_name = argv[i+1];
+	}
 
-    cmd_line = (char *) xmalloc(cmd_length + 1);
+      if ((!strcmp(argv[i],"-cn") || !strcmp(argv[i],"-class")) &&
+	  i < argc - 1)
+	{
+	  XD_resource_class = argv[i+1];
+	}
 
-    for (i = 0; i < argc; i++)
-        {
-        sprintf(cmd_line+offset, "%s ", argv[i]);
-        offset += strlen(argv[i]) + 1;
-        }
-    cmd_line[cmd_length-1] = '\0';
-    return cmd_line;
+      if ((!strcmp(argv[i],"-ud")))
+	{
+	  XD_use_defaults = 1;
+	}
     }
+  cmd_line[cmd_length-1] = '\0';
+  return cmd_line;
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::screen-information",Fx_screen_information,Sx_screen_information,
       0,1,0,
       "Returns a list consisting of (x y width height border-width map-state)")
         (screen) Lisp_Object screen;
-    {
-    Lisp_Object block;
-    struct Lisp_Xresource *xr;
-    struct Root_Block *rb;
-    XWindowAttributes data;
-    int real_x,real_y;
-    Window child;
-    BLOCK_INPUT_DECLARE();
+{
+  Lisp_Object block;
+  struct Lisp_Xresource *xr;
+  struct Root_Block *rb;
+  XWindowAttributes data;
+  int real_x,real_y;
+  Window child;
+  BLOCK_INPUT_DECLARE();
 
-    if (! (xr = ResourceOrScreen(screen,&rb))) return Qnil;
+  if (! (xr = ResourceOrScreen(screen,&rb))) return Qnil;
 
-    BLOCK_INPUT();
-    XGetWindowAttributes(xr->dpy,xr->id,&data);
-    /* x,y returned are relative to parent, but we want root co-ords */
-    XTranslateCoordinates(xr->dpy,xr->id,
-                          RootWindow(xr->dpy,xr->plane),
-                          0,0,&real_x,&real_y,&child);
-    UNBLOCK_INPUT();
+  BLOCK_INPUT();
+  XGetWindowAttributes(xr->dpy,xr->id,&data);
+  /* x,y returned are relative to parent, but we want root co-ords */
+  XTranslateCoordinates(xr->dpy,xr->id,
+                        RootWindow(xr->dpy,xr->plane),
+                        0,0,&real_x,&real_y,&child);
+  UNBLOCK_INPUT();
 
-    /* now we have to create the list of the items */
-    return Fcons(make_number(real_x),
-                 Fcons(make_number(real_y),
-                       Fcons(make_number(data.width),
-                             Fcons(make_number(data.height),
-                                   Fcons(make_number(data.border_width),
-                                         Fcons(make_number(rb ? XXSCREEN(rb->x11)->in_border : 0),
-                                               Fcons(data.map_state == IsUnmapped ? Qnil : Qt,
-                                                     Qnil)))))));
-
-    }
+  /* now we have to create the list of the items */
+  return Fcons(make_number(real_x),
+               Fcons(make_number(real_y),
+                     Fcons(make_number(data.width),
+                           Fcons(make_number(data.height),
+                                 Fcons(make_number(data.border_width),
+                                       Fcons(make_number(rb ? XWSCREEN(rb->win)->in_border : 0),
+                                             Fcons(data.map_state == IsUnmapped ? Qnil : Qt,
+                                                   Qnil)))))));
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::plane-size",Fx_plane_size,Sx_plane_size,0,0,0,
       "Return the size of the display plane in pixels") ()
-    {
-    return Fcons(make_number(DisplayWidth(XD_display,XD_plane)),
-                 make_number(DisplayHeight(XD_display,XD_plane)));
-    }
+{
+  return Fcons(make_number(DisplayWidth(XD_display,XD_plane)),
+               make_number(DisplayHeight(XD_display,XD_plane)));
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::font",Fx_font,Sx_font,0,2,0,
 "Takes optional arguments of FONT-NAME and SCREEN. If FONT-NAME is nil,\
@@ -484,53 +531,124 @@ DEFUN ("epoch::font",Fx_font,Sx_font,0,2,0,
   struct X_Screen *xs;
   struct W_Screen *ws;
   XSizeHints size_hints;
+#ifdef PBaseSize
+  long size_hints_flags;
+#endif
   int minx,miny,z,i;
   BLOCK_INPUT_DECLARE();
+  struct X_font *font;
+  int dx, dy;
 
   block = find_block(seq);
   if (EQ(block,Qnil)) return Qnil;
   xs = XXSCREEN(XROOT(block)->x11);
   ws = XWSCREEN(XROOT(block)->win);
 
-  if (!NULL(font_name))
+
+  RW_FIXUP(XROOT(block));
+
+  if (NIL(font_name))
+    {
+      /* Return base font for current screen */
+      font = XXFONT(XSTYLE(XROOT(block)->stylenorm)->font);
+    }
+  else
     {
       /* stash the current hints for later re-use */
       BLOCK_INPUT();
+#ifdef PBaseSize
+      z = XGetWMNormalHints(xs->display,xs->xid,&size_hints,&size_hints_flags);
+#else      
       z = XGetNormalHints(xs->display,xs->xid,&size_hints);
+#endif      
       UNBLOCK_INPUT();
       if (!z) error("No normal size hints! Can't change font.");
 
-      minx = (size_hints.min_width - xs->in_border) / xs->font_width;
-      miny = (size_hints.min_height - xs->in_border) / xs->font_height;
-
-      if (!SetScreenFont(XROOT(block),XSTRING(font_name)->data))
+      if (!(font = load_font(XSTRING(font_name)->data)))      
 	error("Bad font name");
+      
+      minx = (size_hints.min_width - ws->in_border) / XFWIDTH(font);
+      miny = (size_hints.min_height - ws->in_border) / XFHEIGHT(font);
 
-      xs->pixwidth = ws->width*xs->font_width + 2*xs->in_border;
-      xs->pixheight = ws->height*xs->font_height + 2*xs->in_border;
+      /* Set screen's styles to use this font. */
+      XSET(XSTYLE(XROOT(block)->stylenorm)->font,Lisp_Raw_Data,font);
+#if 0      
+      XSET(XSTYLE(XROOT(block)->stylerev)->font,Lisp_Raw_Data,font);
+      XSET(XSTYLE(XROOT(block)->stylecursor)->font,Lisp_Raw_Data,font);
+#endif
+      /* xs->pixwidth */
+      dx = ws->width* XFWIDTH(font) + 2*ws->in_border;
+      /* xs->pixheight */
+      dy = ws->height* XFHEIGHT(font) + 2*ws->in_border;
 
       /* set new size hints */
-      size_hints.width = xs->pixwidth;
-      size_hints.height = xs->pixheight;
-      size_hints.width_inc = xs->font_width;
-      size_hints.height_inc = xs->font_height;
-      size_hints.min_width = minx*xs->font_width + 2*xs->in_border;
-      size_hints.min_height = miny*xs->font_height + 2*xs->in_border;
+      size_hints.width = dx; /* xs->pixwidth; */
+      size_hints.height = dy; /* xs->pixheight; */
+      if (XFVARIABLE(font))
+	{
+	  size_hints.width_inc = 1;
+	  size_hints.height_inc = 1;
+	}
+      else
+	{
+	  size_hints.width_inc = XFWIDTH(font);
+	  size_hints.height_inc = XFHEIGHT(font);
+	}
+      size_hints.min_width = minx * XFWIDTH(font) + 2*ws->in_border;
+      size_hints.min_height = miny * XFHEIGHT(font) + 2*ws->in_border;
+
+      ws->size_change = 1;
+
+      update_all_attributes (XROOT(block)->ewin);
+      if (!distinct_minibuffer)
+	update_all_attributes (XROOT(block)->minibuf_window);
+
       BLOCK_INPUT();
+#ifdef PBaseSize
+      XSetWMNormalHints(xs->display,xs->xid,&size_hints);
+#else      
       XSetNormalHints(xs->display,xs->xid,&size_hints);
+#endif      
 
       /* Update all of the GC's hanging about on the screen */
-      XSetFont(xs->display,xs->gc_norm,xs->font_id);
-      XSetFont(xs->display,xs->gc_rev,xs->font_id);
-      XSetFont(xs->display,xs->gc_curs,xs->font_id);
+      XSetFont(xs->display,xs->gc_norm,XFID(font));
+      XSetFont(xs->display,xs->gc_rev,XFID(font));
+      XSetFont(xs->display,xs->gc_curs,XFID(font));
 
-      XResizeWindow(xs->display,xs->xid,xs->pixwidth,xs->pixheight);
-      UNBLOCK_INPUT();
+      /*
+       * Ugly:  if resize request will be blown off due to no size change,
+       * or the screen is not currently mapped, resize it and send off the
+       * ConfigureNotify and Expose events anyway, which will kick off all
+       * other required processing.
+       */
+      if (((dx == xs->pixwidth) && (dy == xs->pixheight)) || !xs->mapped)
+	{
+	  XEvent event;
+
+          if (!xs->mapped)
+	    XResizeWindow(xs->display,xs->xid,dx,dy);	  
+
+	  event.type = ConfigureNotify;
+	  event.xconfigure.window = xs->xid;
+	  event.xconfigure.width = dx;
+	  event.xconfigure.height = dy;
+
+	  XSendEvent(xs->display,xs->xid,False,StructureNotifyMask,&event);
+
+	  event.type = Expose;
+	  event.xexpose.window = xs->xid;
+	  event.xexpose.count = 0;
+
+	  XSendEvent(xs->display,xs->xid,False,ExposureMask,&event);
+	}
+      else
+	XResizeWindow(xs->display,xs->xid,dx,dy);
+      UNBLOCK_INPUT();      
     }
 
-  return Fcons(build_string(xs->font_name),
-	       Fcons(make_number(xs->font_width),
-		     Fcons(make_number(xs->font_height),
+  return Fcons(build_string(XFNAME(font)),
+	       Fcons(make_number(XFWIDTH(font)),
+		     Fcons(make_number(XFHEIGHT(font)),
 			   Qnil)));
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -538,18 +656,16 @@ DEFUN ("epoch::get-font", Fepoch_get_font, Sepoch_get_font, 1, 1, 0,
        "Get the FONT by name")
      (name) Lisp_Object name;
 {
-  Font font;
+  struct X_font *f;
   BLOCK_INPUT_DECLARE();
 
   if (XRESOURCEP(name) && XXRESOURCE(name)->type == XA_FONT) return name;
 
   CHECK_STRING(name,0);
 
-  BLOCK_INPUT();
-  font = XLoadFont(XD_display,XSTRING(name)->data);
-  UNBLOCK_INPUT();
-
-  return font ? make_Xresource(XD_display,XD_plane,font,XA_FONT) : Qnil;
+  f = load_font(XSTRING(name)->data);
+  
+  return f ? make_Xresource(XD_display,XD_plane,XFID(f),XA_FONT) : Qnil;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::make-bitmap", Fepoch_make_bitmap, Sepoch_make_bitmap,
@@ -585,10 +701,12 @@ DEFUN ("epoch::free-bitmap", Fepoch_free_bitmap, Sepoch_free_bitmap, 1, 1, 0,
   return Qnil;
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-raw_query_pointer(dpy,win,xs)
+raw_query_pointer(dpy,win,plane,xs,rb)
      Display *dpy;
      Window win;
+     int plane;
      struct X_Screen *xs;		/* if not-0, use to convert to chars */
+     struct Root_Block *rb;
 {
   Lisp_Object result = Qnil;
   Bool z;
@@ -604,52 +722,170 @@ raw_query_pointer(dpy,win,xs)
   if (z == True)
     {
       if (xs)
-	{  
-	  x = ( x - xs->in_border ) / xs->font_width;
-	  y = ( y - xs->in_border ) / xs->font_height;
+	{
+	  Lisp_Object window,screen;
+	  struct window *w;
+	  struct buffer *b;
+	  register int cx = 0, cy = 0;
+	  register struct line_header *l;
+	  register struct char_block *cb;
+	  screen = x_find_screen_seq(rb->seq);
+	  window = Fepoch_first_window(screen);
+
+	  while (!NIL(window))
+	    {
+	      w = XWINDOW(window);
+	      if (NIL(w->buffer)) continue; /* no buffer, not valid */
+	      if (x >= w->pixleft && x < (w->pixleft + w->pixwidth)
+		  && y >= w->pixtop && y < (w->pixtop + w->pixheight))
+		break;
+	      /* Looks stupid, but the problem is that we have to allow the
+	       * minibuf as a valid window the _first_ time through here,
+	       * but not else when, and we can't go on if the first window
+	       * was the minibuffer
+	       */
+	      if (!EQ(window,rb->minibuf_window))
+		window = Fnext_window(window,Qt);
+	      if (EQ(window,rb->minibuf_window)) win = Qnil;
+	    }
+
+	  /* WRONG!!!!!!!!!!!!!! */
+	  if (NIL(window)) return Qnil;
+
+	  /* found the window */
+
+	  b = XBUFFER(w->buffer);	  
+	  
+	  /* Here the fun starts.  We must find character positions */
+	  if (!NIL(w->modeline))
+	    {
+	      l = XLINE(w->modeline);
+	      if (l && l->ypos - l->ascent <= y && y < l->ypos + l->descent)
+		{
+		  cx = window_char_height(w);
+		  cy = 0;
+		}
+	    }
+	  l = XLINE(w->lines);
+	  while (l)
+	    {
+	      if (l->ypos - l->ascent <= y && y < l->ypos + l->descent)
+		break;
+	      l = l->next; cy++;
+	    }
+	  if (l)
+	    {
+	      cb = l->body;
+	      while (cb != l->end)
+		{
+		  if (cb->xpos <= x && x < cb->xpos + cb->width)
+		    break;
+		  cb = cb->next; cx++;
+		}
+	    }
+	  x = cx; y = cy;
 	}
       result = Fcons(make_number(x),
 		     Fcons(make_number(y),
-			   Fcons(make_number(bstate),Qnil)));
+			   Fcons(make_number(bstate),
+				 Fcons((child_win == None) ? Qnil :
+				       make_Xresource(dpy,
+						      plane,
+						      child_win,
+						      XA_WINDOW),Qnil))));
     }
 
   return result;
 }
 /* - - */
 DEFUN ("epoch::query-pointer",Fepoch_query_pointer,Sepoch_query_pointer,0,1,0,
-"Query the location of the X-window cursor.\
- Returns the list (X Y MOD-STATE). Raw X-coordinates.")
-	(scr) Lisp_Object scr;
+"Query the location of the X-window cursor.\n\
+ Returns the list (X Y MOD-STATE X-WINDOW).\n\
+ X,Y X-window pixel coordinates\n\
+ MOD-STATE bitmask of keyboard modifier state\n\
+ X-WINDOW X-window resource of the window containing the pointer")
+	(screen) Lisp_Object screen;
 {
-  struct Lisp_Xresource *xr;
+  struct Root_Block *rb;
+  struct X_Screen *xs;
+  Display *dpy;
+  Window win;
+  int plane;
+  
+  if (XRESOURCEP(screen))
+    {
+      if (XXRESOURCE(screen)->type != XA_WINDOW)
+	error("Screen resource must be of type WINDOW");
+      dpy = XXRESOURCE(screen)->dpy;
+      win = XXRESOURCE(screen)->id;
+      rb = x_find_screen(win);      
+      plane = XXRESOURCE(screen)->plane;
+    }
+  else
+    {
+      screen = find_block(screen);
+      rb = XROOT(screen);
+      xs = XXSCREEN(rb->x11);
+      dpy = xs->display;
+      win = xs->xid;
+      plane = xs->plane;
+    }
 
-  xr = ResourceOrScreen(scr,0);
-  return xr ? raw_query_pointer(xr->dpy,xr->id,0) : Qnil;
+  return raw_query_pointer(dpy,win,plane,0,rb);
 }
 /* - - */
 DEFUN ("epoch::query-mouse",Fepoch_query_mouse,Sepoch_query_mouse,0,1,0,
 "Query the location of the X-window cursor.\
- Returns the list (X Y MOD-STATE). Character coordinates")
+ Returns the list (X Y MOD-STATE X-WINDOW).\n\
+ X,Y Epoch screen character coordinates\n\
+ MOD-STATE bitmask of keyboard modifier state\n\
+ X-WINDOW X-window resource of the window containing the pointer")
 	(screen) Lisp_Object screen;
 {
   struct X_Screen *xs;
+  Display *dpy;
+  Window win;
+  int plane;
+  struct Root_Block *rb;
   
-  screen = find_block(screen);
-  if (NULL(screen)) return Qnil;
-  xs = XXSCREEN(XROOT(screen)->x11);
-  return raw_query_pointer(xs->display,xs->xid,xs);
+  if (XRESOURCEP(screen))
+    {
+      if (XXRESOURCE(screen)->type != XA_WINDOW)
+	error("Screen resource must be of type WINDOW");
+      dpy = XXRESOURCE(screen)->dpy;
+      win = XXRESOURCE(screen)->id;
+      rb = x_find_screen(win);      
+      plane = XXRESOURCE(screen)->plane;
+    }
+  else
+    {
+      screen = find_block(screen);
+      rb = XROOT(screen);
+      xs = XXSCREEN(rb->x11);
+      dpy = xs->display;
+      win = xs->xid;
+      plane = xs->plane;
+    }
+
+  return raw_query_pointer(dpy,win,plane,xs,rb);
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-raw_warp_pointer(dpy,win,x,y,xs)
+raw_warp_pointer(dpy,win,x,y,xs,rb)
      Display *dpy;
      Window win;
      int x,y;
      struct X_Screen *xs;		/* if not-0, use to convert to chars */
+     struct Root_Block *rb;
 {
+  struct X_font *font = XXFONT(XSTYLE(rb->stylenorm)->font);
+  
   if (xs)
     {
-      x = xs->in_border + XINT(x)*xs->font_width  + xs->font_width/2;
-      y = xs->in_border + XINT(y)*xs->font_height + xs->font_height/2;
+      struct W_Screen *ws = XWSCREEN(rb->win);
+      x = ws->in_border + XINT(x)*XFWIDTH(font)
+	+ XFWIDTH(font)/2;
+      y = ws->in_border + XINT(y)*XFHEIGHT(font)
+	+ XFHEIGHT(font)/2;
     }
   XWarpPointer(dpy,None,win,0,0,0,0,x,y);
   return Qt;
@@ -659,27 +895,32 @@ DEFUN ("epoch::warp-pointer",Fepoch_warp_pointer,Sepoch_warp_pointer,2,3,0,
 "Warp the X pointer to X, Y (in pixels) relative to SCREEN.")
         (x,y,screen) Lisp_Object x,y,screen;
 {
-  struct Lisp_Xresource *xr;
+  struct Root_Block *rb;
+  struct X_Screen *xs;
 
   CHECK_NUMBER(x,0); CHECK_NUMBER(y,0);
 
-  xr = ResourceOrScreen(screen,0);
-
-  return xr ? raw_warp_pointer(xr->dpy,xr->id,XINT(x),XINT(y),0) : Qnil;
+  screen = find_block(screen);
+  if (NIL(screen)) return Qnil;
+  rb = XROOT(screen);
+  xs = XXSCREEN(rb->x11);
+  return raw_warp_pointer(xs->display,xs->xid,XINT(x),XINT(y),0,rb);
 }
 /* - - */
 DEFUN("epoch::warp-mouse",Fepoch_warp_mouse,Sepoch_warp_mouse,2,3,0,
 "Warp the X pointer to X,Y (in characters) relative to SCREEN.")
 	(x,y,screen) Lisp_Object x,y,screen;
 {
+  struct Root_Block *rb;
   struct X_Screen *xs;
 
   CHECK_NUMBER(x,0);  CHECK_NUMBER(y,0);
 
   screen = find_block(screen);
-  if (NULL(screen)) return Qnil;
-  xs = XXSCREEN(XROOT(screen)->x11);
-  return raw_warp_pointer(xs->display,xs->xid,XINT(x),XINT(y),xs);
+  if (NIL(screen)) return Qnil;
+  rb = XROOT(screen);
+  xs = XXSCREEN(rb->x11);
+  return raw_warp_pointer(xs->display,xs->xid,XINT(x),XINT(y),xs,rb);
 }
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::query-cursor",Fepoch_query_cursor,Sepoch_query_cursor,0,1,0,
@@ -691,11 +932,43 @@ point for the selected window on the SCREEN, if it has been updated.")
   struct W_Screen *ws;
 
   screen = find_block(screen);
-  if (NULL(screen)) return Qnil;
+  if (NIL(screen)) return Qnil;
   ws = XWSCREEN(XROOT(screen)->win);
-  return Fcons(make_number(ws->cursor_x),make_number(ws->cursor_y));
+  return Fcons(make_number(ws->vis_xpos),make_number(ws->vis_ypos));
 }
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+DEFUN ("epoch::query-cursor-pixels",Fquery_cursor_pixels,Squery_cursor_pixels,0,1,0,
+       "Return the X, Y pixel location of the Epoch cursor in a cons pair.\n\
+The argument is an optional SCREEN.  This location generally corresponds to\n\
+point for the selected window on the SCREEN, if it has been updated.")
+	(screen) Lisp_Object screen;
+{
+  struct W_Screen *ws;
 
+  screen = find_block(screen);
+  if (NIL(screen)) return Qnil;
+  ws = XWSCREEN(XROOT(screen)->win);
+  return Fcons(make_number(ws->cur_char->xpos - XFASTINT(ws->cur_w->pixleft)),
+	       make_number(ws->cur_line->ypos -
+			   ws->cur_line->ascent -
+			   XFASTINT(ws->cur_w->pixtop)));
+}
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+DEFUN ("epoch::get-default",Fepoch_get_default,Sepoch_get_default,1,2,0,
+       "Get resource definition associated with NAME.  If specified, also use CLASS for lookup.")
+     (name,class) Lisp_Object name,class;
+{
+  char *tmp,bob = 0;
+  
+  CHECK_STRING(name,0);
+
+  tmp = (char *) epoch_get_default(XSTRING(name)->data,
+				   (NIL(class) ?
+				    (char *) 0 :
+				    (char *) XSTRING(class)->data));
+
+  return (tmp ? build_string(tmp) : Qnil);
+}
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 DEFUN ("epoch::send-client-message",
        Fx_send_client_message, Sx_send_client_message, 1,5,0,
@@ -725,12 +998,12 @@ DEFUN ("epoch::send-client-message",
   else
     {
       source = find_block(source);
-      if (NULL(source)) error("Source must be screen or X-window resource");
+      if (NIL(source)) error("Source must be screen or X-window resource");
       ev.xclient.window = XXSCREEN(XROOT(source)->x11)->xid;
     }
 
   /* check format before data, because it can cause the data format to vary */
-  if (!NULL(format))
+  if (!NIL(format))
     {
       CHECK_NUMBER(format,2);
       actual_format = XFASTINT(format);
@@ -742,7 +1015,7 @@ DEFUN ("epoch::send-client-message",
   bzero((char *) &ev.xclient.data,20);
 
   /* look for the data */
-  if (!NULL(data))
+  if (!NIL(data))
     {
       ConvertEtoX(data,&addr,&count,&actual_type,&actual_format,&free_storage);
       if ((count * actual_format) > 20*8)
@@ -754,7 +1027,7 @@ DEFUN ("epoch::send-client-message",
       if (free_storage) free(addr);
     }
 
-  if (!NULL(type))
+  if (!NIL(type))
     {
       CHECK_XRESOURCE(type,2);
       if (XXRESOURCE(type)->type != XA_ATOM)
@@ -801,7 +1074,7 @@ DEFUN ("epoch::query-tree",Fepoch_query_tree,Sepoch_query_tree,0,1,0,
   else
     {
       block = find_block(screen);
-      if (NULL(block)) return Qnil;
+      if (NIL(block)) return Qnil;
       dpy = XXSCREEN(XROOT(block)->x11)->display;
       plane = XXSCREEN(XROOT(block)->x11)->plane;
       win = XXSCREEN(XROOT(block)->x11)->xid;
@@ -828,23 +1101,6 @@ DEFUN ("epoch::query-tree",Fepoch_query_tree,Sepoch_query_tree,0,1,0,
 		     make_Xresource(dpy,plane,parent,XA_WINDOW) : Qnil,
 		     value));
 }
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-#ifdef DENYS_ALARM
-DEFUN ("epoch::set-timer",Fset_timer,Sset_timer,1,1,0,
-      "sets up a timer to wake up in ARG milliseconds. When that happens\n\
-an event of type timer is entered in the queue.")
-     (delay)
-     Lisp_Object delay;
-{
-  extern void set_alarm();
-  CHECK_NUMBER (delay,0);
-  if (XINT (delay) < 0)
-    Fsignal(Qerror,Fcons(build_string("epoch::set-timer expects an argument >= 0"),Fcons(delay,Qnil)));
-  set_alarm ((long) XFASTINT(delay),1);
-  return Qt;
-}
-#endif /* DENYS_ALARM */
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 #ifdef HAVE_TIMEVAL
 #include <sys/time.h>
@@ -862,7 +1118,9 @@ DEFUN ("the-time",Fthe_time,Sthe_time,0,0,0,
 #endif /* HAVE_TIMEVAL */
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 void syms_of_xutil()
+
 {
+  defsubr(&Sepoch_get_default);
   defsubr(&Sepoch_color_components);
   defsubr(&Sx_foreground);
   defsubr(&Sx_background);
@@ -870,6 +1128,7 @@ void syms_of_xutil()
   defsubr(&Sx_cursor_glyph);
   defsubr(&Sx_title);
   defsubr(&Sx_icon_name);
+  defsubr(&Sepoch_ungrab_pointer);
   defsubr(&Sx_font);
   defsubr(&Sepoch_get_font);
   defsubr(&Sepoch_make_bitmap);
@@ -881,14 +1140,8 @@ void syms_of_xutil()
   defsubr(&Sepoch_warp_pointer);
   defsubr(&Sepoch_warp_mouse);
   defsubr(&Sepoch_query_cursor);
+  defsubr(&Squery_cursor_pixels);
   defsubr(&Sx_send_client_message);
-  /*
-    defsubr(&Sx_cursor_foreground);
-    defsubr(&Sx_cursor_background);
-    */
-#ifdef DENYS_ALARM
-  defsubr(&Sset_timer);
-#endif				/* DENYS_ALARM */
 #ifdef HAVE_TIMEVAL
   defsubr(&Sthe_time);
 #endif
